package com.hartmath.classes;

import com.hartmath.lib.*;
import com.hartmath.util.*;
import com.hartmath.expression.*;
import com.hartmath.mapping.*;
import com.hartmath.JSci.chemistry.*;
import com.hartmath.JSci.chemistry.periodictable.*;

import java.applet.*;
import java.awt.*;

class EPeriodicTable implements FunctionEvaluator {
    private PeriodicTable f;
		public HObject evaluate(HFunction vect) {
      //if ( vect.size() == 0 ) {
        f = new PeriodicTable();
        //f.setLayout(new BorderLayout());
        //f.add(graph,"Center");
        f.show();
      //}
      return C.Null;
    }
}

class HSymbolElement extends HSymbol
{
   public HSymbolElement(Element feval)
   {
      super("$"+feval.toString(), feval);
   }
}

class EAtomicNumber extends E1Arg {

		public HObject e1ObjArg(HObject o) {
	    if (o instanceof HSymbol) 
	    {
	    	FunctionEvaluator fe = ((HSymbol) o).getEval();
	    	if (fe instanceof Element) 
	    	{
	    		return evalElement((Element)fe);
	    	}
	    }

			return null;
		}
		
		public HObject evalElement(Element elem) 
		{
		  return HInteger.valueOf(elem.atomicNumber());
		}

	}

class EMassNumber extends EAtomicNumber {
    public HObject evalElement(Element elem) 
		{
		  return HInteger.valueOf(elem.massNumber());
		}
	}

class EElectroNegativity extends EAtomicNumber {
    public HObject evalElement(Element elem) 
		{
		  return new HDouble(elem.electronegativity());
		}
	}

class ECovalentRadius extends EAtomicNumber {
    public HObject evalElement(Element elem) 
		{
		  return new HDouble(elem.covalentRadius());
		}
	}

class EAtomicRadius extends EAtomicNumber {
    public HObject evalElement(Element elem) 
		{
		  return new HDouble(elem.atomicRadius());
		}
		
	}

class EMeltingPoint extends EAtomicNumber {
    public HObject evalElement(Element elem) 
		{
		  return new HDouble(elem.meltingPoint());
		}
		
	}

class EBoilingPoint extends EAtomicNumber {
    public HObject evalElement(Element elem)
		{
		  return new HDouble(elem.boilingPoint());
		}

	}

class EDensity extends EAtomicNumber {
    public HObject evalElement(Element elem)
		{
		  return new HDouble(elem.density());
		}

	}
  
class ESpecificHeat extends EAtomicNumber {
    public HObject evalElement(Element elem)
		{
		  return new HDouble(elem.specificHeat());
		}

	}

class EElectricalConductivity extends EAtomicNumber {
    public HObject evalElement(Element elem)
		{
		  return new HDouble(elem.electricalConductivity());
		}

	}

class EThermalConductivity extends EAtomicNumber {
    public HObject evalElement(Element elem)
		{
		  return new HDouble(elem.thermalConductivity());
		}

	}

class EIsHalogen extends EB1ObjectArg {

    public boolean evalArg1(HObject o)
    {
	    if (o instanceof HSymbol) 
	    {
	    	FunctionEvaluator fe = ((HSymbol) o).getEval();
	    	if (fe instanceof Element) 
	    	{
	    		return evalElement((Element)fe);
	    	}
	    }

			return false;
		}
		
		public boolean evalElement(Element elem)
		{
		  return (elem instanceof Halogen);
		}

	}

class EIsMetal extends EIsHalogen
{
   public boolean evalElement(Element elem)
	 {
	   return (elem instanceof Metal);
	 }
}

class EIsAlkaliMetal extends EIsHalogen
{
   public boolean evalElement(Element elem)
	 {
	   return (elem instanceof AlkaliMetal);
	 }
}

class EIsAlkaliEarthMetal extends EIsHalogen
{
   public boolean evalElement(Element elem)
	 {
	   return (elem instanceof AlkaliEarthMetal);
	 }
}

class EIsNonMetal extends EIsHalogen
{
   public boolean evalElement(Element elem)
	 {
	   return (elem instanceof NonMetal);
	 }
}

class EIsRareEarthMetal extends EIsHalogen
{
   public boolean evalElement(Element elem)
	 {
	   return (elem instanceof RareEarthMetal);
	 }
}

class EIsTransitionMetal extends EIsHalogen
{
   public boolean evalElement(Element elem)
	 {
	   return (elem instanceof TransitionMetal);
	 }
}

class EIsNobleGas extends EIsHalogen
{
   public boolean evalElement(Element elem)
	 {
	   return (elem instanceof NobleGas);
	 }
}

public class Chemistry implements FunctionEvaluator {

  final public HSymbol PeriodicTable = new HSymbol("PeriodicTable",
			  new EPeriodicTable());

  final public HSymbol AtomicNumber = new HSymbol("AtomicNumber",
			  new EAtomicNumber(), HSymbol.LISTABLE);

  final public HSymbol MassNumber = new HSymbol("MassNumber", 
			  new EMassNumber(), HSymbol.LISTABLE);

  final public HSymbol ElectroNegativity = new HSymbol("ElectroNegativity", 
			  new EElectroNegativity(), HSymbol.LISTABLE);
  
  final public HSymbol CovalentRadius = new HSymbol("CovalentRadius", 
			  new ECovalentRadius(), HSymbol.LISTABLE);

  final public HSymbol AtomicRadius = new HSymbol("AtomicRadius", 
			  new EAtomicRadius(), HSymbol.LISTABLE);

  final public HSymbol MeltingPoint = new HSymbol("MeltingPoint",
			  new EMeltingPoint(), HSymbol.LISTABLE);

  final public HSymbol BoilingPoint = new HSymbol("BoilingPoint",
			  new EBoilingPoint(), HSymbol.LISTABLE);

  final public HSymbol Density = new HSymbol("Density",
			  new EDensity(), HSymbol.LISTABLE);

  final public HSymbol SpecificHeat = new HSymbol("SpecificHeat",
			  new ESpecificHeat(), HSymbol.LISTABLE);

  final public HSymbol ElectricalConductivity = new HSymbol("ElectricalConductivity",
			  new EElectricalConductivity(), HSymbol.LISTABLE);

  final public HSymbol ThermalConductivity = new HSymbol("ThermalConductivity",
			  new EThermalConductivity(), HSymbol.LISTABLE);

  final public HSymbol IsAlkaliMetal = new HSymbol("IsAlkaliMetal",
			  new EIsAlkaliMetal(), HSymbol.LISTABLE);
  final public HSymbol IsAlkaliEarthMetal = new HSymbol("IsAlkaliEarthMetal",
			  new EIsAlkaliEarthMetal(), HSymbol.LISTABLE);
  final public HSymbol IsHalogen = new HSymbol("IsHalogen",
			  new EIsHalogen(), HSymbol.LISTABLE);
  final public HSymbol IsMetal = new HSymbol("IsMetal",
			  new EIsMetal(), HSymbol.LISTABLE);
  final public HSymbol IsNobleGas = new HSymbol("IsNobleGas",
			  new EIsNobleGas(), HSymbol.LISTABLE);
  final public HSymbol IsNonMetal = new HSymbol("IsNonMetal",
			  new EIsNonMetal(), HSymbol.LISTABLE);
  final public HSymbol IsRareEarthMetal = new HSymbol("IsRareEarthMetal",
			  new EIsRareEarthMetal(), HSymbol.LISTABLE);
  final public HSymbol IsTransitionMetal = new HSymbol("IsTransitionMetal",
			  new EIsTransitionMetal(), HSymbol.LISTABLE);

  HSymbol ElementSymbolTable[] =
  {
    new HSymbolElement(Element.Actinium),
    new HSymbolElement(Element.Aluminium),
    new HSymbolElement(Element.Americium),
    new HSymbolElement(Element.Antimony),
    new HSymbolElement(Element.Argon),
    new HSymbolElement(Element.Arsenic),
    new HSymbolElement(Element.Astatine),
    new HSymbolElement(Element.Barium),
    new HSymbolElement(Element.Berkelium),
    new HSymbolElement(Element.Beryllium),
    new HSymbolElement(Element.Bismuth),
    new HSymbolElement(Element.Boron),
    new HSymbolElement(Element.Bromine),
    new HSymbolElement(Element.Cadmium),
    new HSymbolElement(Element.Caesium),
    new HSymbolElement(Element.Calcium),
    new HSymbolElement(Element.Californium),
    new HSymbolElement(Element.Carbon),
    new HSymbolElement(Element.Cerium),
    new HSymbolElement(Element.Chlorine),
    new HSymbolElement(Element.Chromium),
    new HSymbolElement(Element.Cobalt),
    new HSymbolElement(Element.Copper),
    new HSymbolElement(Element.Curium),
    new HSymbolElement(Element.Dysprosium),
    new HSymbolElement(Element.Einsteinium),
    new HSymbolElement(Element.Erbium),
    new HSymbolElement(Element.Europium),
    new HSymbolElement(Element.Fermium),
    new HSymbolElement(Element.Fluorine),
    new HSymbolElement(Element.Francium),
    new HSymbolElement(Element.Gadolinium),
    new HSymbolElement(Element.Gallium),
    new HSymbolElement(Element.Germanium),
    new HSymbolElement(Element.Gold),
    new HSymbolElement(Element.Hafnium),
    new HSymbolElement(Element.Helium),
    new HSymbolElement(Element.Holmium),
    new HSymbolElement(Element.Hydrogen),
    new HSymbolElement(Element.Indium),
    new HSymbolElement(Element.Iodine),
    new HSymbolElement(Element.Iridium),
    new HSymbolElement(Element.Iron),
    new HSymbolElement(Element.Krypton),
    new HSymbolElement(Element.Lanthanum),
    new HSymbolElement(Element.Lawrencium),
    new HSymbolElement(Element.Lead),
    new HSymbolElement(Element.Lithium),
    new HSymbolElement(Element.Lutetium),
    new HSymbolElement(Element.Magnesium),
    new HSymbolElement(Element.Manganese),
    new HSymbolElement(Element.Mendelevium),
    new HSymbolElement(Element.Mercury),
    new HSymbolElement(Element.Molybdenum),
    new HSymbolElement(Element.Neodymium),
    new HSymbolElement(Element.Neon),
    new HSymbolElement(Element.Neptunium),
    new HSymbolElement(Element.Nickel),
    new HSymbolElement(Element.Niobium),
    new HSymbolElement(Element.Nitrogen),
    new HSymbolElement(Element.Nobelium),
    new HSymbolElement(Element.Osmium),
    new HSymbolElement(Element.Oxygen),
    new HSymbolElement(Element.Palladium),
    new HSymbolElement(Element.Plutonium),
    new HSymbolElement(Element.Polonium),
    new HSymbolElement(Element.Potassium),
    new HSymbolElement(Element.Praseodymium),
    new HSymbolElement(Element.Protactinium),
    new HSymbolElement(Element.Radium),
    new HSymbolElement(Element.Radon),
    new HSymbolElement(Element.Rhenium),
    new HSymbolElement(Element.Rhodium),
    new HSymbolElement(Element.Ruthenium),
    new HSymbolElement(Element.Samarium),
    new HSymbolElement(Element.Scandium),
    new HSymbolElement(Element.Selenium),
    new HSymbolElement(Element.Silicon),
    new HSymbolElement(Element.Silver),
    new HSymbolElement(Element.Sodium),
    new HSymbolElement(Element.Strontium),
    new HSymbolElement(Element.Sulphur),
    new HSymbolElement(Element.Tantalum),
    new HSymbolElement(Element.Technetium),
    new HSymbolElement(Element.Tellurium),
    new HSymbolElement(Element.Terbium),
    new HSymbolElement(Element.Thallium),
    new HSymbolElement(Element.Thorium),
    new HSymbolElement(Element.Thulium),
    new HSymbolElement(Element.Tin),
    new HSymbolElement(Element.Titanium),
    new HSymbolElement(Element.Tungsten),
    new HSymbolElement(Element.Rutherfordium),
    new HSymbolElement(Element.Dubnium),
    new HSymbolElement(Element.Seaborgium),
    new HSymbolElement(Element.Bohrium),
    new HSymbolElement(Element.Hassium),
    new HSymbolElement(Element.Meitnerium),
    new HSymbolElement(Element.Uranium),
    new HSymbolElement(Element.Vanadium),
    new HSymbolElement(Element.Xenon),
    new HSymbolElement(Element.Ytterbium),
    new HSymbolElement(Element.Yttrium),
    new HSymbolElement(Element.Zinc),
    new HSymbolElement(Element.Zirconium)
  };

	/**
	 * Class Constructor.
	 *
	 *
	 * @see
	 */
	public Chemistry() {
    StringBuffer buf = new StringBuffer();
    PeriodicTable.putSymbol();
		AtomicNumber.putSymbol();
		MassNumber.putSymbol();
		ElectroNegativity.putSymbol();
		CovalentRadius.putSymbol();
		AtomicRadius.putSymbol();
    MeltingPoint.putSymbol();
    BoilingPoint.putSymbol();
    Density.putSymbol();
    SpecificHeat.putSymbol();
    ElectricalConductivity.putSymbol();
    ThermalConductivity.putSymbol();
    IsAlkaliMetal.putSymbol();
    IsAlkaliEarthMetal.putSymbol();
    IsHalogen.putSymbol();
    IsMetal.putSymbol();
    IsNonMetal.putSymbol();
    IsRareEarthMetal.putSymbol();
    IsTransitionMetal.putSymbol();
    IsNobleGas.putSymbol();
    buf.append("Loading Functions:\n"
                 + "PeriodicTable()\n"
								 + "AtomicNumber(<element>)\n"
								 + "MassNumber(<element>)\n"
								 + "ElectroNegativity(<element>)\n"
								 + "CovalentRadius(<element>)\n"
								 + "AtomicRadius(<element>)\n"
                 + "MeltingPoint(<element>)\n"
                 + "BoilingPoint(<element>)\n"
                 + "Density(<element>)\n"
                 + "SpecificHeat(<element>)\n"
                 + "ElectricalConductiv(<element>)\n"
                 + "ThermalConductivity(<element>)\n"
                 + "IsAlkaliEarthMetal(<element>)\n"
                 + "IsAlkaliMetal(<element>)\n"
                 + "IsHalogen(<element>)\n"
                 + "IsMetal(<element>)\n"
                 + "IsNonMetal(<element>)\n"
                 + "IsRareEarthMetal(<element>)\n"
                 + "IsTransitionMetal(<element>)\n"
                 + "IsNobleGas(<element>)\n"
								 	);
    buf.append("Loading Periodic Table:\n");
    buf.append("{");
	  for (int i=0; i<ElementSymbolTable.length;i++) {
      ElementSymbolTable[i].putSymbol();
      buf.append(ElementSymbolTable[i].toString());
      if (i!=ElementSymbolTable.length-1) {
        buf.append(", ");
      }
    }
    buf.append("}");
    C.resultOut.append(buf.toString());
}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param vect
	 * 
	 * @return
	 * 
	 * @see
	 */
	public HObject evaluate(HFunction vect) {
		return C.Null;
	}

}