/*--- formatted by Jindent 2.1, (www.c-lab.de/~jindent) ---*/

package com.hartmath.classes;

import com.hartmath.lib.*;
import com.hartmath.util.*;
import com.hartmath.expression.*;
import com.hartmath.mapping.*;
/**
 * Conversion algorithms
 * 
 * @author Klaus Hartlage  <A HREF="mailto:khartlage@t-online.de">khartlage@t-online.de</A>
 */

/**
 * 
 */
class SISymbol extends HSymbol
{
  HObject toSI;
  HObject fromSI;
  HSymbol sym;
  SISymbol(String str, HSymbol s, HObject tSI, HObject fSI)
  {
    super(str, HSymbol.CONSTANT);
    toSI = tSI;
    fromSI = fSI;
    sym = s;
  }

  public HObject convertToSI()
  {
    return C.Multiply.f(toSI, sym);
  }
  public HObject convertFromSI(HObject obj)
  {
    if (sym.equals(obj)) {
      return fromSI;
    }
    return null;
  }
}

class SIToEvaluator implements ObjectEvaluator
{
  public HObject evaluate(HObject obj)
  {
    if (obj instanceof SISymbol) {
      return ((SISymbol)obj).convertToSI();
    }
    if (obj instanceof HFunction) {
      return ((HFunction)obj).substituteArg(new SIToEvaluator());
    }
    return null;
  }
}

class SIFromEvaluator implements ObjectEvaluator
{
  SISymbol sSym;
  SIFromEvaluator(SISymbol s)
  {
    sSym = s;
  }
  public HObject evaluate(HObject obj)
  {
    return sSym.convertFromSI(obj);
  }
}

class ESI implements FunctionEvaluator
{

   /**
    * Method declaration
    *
    *
    * @param f
    *
    * @return
    *
    * @see
    */
   public HObject evaluate(HFunction f)
   {
     if (f.size()==1)
     {
       if (f.get(0) instanceof SISymbol) {
         return ((SISymbol)f.get(0)).convertToSI();
       }
       if (f.get(0) instanceof HFunction) {
         HObject temp = ((HFunction)f.get(0)).substituteArg(new SIToEvaluator());
         if (temp!=null) {
           return temp;
         }
       }
       return f.get(0);
     }
     return null;
   }

}
class EConvert implements FunctionEvaluator
{

   /**
    * Method declaration
    *
    *
    * @param f
    *
    * @return
    *
    * @see
    */
   public HObject evaluate(HFunction f)
   {
     if ((f.size()==2) && (f.get(1) instanceof SISymbol))
     {
       HObject temp=null;
       if (f.get(0) instanceof SISymbol) {
         temp = ((SISymbol)f.get(0)).convertToSI();
       }
       if (f.get(0) instanceof HFunction) {
         temp = ((HFunction)f.get(0)).substituteArg(new SIToEvaluator());
       }
       if (temp==null) {
         temp = f.get(0);
       }
       temp = C.EV(temp);
       HObject ftemp = null;
       if (temp instanceof SISymbol) {
         ftemp = ((SISymbol)temp).convertFromSI(f.get(1));
       }
       if (temp instanceof HFunction) {
         ftemp = ((HFunction)temp).substituteArg(new SIFromEvaluator((SISymbol)f.get(1)));
       }
       if (ftemp!=null) {
         return ftemp;
       }
       return temp;
     }
     return null;
   }

}
/**
 * Class declaration
 *
 *
 * @author
 * @version %I%, %G%
 */
public class Conversion implements FunctionEvaluator
{

   final static public HSymbol SI = new HSymbol("SI",
     new ESI(), HSymbol.LISTABLE);
   final static public HSymbol Convert = new HSymbol("Convert",
     new EConvert(), HSymbol.LISTABLE);

   final static public HSymbol Milli     = new HSymbol("Milli",
     new ConstantEvaluator(new HDouble(1/1000.0),
     	                     HFraction.valueOf(1, 1000)), HSymbol.CONSTANT);
   final static public HSymbol Micro     = new HSymbol("Micro",
     new ConstantEvaluator(new HDouble(1/1000000.0),
     	                     HFraction.valueOf(1, 1000000)), HSymbol.CONSTANT);
   final static public HSymbol Nano     = new HSymbol("Nano",
     new ConstantEvaluator(new HDouble(1/1000000.0*1/1000),
     	                     new HFraction("1", "1000000000")), HSymbol.CONSTANT);
   final static public HSymbol Kilo     = new HSymbol("Kilo",
     new ConstantEvaluator(new HDouble(1000.0),
     	                     HInteger.valueOf(1000)), HSymbol.CONSTANT);
   final static public HSymbol Mega     = new HSymbol("Mega",
     new ConstantEvaluator(new HDouble(1000000.0),
     	                     HInteger.valueOf(1000000)), HSymbol.CONSTANT);
   final static public HSymbol Giga     = new HSymbol("Giga",
     new ConstantEvaluator(new HDouble(1000000.0*1000.0),
     	                     new HInteger("1000000000")), HSymbol.CONSTANT);

   // base units:
   final static public HSymbol Meter    = new HSymbol("Meter",    HSymbol.CONSTANT);
   final static public HSymbol Kilogram = new HSymbol("Kilogram", HSymbol.CONSTANT);
   final static public HSymbol Second   = new HSymbol("Second",   HSymbol.CONSTANT);
   final static public HSymbol Ampere   = new HSymbol("Ampere",   HSymbol.CONSTANT);
   final static public HSymbol Kelvin   = new HSymbol("Kelvin",   HSymbol.CONSTANT);
   final static public HSymbol Mole     = new HSymbol("Mole",     HSymbol.CONSTANT);
   final static public HSymbol Candela  = new HSymbol("Candela",  HSymbol.CONSTANT);

   final static public SISymbol Inch = new SISymbol("Inch", Meter,
     new HDouble(0.0254), new HDouble(1/0.0254));
   final static public SISymbol Foot = new SISymbol("Foot", Meter, new HDouble(0.3048), new HDouble(1/0.3048));
   final static public SISymbol Yard = new SISymbol("Yard", Meter, new HDouble(0.9144), new HDouble(1/0.9144));
   final static public SISymbol StatuteMile = new SISymbol("StatuteMile", Meter,
     HInteger.valueOf(1609), HFraction.valueOf(1, 1609));

   final static public SISymbol Ounce = new SISymbol("Ounce", Kilogram,
     new HDouble(0.02835), new HDouble(1/0.02835));
   final static public SISymbol Pound = new SISymbol("Pound", Kilogram,
     new HDouble(0.4536), new HDouble(1/0.4536));
   final static public SISymbol Ton = new SISymbol("Ton", Kilogram,
     HInteger.valueOf(1016), HFraction.valueOf(1, 1016));

   final static public SISymbol Minute = new SISymbol("Minute", Second,
     HInteger.valueOf(60), HFraction.valueOf(1, 60));
   final static public SISymbol Hour = new SISymbol("Hour", Second,
     HInteger.valueOf(3600), HFraction.valueOf(1, 3600));
   final static public SISymbol Day = new SISymbol("Day", Second,
     HInteger.valueOf(86400), HFraction.valueOf(1, 86400));
   final static public SISymbol Week = new SISymbol("Week", Second,
     HInteger.valueOf(604800), HFraction.valueOf(1, 604800));

  //final static public HSymbol Kelvin = new HSymbol("Kelvin", HSymbol.CONSTANT);
   /**
    * Constructor declaration
    *
    *
    * @see
    */
   public Conversion()
   {
      SI.putSymbol();
      Convert.putSymbol();

      Milli.putSymbol();
      Micro.putSymbol();
      Nano.putSymbol();
      Kilo.putSymbol();
      Mega.putSymbol();
      Giga.putSymbol();

      Meter.putSymbol();
      Kilogram.putSymbol();
      Second.putSymbol();
      Ampere.putSymbol();
      Kelvin.putSymbol();
      Mole.putSymbol();
      Candela.putSymbol();

      Inch.putSymbol();
      Foot.putSymbol();
      Yard.putSymbol();
      StatuteMile.putSymbol();

      Ounce.putSymbol();
      Pound.putSymbol();
      Ton.putSymbol();
      // Time
      Minute.putSymbol();
      Hour.putSymbol();
      Day.putSymbol();
      Week.putSymbol();

      C.resultOut.append("Loading Functions:\n"
                         + "ConvertTemperature(<value>, <from-unit>, <to-unit>)\n"
                         + "SI(<unit-value>)\n"
                         + "Convert(<from-unit>, <to-unit>)\n"
                         + "Constants: Nano, Micro, Milli, Kilo, Mega, Giga\n"
                         + "  Meter, Inch, Foot, Yard, StatuteMile\n"
                         + "  Ounce, Pound, Ton\n"
                         + "  Minute, Hour, Day, Week\n"
                         );
   }

   /**
    * Method declaration
    *
    *
    * @param vect
    *
    * @return
    *
    * @see
    */
   public HObject evaluate(HFunction vect)
   {
      return C.Null;
   }


}




