/*--- formatted by Jindent 2.0b2, (www.c-lab.de/~jindent) ---*/

package com.hartmath.mapping;

import com.hartmath.expression.*;
import com.hartmath.lib.*;

/**
 * Evaluate functions with 2 arguments
 * 
 * @see FunctionEvaluator
 * @see HSymbol
 * @author Klaus Hartlage  <A HREF="mailto:khartlage@t-online.de">khartlage@t-online.de</A>
 */
public class E2Arg implements FunctionEvaluator {

	/**
	 * @param vect   vector of the function, which should be evaluated
	 * @return       <code>null</code> or the evaluated object
	 */
	public HObject evaluate(HFunction vect) {
		if (vect.size() == 2) {
			return binaryOperator(vect.get(0), vect.get(1));
		}

		return null;
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param o0
	 * @param o1
	 *
	 * @return
	 *
	 * @see
	 */
	public HObject binaryOperator(HObject o0, HObject o1) {
		HObject  result = e2ObjArg(o0, o1);

		if (result != null) {
			return result;
		}
		if (o0 instanceof HDouble) {
			if (o1 instanceof HDouble) {
				return e2DblArg((HDouble) o0, (HDouble) o1);
			}
			if (o1 instanceof HDoubleComplex) {
				return e2DblComArg(new HDoubleComplex((HDouble) o0), 
										 (HDoubleComplex) o1);
			}

			return null;
		}
		if (o0 instanceof HDoubleComplex) {
			if (o1 instanceof HDouble) {
				return e2DblComArg((HDoubleComplex) o0, 
										 new HDoubleComplex((HDouble) o1));
			}
			if (o1 instanceof HDoubleComplex) {
				return e2DblComArg((HDoubleComplex) o0, (HDoubleComplex) o1);
			}

			return null;
		}
		if (o0 instanceof HInteger) {
			if (o1 instanceof HInteger) {
				return e2IntArg((HInteger) o0, (HInteger) o1);
			}
			if (o1 instanceof HFraction) {
				return e2FraArg(new HFraction((HInteger) o0, C.C1), 
									 (HFraction) o1);
			}
			if (o1 instanceof HComplex) {
				return e2ComArg(new HComplex((HInteger) o0, C.C0), (HComplex) o1);
			}

			return null;
		}
		if (o0 instanceof HFraction) {
			if (o1 instanceof HInteger) {
				return e2FraArg((HFraction) o0, 
									 new HFraction((HInteger) o1, C.C1));
			}
			if (o1 instanceof HFraction) {
				return e2FraArg((HFraction) o0, (HFraction) o1);
			}
			if (o1 instanceof HComplex) {
				return e2ComArg(new HComplex((HFraction) o0, C.C0), 
									 (HComplex) o1);
			}

			return null;
		}
		if (o0 instanceof HComplex) {
			if (o1 instanceof HInteger) {
				return eComIntArg((HComplex) o0, (HInteger) o1);
			}
			if (o1 instanceof HComplex) {
				return e2ComArg((HComplex) o0, (HComplex) o1);
			}
		}
		if (o0 instanceof HFunction) {
			if (o1 instanceof HInteger) {
				return eFunIntArg((HFunction) o0, (HInteger) o1);
			}
			if (o1 instanceof HFunction) {
				return e2FunArg((HFunction) o0, (HFunction) o1);
			}
		}

		return null;
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param o0
	 * @param o1
	 *
	 * @return
	 *
	 * @see
	 */
	public HObject e2ObjArg(HObject o0, HObject o1) {
		return null;
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param d0
	 * @param d1
	 *
	 * @return
	 *
	 * @see
	 */
	public HObject e2DblArg(HDouble d0, HDouble d1) {
		return null;
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param d0
	 * @param d1
	 *
	 * @return
	 *
	 * @see
	 */
	public HObject e2DblComArg(HDoubleComplex d0, HDoubleComplex d1) {
		return null;
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param i0
	 * @param i1
	 *
	 * @return
	 *
	 * @see
	 */
	public HObject e2IntArg(HInteger i0, HInteger i1) {
		return null;
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param f0
	 * @param f1
	 *
	 * @return
	 *
	 * @see
	 */
	public HObject e2FraArg(HFraction f0, HFraction f1) {
		return null;
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param c0
	 * @param i1
	 *
	 * @return
	 *
	 * @see
	 */
	public HObject eComIntArg(HComplex c0, HInteger i1) {
		return null;
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param c0
	 * @param c1
	 *
	 * @return
	 *
	 * @see
	 */
	public HObject e2ComArg(HComplex c0, HComplex c1) {
		return null;
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param f0
	 * @param f1
	 *
	 * @return
	 *
	 * @see
	 */
	public HObject e2FunArg(HFunction f0, HFunction f1) {
		return null;
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param f0
	 * @param i1
	 *
	 * @return
	 *
	 * @see
	 */
	public HObject eFunIntArg(HFunction f0, HInteger i1) {
		return null;
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param fun
	 * @param str
	 * @param prec
	 *
	 * @return
	 *
	 * @see
	 */
	static public String operatorString(HFunction fun, String str, int prec) {
		StringBuffer	buf = new StringBuffer();
		HObject			temp;

		if (fun.size() <= 1) {		// important for tracing pattern-matching
			buf.append(fun.head().toString());
			buf.append("(");

			if (fun.size() == 1) {
				buf.append(fun.get(0).toString());
			}

			buf.append(")");

			return buf.toString();
		}

		for (int i = 0; i < fun.size(); i++) {
			temp = fun.get(i);

			if (temp.precedence() < prec) {
				buf.append('(');
			}

			buf.append(temp.toString());

			if (temp.precedence() < prec) {
				buf.append(')');
			}
			if (i != (fun.size() - 1)) {
				buf.append(str);
			}
		}

		return buf.toString();
	}

}



/*--- formatting done in "Sun Java Convention" style on 06-18-1999 ---*/
