package com.hartmath.loadable;

import com.hartmath.lib.*;
import com.hartmath.util.*;
import com.hartmath.expression.*;
import com.hartmath.mapping.*;

/**
 * Class Declaration.
 * @see
 * @author
 * @version   %I%, %G%
 */
public class EDet extends E1Arg {
    public EDet()
    {
    }
    /**
     * Return the determinante of a square matrix. This algorithm uses LU decomposition
     * @param matrix mat
     * @return determinante of matrix mat
     */
    public HObject e1FunArg(HFunction mat) {
        int[] sz = mat.isMatrix();
        if (sz == null) {
            return null;
        }
        final int len0 = sz[0];
        if (len0 != sz[1]) {
            return null;
        }
        if (len0 == 0) {
            return C.C0;
        }
        if (sz[2] == -1) {
            // complex numeric
            HMatrix cm = new HMatrix(mat);
            ComplexSquareMatrix csm = new ComplexSquareMatrix(cm.toDoubleComplexMatrixArray());
            return csm.det();
        }
        if (sz[2] == 1) {
            // double numeric
            HMatrix dm = new HMatrix(mat);
            DoubleSquareMatrix dsm = new DoubleSquareMatrix(dm.toDoubleMatrixArray());
            return new HDouble(dsm.det());
        }
        HMatrix cm = new HMatrix(mat);
        HMatrix m = cm.copy(HFunction.MATRIXFORM);
//        HFunction m = mat.mClone(HFunction.MATRIXFORM);
        return det(m);
    }

    /**
     * Method Declaration.
     * @param m
     * @return
     * @see
     */
    //public HObject det(HFunction m) {
    public HObject det(HMatrix m) {
        HObject det = C.C1;
        int len0 = m.size();
        if (len0 == 1) {
            det = m.get(0, 0);
            if (det instanceof HNumber && ((HNumber)det).isZero()) {
                return C.C0;
            }
            return C.EV(C.Pow.f(det, C.CN1));
        }
        for (int pivot = 0; pivot < len0 - 1; pivot++) {
            HObject big;
            HObject test = m.get(pivot, pivot);
            if (test instanceof HNumber) {
                big = C.EV(C.Abs.f(((HNumber)test).re()));
            } else {
                big = test;
            }
            int swapRow = 0;
            for (int row = pivot + 1; row < len0; row++) {
                test = m.get(row, pivot);
                if (test instanceof HNumber) {
                    HObject absElement = C.EV(C.Abs.f(((HNumber)test).re()));
                    if (C.EV(C.GreaterEqual.f(absElement, big)).equals(C.True)) {
                        swapRow = row;
                        big = absElement;
                    }
                } else {
                    if (big instanceof HNumber && ((HNumber)big).isZero()) {
                        swapRow = row;
                        big = test;
                    }
                }
            }
            if (swapRow != 0) {
                HFunction prow = m.get(pivot);
                m.set(pivot, m.get(swapRow));
                m.set(swapRow, prow);
                det = C.EV(C.Multiply.f(C.CN1, det, m.get(pivot, pivot)));
            } else {
                det = C.EV(C.Multiply.f(det, m.get(pivot, pivot)));
            }
            if (det instanceof HNumber && ((HNumber)det).isZero()) {
                return det;
            }
            HObject pivotInverse = C.EV(C.Pow.f(m.get(pivot, pivot), C.CN1));
            for (int col = pivot + 1; col < len0; col++) {
                m.set(C.EV(C.Multiply.f(pivotInverse, m.get(pivot, col))), pivot, col);
            }
            for (int row = pivot + 1; row < len0; row++) {
                HObject temp = m.get(row, pivot);
                for (int col = pivot + 1; col < len0; col++) {
                    m.set(C.EV(C.Add.f(m.get(row, col), C.Multiply.f(C.CN1, m.get(pivot, col), temp))), row, col);
                }
            }
        }
        det = C.EV(C.Multiply.f(det, m.get(len0 - 1, len0 - 1)));
        return det;
    }
}

