package com.hartmath.loadable;

import com.hartmath.expression.*;
import com.hartmath.util.*;
import com.hartmath.mapping.*;
import com.hartmath.patternmatching.*;
import com.hartmath.lib.*;

public class EDot extends EMultiArg implements FunctionOpEvaluator
{
   public EDot()
   {
   }
   /**
    * Method Declaration.
    * @param o0
    * @param o1
    * @return
    * @see
    */
   public HObject e2ObjArg(HObject o0, HObject o1)
   {
      if (C.EV(C.IsVector.f(o0)) == C.True)
      {
         HFunction      f0 = (HFunction) o0;    // vector
         int            v0len = f0.size();

         if (C.EV(C.IsVector.f(o1)) == C.True)
         {
            HFunction   f1 = (HFunction) o1;

            if (v0len != f1.size())
            {
               return null;
            }

            HFunction   res = new HFunction(C.Add);

            for (int k = 0; k < v0len; k++)
            {
               res.add(C.Multiply.f(f0.get(k), f1.get(k)));
            }

            return res;
         }

         if (C.EV(C.IsMatrix.f(o1)) == C.True)
         {
            HFunction   f1 = (HFunction) o1;
            HFunction   g1 = (HFunction) f1.get(0);
            int         rowLen1 = f1.size();
            int         colLen1 = g1.size();

            if (v0len != rowLen1)
            {
               return null;
            }

            HFunction   res = new HFunction(C.List);
            HFunction   rowres;

            for (int k = 0; k < colLen1; k++)
            {
               rowres = new HFunction(C.Add);

               for (int i = 0; i < v0len; i++)
               {
                  rowres.add(C.Multiply.f(f0.get(i), f1.get(i, k)));
               }

               res.add(rowres);
            }

            return res;
         }

         return null;
      }

      if (C.EV(C.IsMatrix.f(o0)) == C.True)
      {
         HFunction      f0 = (HFunction) o0;            // matrix
         HFunction      g0 = (HFunction) f0.get(0);     // matrix-row

         if (C.EV(C.IsMatrix.f(o1)) == C.True)
         {
            HFunction   f1 = (HFunction) o1;
            HFunction   g1 = (HFunction) f1.get(0);

            // {{a,b},{c,d}}.{{e,f},{g,h}}
            int         colLen0 = g0.size();
            int         rowLen1 = f1.size();

            if (colLen0 != rowLen1)
            {
               return null;
            }

            HFunction   res = new HFunction(C.List);

            res.setProperties(HFunction.MATRIXFORM);

            HFunction   rowres;
            HFunction   temp;

            for (int k = 0; k < f0.size(); k++)
            {
               rowres = new HFunction(C.List);
               g0 = (HFunction) f0.get(k);

               for (int i = 0; i < g1.size(); i++)
               {
                  temp = new HFunction(C.Add);

                  for (int j = 0; j < colLen0; j++)
                  {
                     g1 = (HFunction) f1.get(j);

                     temp.add(C.Multiply.f(g0.get(j), g1.get(i)));
                  }

                  rowres.add(temp);
               }

               res.add(rowres);
            }

            return res;
         }

         if (C.EV(C.IsVector.f(o1)) == C.True)
         {
            HFunction   f1 = (HFunction) o1;
            int         colLen1 = g0.size();

            if (f1.size() != colLen1)
            {
               return null;
            }

            HFunction   res = new HFunction(C.List);
            HFunction   rowres;

            for (int k = 0; k < f0.size(); k++)
            {
               rowres = new HFunction(C.Add);
               g0 = (HFunction) f0.get(k);

               for (int i = 0; i < g0.size(); i++)
               {
                  rowres.add(C.Multiply.f(f1.get(i), g0.get(i)));
               }

               res.add(rowres);
            }

            return res;
         }
      }

      return null;
   }

   /**
    * Method Declaration.
    * @param fun
    * @return
    * @see
    */
   public String toOpString(HFunction fun)
   {
      return E2Arg.operatorString(fun, ".", 700);
   }

   /**
    * Method Declaration.
    * @return
    * @see
    */
   public int precedence()
   {
      return 700;
   }

}

