/*--- formatted by Jindent 2.1, (www.c-lab.de/~jindent) ---*/

package com.hartmath.awt;

import java.io.*;
import java.awt.*;
import java.awt.event.*;

/**
 * The EFrame class is intended to provide some of the functionality needed to make a Java application look and behave like
 * a C++ EPOC application
 * 
 * So far EFrame provides:
 * Full screen display
 * A menu bar with 'File' and 'Tools' menus (these will need to be internationalised)
 * An About dialog with a public property 'String[] aboutDialog.text'
 * A windowClosing event handler
 * 
 * A Java application can extend EFrame, rather than Frame, to use this additional functionality
 * 
 * Additional 'E' widgets to develop include:
 * a Dialog with a layout manager that conforms to EPOC guidelines
 * a Button which includes a short cut legend
 * 
 * @author  Jonathan Allin. (C) Jonathan Allin 1999
 * @version 0.1, 25/04/99
 */
public class EFrame extends Frame implements ActionListener {
    public MenuBar     menuBar = new MenuBar();
    public Menu	       fileMenu = new Menu("File");
    public MenuItem    closeItem = 
	new MenuItem("Close", new MenuShortcut(KeyEvent.VK_E));
    public MenuItem    openItem =
	new MenuItem("Open..", new MenuShortcut(KeyEvent.VK_O));
    public MenuItem    saveItem =
	new MenuItem("Save..", new MenuShortcut(KeyEvent.VK_S));
    public MenuItem    saveOutputItem =
	new MenuItem("Save Output..", new MenuShortcut(KeyEvent.VK_R));
    public Menu	       toolMenu = new Menu("Tools");
    public MenuItem    aboutItem = 
	new MenuItem("About..", new MenuShortcut(KeyEvent.VK_A));
    public AboutDialog aboutDialog;
    static final int   BORDER_WIDTH = 5;
    static final int   TITLE_HEIGHT = 25;

    boolean graphicsWindow;

    public TextArea        inputArea;
    public TextArea        outputArea;
    /**
     * Constructor declaration
     *
     *
     * @see
     */
    public EFrame(boolean subWin) {
	this("", subWin);
    }

    /**
     * Constructor declaration
     *
     *
     * @param title
     *
     * @see
     */
    public EFrame(String title, boolean subWin) {
	super(title);
  graphicsWindow = subWin;

	WindowListener wl = new WindowCloser(this);

	/*
	 * WindowListener wl = new WindowAdapter()
	 * {
	 * public void windowClosing(WindowEvent we)
	 * {
	 * shutDown();
	 * }
	 * };
	 */
	addWindowListener(wl);
  if (! graphicsWindow) {
    fileMenu.add(openItem);
    openItem.addActionListener(this);
    fileMenu.add(saveItem);
    saveItem.addActionListener(this);
    fileMenu.addSeparator();

    fileMenu.add(saveOutputItem);
    saveOutputItem.addActionListener(this);
    fileMenu.addSeparator();
  }
  fileMenu.add(closeItem);
	closeItem.addActionListener(this);


	toolMenu.add(aboutItem);
	aboutItem.addActionListener(this);

	aboutDialog = new AboutDialog(this);

	menuBar.add(fileMenu);
	menuBar.add(toolMenu);
	setMenuBar(menuBar);

	Dimension screenSize = getToolkit().getScreenSize();

	if (screenSize.width > 640) {
   // revo size:
   //     setSize(480, 160);
   // test 1/2 VGA
   //     setSize(640, 240);
   // full vga
	      setSize(640, 480);
	} else {
	    setBounds(-BORDER_WIDTH, -TITLE_HEIGHT, 
		      screenSize.width + 2 * BORDER_WIDTH, 
		      screenSize.height + TITLE_HEIGHT + BORDER_WIDTH);
	}
    }

    public void initTextArea(TextArea edi1, TextArea edi2) {
      inputArea  = edi1;
      outputArea = edi2;
    }
    /**
     * A sub class of EFrame that implements actionPerformed should call super.actionPeformed(ActionEvent e), to ensure
     * that the actions handled by EFrame are not ignored
     */
    public void actionPerformed(ActionEvent ae) {
	    if (ae.getSource() == closeItem) {
	        shutDown();
	        return;
      }
      if (ae.getSource() == openItem) {
          FileDialog d = new FileDialog(this, "Open HartMath file", FileDialog.LOAD);
        //  d.setFilenameFilter("*.hmt");
          d.setDirectory(".");
          d.show();
          String directory = d.getDirectory();
          String fileName = d.getFile();
          if (fileName!=null && directory!=null) {
            fileName = directory + fileName;
          //  C.inputArea.append(fileName);
            try {
  					  FileInputStream	fin =
	  				  	new FileInputStream(fileName);
		  			  DataInputStream	in = new DataInputStream(fin);
              BufferedReader bw = new BufferedReader(new InputStreamReader(in));
              boolean flag = false;
              String str;
              while ( (str=bw.readLine()) != null) {
                if (flag) {
                  inputArea.append( "\n" );
                }
                flag = true;
                inputArea.append( str );

              }
              bw.close();
            }
            catch (IOException me) {
            }
          }
	        return;
      }
      if (ae.getSource() == saveItem) {
          FileDialog d = new FileDialog(this, "Save HartMath file", FileDialog.SAVE);
        //  d.setFilenameFilter("*.hmt");
          d.setDirectory(".");
          d.show();
          String directory = d.getDirectory();
          String fileName = d.getFile();
          if (fileName!=null && directory!=null) {
            fileName = directory + fileName;
          //  C.inputArea.append(fileName);
            try {
  					  FileOutputStream	fin =
	  				  	new FileOutputStream(fileName);
		  			  DataOutputStream	in = new DataOutputStream(fin);
              BufferedWriter br = new BufferedWriter(new OutputStreamWriter(in));
              String str = inputArea.getText();
              if (str!=null) {
                br.write(str, 0, str.length());
              }
              br.close();
            }
            catch (IOException me) {
            }
          }
	        return;
      }
      if (ae.getSource() == saveOutputItem) {
          FileDialog d = new FileDialog(this, "Save Output file", FileDialog.SAVE);
        //  d.setFilenameFilter("*.hmt");
          d.setDirectory(".");
          d.show();
          String directory = d.getDirectory();
          String fileName = d.getFile();
          if (fileName!=null && directory!=null) {
            fileName = directory + fileName;
          //  C.inputArea.append(fileName);
            try {
  					  FileOutputStream	fin =
	  				  	new FileOutputStream(fileName);
		  			  DataOutputStream	in = new DataOutputStream(fin);
              BufferedWriter br = new BufferedWriter(new OutputStreamWriter(in));
              String str = outputArea.getText();
              if (str!=null) {
                br.write(str, 0, str.length());
              }
              br.close();
            }
            catch (IOException me) {
            }
          }
	        return;
      }
      if (ae.getSource() == aboutItem) {
	        aboutDialog.setVisible(true);
          return;
      }
    } 

    /**
     * Override shutDown() for different or additional behaviour
     */
    public void shutDown() {
      if (graphicsWindow) {
        dispose();
      }
      else {
    	  System.exit(0);
      }
    }

}

/**
 * Provides the about dialog
 */
class AboutDialog extends Dialog implements ActionListener {
    Button		   continueB = new Button("Continue");
    static public String[] text = {  
	"HartMath v0.6", "Computer Algebra System",
	"http://www.hartmath.com", 
	"Copyright khartlage@t-online.de"
    };
    static final int       WIDTH = 300;
    static final int       HEIGHT = 100;

    /**
     * Constructor declaration
     *
     *
     * @param parent
     *
     * @see
     */
    AboutDialog(Frame parent) {
	super(parent);

	setLayout(new FlowLayout());

	GridPanel panel = new GridPanel();
 	Canvas    canvas = new PaintCanvas();

	/*
	 * Canvas canvas = new Canvas()
	 * {
	 * public void paint(Graphics g)
	 * {
	 * paintText(g);
	 * }
	 * };
	 */
	canvas.setSize(WIDTH, HEIGHT);
	panel.add(canvas, 0, 0);
	panel.add(continueB, 0, 1);
	continueB.addActionListener(this);
	add(panel);
	pack();
    }

    public void actionPerformed(ActionEvent ae) {
	    if (ae.getSource() == continueB) {
        setVisible(false);
      }
    }

    /**
     * Method declaration
     *
     *
     * @param g
     *
     * @see
     */
    static public void paintText(Graphics g) {
	FontMetrics fm = g.getFontMetrics();
	int	    lineCount = text.length;

	for (int i = 0; i < lineCount; i++) {
	    int xOffset = (WIDTH - fm.stringWidth(text[i])) / 2;

	    g.drawString(text[i], xOffset, 
			 HEIGHT * (i + 1) / (lineCount + 1));
	} 
    } 

}

/**
 * Class declaration
 *
 *
 * @author
 * @version %I%, %G%
 */        
class WindowCloser extends WindowAdapter {
    EFrame frame;
    public WindowCloser(EFrame f) {
      frame = f;
    }
    /**
     * Method declaration
     *
     *
     * @param we
     *
     * @see
     */
    public void windowClosing(WindowEvent we) {
	    frame.shutDown();
    } 

}

class PaintCanvas extends Canvas {

    public void paint(Graphics g) {
	AboutDialog.paintText(g);
    }

}



/*--- formatting done in "Sun Java Convention" style on 08-09-1999 ---*/

