package com.hartmath.loadable;

import com.hartmath.lib.*;
import com.hartmath.util.*;
import com.hartmath.expression.*;
import com.hartmath.mapping.*;

/**
 * Class declaration
 * @author
 * @version %I%, %G%
 */
public class EInverse extends E1Arg {
    public EInverse()
    {
    }
    /**
     * Invert a matrix. If the input matrix is singular, return null.
     * @param matrix mat
     * @return inverse of matrix mat
     */
    public HObject e1FunArg(HFunction mat) {
        int[] sz = mat.isMatrix();
        if (sz == null) {
            return null;
        }
        final int len0 = sz[0];
        if (len0 != sz[1]) {
            return null;
        }
        if (len0 == 0) {
            return mat;
        }
        HMatrix cm = new HMatrix(mat);
      //  HFunction m = mat.copy(HFunction.MATRIXFORM);
        HMatrix m = cm.copy(HFunction.MATRIXFORM);
        if (len0 == 1) {
            HObject det = m.get(0, 0);
            if (det instanceof HNumber && ((HNumber)det).isZero()) {
                return null;
            }
            m.set(C.EV(C.Pow.f(det, C.CN1)), 0, 0);
            return m.getFunction();
        }
        if (sz[2] == -1) {
            // complex numeric
            ComplexSquareMatrix csm = new ComplexSquareMatrix(m.toDoubleComplexMatrixArray());
            csm = csm.inverse();
            return csm.toFunction();
        }
        if (sz[2] == 1) {
            // double numeric
            DoubleSquareMatrix dsm = new DoubleSquareMatrix(m.toDoubleMatrixArray());
            dsm = dsm.inverse();
            return dsm.toFunction();
        }
        int[] pivotFlag = new int[len0];
        int[] swapCol = new int[len0];
        int[] swapRow = new int[len0];
        int irow = 0, icol = 0;
        for (int i = 0; i < len0; i++) {
            HObject big = C.C0;
            for (int row = 0; row < len0; row++) {
                if (pivotFlag[row] == 0) {
                    for (int col = 0; col < len0; col++) {
                        if (pivotFlag[col] == 0) {
                            HObject test = m.get(row, col);
                            HObject absElement;
                            if (test instanceof HNumber) {
                                absElement = C.EV(C.Abs.f(((HNumber)test).re()));
                                if (C.EV(C.GreaterEqual.f(absElement, big)).equals(C.True)) {
                                    big = absElement;
                                    irow = row;
                                    icol = col;
                                }
                            }
                            else {
                                big = test;
                                irow = row;
                                icol = col;
                            }
                        }
                    }
                }
            }
            pivotFlag[icol] ++;
            if (irow != icol) {
                for (int col = 0; col < len0; col++) {
                    HObject temp = m.get(irow, col);
                    m.set(m.get(icol, col), irow, col);
                    m.set(temp, icol, col);
                }
            }
            swapRow[i] = irow;
            swapCol[i] = icol;
            if (m.get(icol, icol) instanceof HNumber && ((HNumber)m.get(icol, icol)).isZero()) {
                return null;
            }
            HObject pivotInverse = C.EV(C.Pow.f(m.get(icol, icol), C.CN1));
            m.set(C.C1, icol, icol);
            for (int col = 0; col < len0; col++) {
                m.set(C.EV(C.Multiply.f(m.get(icol, col), pivotInverse)), icol, col);
            }
            for (int row = 0; row < len0; row++) {
                if (row != icol) {
                    HObject temp = m.get(row, icol);
                    m.set(C.C0, row, icol);
                    for (int col = 0; col < len0; col++) {
                        m.set(C.EV(C.Add.f(m.get(row, col), C.Multiply.f(C.CN1, m.get(icol, col), temp))), row, col);
                    }
                }
            }
        }
        // undo the swaps
        for (int swap = len0 - 1; swap >= 0; swap--) {
            if (swapRow[swap] != swapCol[swap]) {
                for (int row = 0; row < len0; row++) {
                    HObject temp = m.get(row, swapRow[swap]);
                    m.set(m.get(row, swapCol[swap]), row, swapRow[swap]);
                    m.set(temp, row, swapCol[swap]);
                }
            }
        }
        return m.getFunction();
    }
}

