package com.hartmath.loadable;

import com.hartmath.expression.*;
import com.hartmath.util.*;
import com.hartmath.mapping.*;
import com.hartmath.patternmatching.*;
import com.hartmath.lib.*;

public class EReplaceAll implements FunctionEvaluator, FunctionOpEvaluator
{
   public EReplaceAll()
   {
   }
   /**
    * @param vect   the function, which should be evaluated
    * @return       <code>null</code> or the evaluated object
    */
   public HObject evaluate(HFunction vect)
   {
      if (vect.size() == 2 && vect.get(1) instanceof HFunction)
      {
         HFunction      f = (HFunction) vect.get(1);
         HObject        a0 = vect.get(0);

         if (f.isList())
         {
            for (int i = 0; i < f.size(); i++)
            {
               if (f.get(i) instanceof HFunction)
               {
                  a0 = useRule(a0, (HFunction) f.get(i));
               }
            }

            return a0;
         }
         else
         {
            return useRule(a0, f);
         }
      }

      return null;
   }

   /**
    * Method Declaration.
    * 
    * 
    * @param a0
    * @param f
    * 
    * @return
    * 
    * @see
    */
   HObject useRule(HObject a0, HFunction f)
   {
      HObject   result;
      HRuleSet  rset;

      if (f.size() == 2 && (f.head() == C.Rule || f.head() == C.RuleDelayed))
      {
         rset = new HRuleSet(f.get(0).head());

         if (f.head() == C.Rule)
         {
            rset.putDownRule(HRule.RULE, f.get(0), f.get(1));
         }
         else
         {
            rset.putDownRule(HRule.RULEDELAYED, f.get(0), 
                             f.get(1));
         }

         result = rset.getDownRule(a0);

         if (result != null)
         {
            return result;
         }

         if (a0 instanceof HFunction)
         {
            result = ((HFunction) a0).replaceAll(rset);

            if (result != null)
            {
               return result;
            }
         }
      }

      return a0;
   }

   /**
    * Method Declaration.
    * 
    * 
    * @param fun
    * 
    * @return
    * 
    * @see
    */
   public String toOpString(HFunction fun)
   {
      return E2Arg.operatorString(fun, "/.", 34);
   }

   /**
    * Method Declaration.
    * 
    * 
    * @return
    * 
    * @see
    */
   public int precedence()
   {
      return 34;
   }

}