package com.hartmath.loadable;

import com.hartmath.lib.*;
import com.hartmath.util.*;
import com.hartmath.expression.*;
import com.hartmath.mapping.*;

import java.io.*;
import java.net.*;
import java.math.*;

/**
 * Class Declaration.
 *
 *
 * @see
 *
 * @author
 * @version   %I%, %G%
 */
public class EWriteObject implements FunctionEvaluator
{
   DataOutputStream    os;
   public EWriteObject()
   {
   }
   /**
    * @param vect   the function, which should be evaluated
    * @return       <code>null</code> or the evaluated object
    */
   public HObject evaluate(HFunction vect)
   {
      if ((vect.size() == 2) && (vect.get(0) instanceof HString))
      {
         try
         {
            HSymbol.IOCounter = 0;

            HSymbol.IOOut.clear();
            HSymbol.IOIn.clear();
            FileOutputStream    fos =
               new FileOutputStream(vect.get(0).toString());
            os = new DataOutputStream(fos);
            HObject             obj = vect.get(1);

            writeTo(obj);
            os.close();
             /*
            FileOutputStream    fos =
               new FileOutputStream(vect.get(0).toString());
            DataOutputStream    out = new DataOutputStream(fos);
            HObject             obj = vect.get(1);

            obj.writeTo(out);
            out.close();
                */
            return C.Null;
         }
         catch (Exception e)
         {
            C.errorOut.append(e.toString());

            return null;
         }
      }

      return null;
   }

   public void writeTo(HObject obj) throws IOException
   {
         int hier = obj.hierarchy();
         if (hier<=HObject.INTEGERID) {
           if (hier<=HObject.DOUBLECOMPLEXID) {
             if (hier==HObject.DOUBLEID) {
               e1DblArg((HDouble) obj);
               return;
             }
             e1DblComArg((HDoubleComplex) obj);
             return;
           }
           e1IntArg((HInteger) obj);
           return;
         }
         if (hier<=HObject.COMPLEXID) {  
           if (hier==HObject.FRACTIONID) {  
             e1FraArg((HFraction) obj);  
             return ;  
           }  
           e1ComArg((HComplex) obj);  
           return;  
         }
         if (hier<=HObject.SYMBOLID) {    
           if (hier==HObject.STRINGID) {    
             e1StrArg((HString) obj);    
             return ;    
           }    
           e1SymArg((HSymbol) obj);    
           return;    
         }    
         if (hier==HObject.FUNCTIONID) {    
           e1FunArg((HFunction) obj);    
           return;    
         }    
         e1PatArg((HPattern) obj);    
         return;
   }

   public void e1DblArg(HDouble d) throws IOException
   {
      os.writeByte(HObject.DOUBLEFILEID);
      os.writeDouble(d.doubleValue());
   }
   public void e1DblComArg(HDoubleComplex d) throws IOException
   {
      os.writeByte(HObject.DOUBLECOMPLEXFILEID);
      os.writeDouble(d.real());
      os.writeDouble(d.imag());
   }

   public void e1IntArg(HInteger i) throws IOException
   {
     byte[]	barr = i.toByteArray();
     if (barr.length <= 8) {
		 	if (barr.length <= 4) {
		 		if (barr.length <= 2) {
		 			if (-5 <= i.intValue() && i.intValue() <= 5) {
		 				int	b = (i.intValue() + 58) * -1;
		
     				// C.traceOut.append( new Integer( b ).toString() );
		
     				os.writeByte((byte) b);
		 			} else {
            if (barr.length == 1) {
              os.writeByte((byte) 23);
		 				  os.writeByte(barr[0]);
            }
            else
            {
		 				  os.writeByte((byte) 22);
		 				  os.writeShort((short) i.intValue());
            }
		 			}
		 		} else {
		 			os.writeByte((byte) 21);
		 			os.writeInt(i.intValue());
		 		}
		 	} else {
		 		os.writeByte((byte) 20);
		 		os.writeLong(i.longValue());
		 	}
		 } else {
		 	os.writeByte(HObject.INTEGERFILEID);
		 	os.writeInt(barr.length);
		 	os.write(barr);
		 }

   }

   public void e1FraArg(HFraction f) throws IOException
   {
     BigInteger numer = f.toBigIntegerNumerator();
     BigInteger denom = f.toBigIntegerDenominator();
     if (numer.equals(HInteger.B1) && denom.compareTo(HInteger.B2) >= 0 
		 		  && denom.compareTo(HInteger.B10) <= 0) {
		 	int	b = (denom.intValue() + 62) * -1;

		 	os.writeByte((byte) b);
		 }
     else
     {
		 	byte[]	narr = numer.toByteArray();
      byte[]  darr = denom.toByteArray();
      if (narr.length <= 8 && darr.length <= 8) {
		 	  if (narr.length <= 4 && darr.length <= 4) {
		 	  	if (narr.length <= 2 && darr.length <= 2) {
            if (narr.length == 1 && darr.length == 1) {
              os.writeByte((byte) 27);
		 				  os.writeByte(narr[0]);
              os.writeByte(darr[0]);
            }
            else
            {
	 	  			  os.writeByte((byte) 26);
	 	  			  os.writeShort((short) numer.intValue());
              os.writeShort((short) denom.intValue());
            }
		 	  	}
          else
          {
		 	  		os.writeByte((byte) 25);
		 	  		os.writeInt(numer.intValue());
            os.writeInt(denom.intValue());
		 	  	}
		 	  } else {
		 	  	os.writeByte((byte) 24);
		 	  	os.writeLong(numer.longValue());
          os.writeLong(denom.longValue());
        }
		  }
      else
      {
		 	  os.writeByte(HObject.FRACTIONFILEID);
		 	  os.writeInt(narr.length);
		 	  os.write(narr);
		 	  os.writeInt(darr.length);
        os.write(darr);
      }
     }
   }

   public void e1ComArg(HComplex c) throws IOException
   {
     os.writeByte(HObject.COMPLEXFILEID);
	 	 writeTo(c.re());
		 writeTo(c.im());
   }

   public void e1StrArg(HString s) throws IOException
   {
     os.writeByte(HObject.STRINGFILEID);
		 os.writeUTF(s.toString());
   }

   public void e1SymArg(HSymbol s) throws IOException
   {
      String str = s.toString();
      if ((str.length() == 1) && (str.charAt(0) >= 'a') 
          && (str.charAt(0) <= 'z'))
      {
         int    b = ((str.charAt(0) - 'a') + 1) * -1;

         // C.traceOut.append( new Integer( b ).toString() );

         os.writeByte((byte) b);
      }
      else
      {
         Short  num = (Short) HSymbol.IOOut.get(str);

         if (num != null)
         {
            os.writeByte((byte) 33);
            os.writeShort(num.shortValue());
         }
         else
         {
            int       indx = -1;

            for (int i = 0; i < C.IOConstantTable.length; i++)
            {
               if (s.equals(C.IOConstantTable[i]))
               {
                  indx = i;

                  break;
               }
            }

            if (indx != -1)
            {
               os.writeByte((byte) (34));
               os.writeByte((byte) (indx));
               return;
            }
            for (int i = 0; i < C.IOSymbolTable.length; i++)
            {
               if (s.equals(C.IOSymbolTable[i]))
               {
                  indx = i;

                  break;
               }
            }

            if (indx != -1)
            {
               os.writeByte((byte) (35));
               os.writeByte((byte) (indx));
               return;
            }

            os.writeByte(HObject.SYMBOLFILEID);
            os.writeUTF(str);
            os.writeInt(s.getAttributes());

            num = new Short((short) HSymbol.IOCounter);

            HSymbol.IOOut.put(str, num);

            HSymbol.IOCounter++;
         }
      }
   }

   public void e1FunArg(HFunction f) throws IOException
   {
      int       sz = f.size();
      int       indx = -1;

      if (sz <= 5)
      {
         for (int i = 0; i < C.IOSymbolTable.length; i++)
         {
            if (f.head().equals(C.IOSymbolTable[i]))
            {
               indx = i;

               break;
            }
         }

         if (indx != -1)
         {
            os.writeByte((byte) (80 + sz));
            os.writeByte((byte) (indx));

            for (int i = 0; i < sz; i++)
            {
              writeTo(f.get(i));
            }
         }
         else
         {
            os.writeByte((byte) (HObject.FUNCTIONFILEID - sz - 1));
            e1SymArg(f.head());

            for (int i = 0; i < sz; i++)
            {
               writeTo(f.get(i));
            }
         }
      }
      else
      {
         os.writeByte(HObject.FUNCTIONFILEID);
         os.writeInt(sz);
         e1SymArg(f.head());

         for (int i = 0; i < sz; i++)
         {
           writeTo(f.get(i));
         }
      }
   }

   public void e1PatArg(HPattern p) throws IOException
   {
     HSymbol symb = p.getSymbol();
     HObject headCheck = p.getCheckFunction();

     if (headCheck != null) {
		 	os.writeByte(HObject.PATTERNFILEID);
		 	e1SymArg(symb);
		 	writeTo(headCheck);
		 } else {
      String str = symb.toString();
		 	if ((str.length() == 1) && (str.charAt(0) >= 'a')
		 			  && (str.charAt(0) <= 'z')) {
		 		int	b = ((str.charAt(0) - 'a' + 26)+1) * -1;

		 		// C.traceOut.append( new Integer( b ).toString() );

		 		os.writeByte((byte) b);
		 	} else {
		 		os.writeByte((byte) 66);
		 		e1SymArg(symb);
		 	}
     }
   }
}
