package com.hartmath.JSci.chemistry;

import com.hartmath.JSci.chemistry.periodictable.*;
import com.hartmath.lib.*;
import com.hartmath.expression.*;
import com.hartmath.mapping.*;
/**
* A class representing chemical elements.
* @version 1.5
* @author Mark Hale
*/
//public abstract class Element extends Object implements java.io.Serializable {
//hartmath
public class Element implements FunctionEvaluator {
  private String elementName;
  private String shortCut;
  private int    atomicNumber;             //Ordnungszahl
  private int    massNumber;               //rel. Atommasse
  private double electroNegativity;        //Elektronegativitt 
  private double covalentRadius;
  private double atomicRadius;
  private double meltingPoint;             //Schmelzpunkt in K
  private double boilingPoint;
  private double density;
  private double specificHeat;
  private double electricalConductivity;
  private double thermalConductivity;
        public Element()  {}
        /**
        * Constructs an element.
        */   /*
        public Element(String s,
                       int an, int mn, double en, double cr, double ar,
                       double mp, double bp, double de, double sh,
                       double ec, double tc)
        {
          elementName             = s;
          shortCut                = s;
          atomicNumber            = an;
          massNumber              = mn;
          electroNegativity       = en;
          covalentRadius          = cr;
          atomicRadius            = ar;
          meltingPoint            = mp;
          boilingPoint            = bp;
          density                 = de;
          specificHeat            = sh;
          electricalConductivity  = ec;
          thermalConductivity     = tc;
        }          */
        public Element(String elemName, String s,
                       int an, int mn, double en, double cr, double ar,
                       double mp, double bp, double de, double sh,
                       double ec, double tc)
        {
          elementName             = elemName;
          shortCut                = s;
          atomicNumber            = an;
          massNumber              = mn;
          electroNegativity       = en;
          covalentRadius          = cr;
          atomicRadius            = ar;
          meltingPoint            = mp;
          boilingPoint            = bp;
          density                 = de;
          specificHeat            = sh;
          electricalConductivity  = ec;
          thermalConductivity     = tc;
        }
        /**
        * Returns the elements name in english:
        */
        public String elementName() {
          return elementName;
        }
        /**
        * Returns the chemical symbol.
        */
        public String toString() {
          return shortCut;
        }
        /**
        * Returns the atomic number.
        */
        public  int atomicNumber()
        {
          return atomicNumber;
        }
        /**
        * Returns the mass number.
        */
        public  int massNumber()
        {
          return massNumber;
        }
        /**
        * Returns the electronegativity.
        */
        public  double electronegativity()
        {
          return electroNegativity;
        }
        /**
        * Returns the covalent radius.
        */
        public  double covalentRadius()
        {
          return covalentRadius;
        }
        /**
        * Returns the atomic radius.
        */
        public  double atomicRadius()
        {
          return atomicRadius;
        }
        /**
        * Returns the melting point (K).
        */
        public  double meltingPoint()
        {
          return meltingPoint;
        }
        /**
        * Returns the boiling point (K).
        */
        public  double boilingPoint()
        {
          return boilingPoint;
        }
        /**
        * Returns the density (293K).
        */
        public  double density()
        {
          return density;
        }
        /**
        * Returns the specific heat.
        */
        public  double specificHeat()
        {
          return specificHeat;
        }
        /**
        * Returns the electrical conductivity.
        */
        public  double electricalConductivity()
        {
          return electricalConductivity;
        }
        /**
        * Returns the thermal conductivity.
        */
        public  double thermalConductivity()
        {
          return thermalConductivity;
        }
        /**
        * Compares two elements for equality.
        * @param e an element
        */
        public boolean equals(Object e) {
                return (e!=null) && (e instanceof Element) &&
                        (atomicNumber()==((Element)e).atomicNumber()) &&
                        (massNumber()==((Element)e).massNumber());
        }

        //hartmath:
        public HObject evaluate(HFunction vect) {
		      return C.Null;
	      }
  final static public Element Actinium =
    new TransitionMetal("Actinium", "Ac", 89, 227, 1.1, Double.NaN, Double.NaN, 1323.0, 3473.0,
                10.07, Double.NaN, Double.NaN, 0.12);
  final static public Element Aluminium =
    new Metal("Aluminium", "Al", 13, 27, 1.61, 1.25, 1.82, 933.25, 2793.0,
                2.702, 0.9, 0.377e6, 2.37);
  final static public Element Americium =
    new RareEarthMetal("Americium", "Am", 95, 243, 1.3, Double.NaN, Double.NaN, 1268.0, 2880.0,
                13.6, 0.11, 0.022e6, 0.1);
  final static public Element Antimony =
    new Metal("Antimony", "Sb", 51, 122, 2.05, 1.41, 1.53, 904.0, 1860.0,
                6.684, 0.21, 0.0288e6, 0.243);
  final static public Element Argon =
    new NobleGas("Argon", "Ar", 18, 40, 0.0, 0.98, 0.88, 83.81, 87.3,
                0.0017824, 0.520, 0.0, 0.0001772);
  final static public Element Arsenic =
    new NonMetal("Arsenic", "As", 33, 75, 2.18, 1.21, 1.33, 1081.0, 876.0,
                5.72, 0.33, 0.0345e6, 0.5);
  final static public Element Astatine =
    new Halogen("Astatine", "At", 85, 210, 2.2, 1.45, 1.43, 575.0, 610.0,
                Double.NaN, Double.NaN, 0.0, 0.017);
  final static public Element Barium =
    new AlkaliEarthMetal("Barium", "Ba", 56, 137, 0.89, 1.98, 2.78, 1002.0, 2171.0,
                3.51, 0.204, 0.030e6, 0.184);
  final static public Element Berkelium =
    new RareEarthMetal("Berkelium", "Bk", 97, 249, 1.3, Double.NaN, Double.NaN, Double.NaN, Double.NaN,
                Double.NaN, Double.NaN, Double.NaN, 0.1);
  final static public Element Beryllium =
    new AlkaliEarthMetal("Beryllium", "Be", 4, 9, 1.57, 0.9, 1.4, 1560.0, 2745.0,
                1.848, 1.82, 0.313e6, 2.0);
  final static public Element Bismuth =
    new Metal("Bismuth", "Bi", 83, 209, 2.02, 1.52, 1.63, 544.52, 1837.0,
                9.8, 0.12, 8.67e3, 0.078);
  final static public Element Boron =
    new NonMetal("Boron", "B", 5, 11, 2.04, 0.88, 1.17, 2300.0, 4275.0,
                2.34, 1.02, 1.0e-6, 0.270);
  final static public Element Bromine =
    new Halogen("Bromine", "Br", 35, 80, 2.96, 1.142, 1.12, 265.9, 332.25,
                3.119, 0.473, 0.0, 0.00122);
  final static public Element Cadmium =
    new TransitionMetal("Cadmium", "Cd", 48, 112, 1.69, 1.41, 1.71, 594.18, 1040,
                8.65, 0.23, 0.138e6, 0.968);
  final static public Element Caesium =
    new AlkaliMetal("Caesium", "Cs", 55, 133, 0.79, 2.35, 3.34, 301.55, 944.0,
                1.873, 0.24, 0.0489e6, 0.359);
  final static public Element Calcium =
    new AlkaliEarthMetal("Calcium", "Ca", 20, 40, 1.00, 1.74, 2.23, 1112.0, 1757.0,
                1.55, 0.63, 0.298e6, 2.0);
  final static public Element Californium =
    new RareEarthMetal("Californium", "Cf", 98, 251, 1.3, Double.NaN, Double.NaN, 900.0, Double.NaN,
                Double.NaN, Double.NaN, Double.NaN, 0.1);
  final static public Element Carbon =
    new NonMetal("Carbon", "C", 6, 12, 2.55, 0.77, 0.91, 3550.0, 4827.0,
                2.62, 0.71, 0.61e9, 1.29);
  final static public Element Cerium =
    new RareEarthMetal("Cerium", "Ce", 58, 140, 1.12, 1.65, 2.70, 1071.0, 3699.0,
                6.78, 0.19, 0.0115e6, 0.114);
  final static public Element Chlorine =
    new Halogen("Chlorine", "Cl", 17, 35, 3.16, 0.99, 0.97, 172.16, 239.1,
                0.003214, 0.48, 0.0, 0.089e-3);
  final static public Element Chromium =
    new TransitionMetal("Chromium", "Cr", 24, 52, 1.66, 1.17, 1.85, 2130, 2945,
                7.19, 0.45, 0.0774e6, 0.937);
  final static public Element Cobalt =
    new TransitionMetal("Cobalt", "Co", 27, 59, 1.88, 1.16, 1.67, 1768, 3201,
                8.9, 0.42, 0.172e6, 1.00);
  final static public Element Copper =
    new TransitionMetal("Copper", "Cu", 29, 63, 1.90, 1.17, 1.57, 1357.6, 2836,
                8.96, 0.38, 0.596e6, 4.01);
  final static public Element Curium =
    new RareEarthMetal("Curium","Cm", 96, 247, 1.3, Double.NaN, Double.NaN, 1340.0, Double.NaN,
                13.51, Double.NaN, Double.NaN, 0.1);
  final static public Element Dysprosium =
    new RareEarthMetal("Dysprosium","Dy", 66, 162, 1.22, 1.59, 2.49, 1682.0, 2835.0,
                8.536, 0.17, 0.0108e6, 0.107);
  final static public Element Einsteinium =
    new RareEarthMetal("Einsteinium","Es", 99, 254, 1.3, Double.NaN, Double.NaN, Double.NaN, Double.NaN,
                Double.NaN, Double.NaN, Double.NaN, 0.1);
  final static public Element Erbium =
    new RareEarthMetal("Erbium", "Er", 68, 167, 1.24, 1.57, 2.45, 1795.0, 3136.0,
                9.05, 0.17, 0.0117e6, 0.143);
  final static public Element Europium =
    new RareEarthMetal("Europium", "Eu", 63, 152, 1.2, 1.85, 2.56, 1090.0, 1870.0,
                5.259, 0.18, 0.0112e6, 0.139);
  final static public Element Fermium =
    new RareEarthMetal("Fermium", "Fm", 100, 257, 1.3, Double.NaN, Double.NaN, Double.NaN, Double.NaN,
                Double.NaN, Double.NaN, Double.NaN, 0.1);
  final static public Element Fluorine =
    new Halogen("Fluorine", "F", 9, 19, 3.98, 0.64, 0.57, 53.48, 84.95,
                0.001696, 0.82, 0.0, 0.000279);
  final static public Element Francium =
    new AlkaliEarthMetal("Francium", "Fr", 87, 223, 0.7, Double.NaN, Double.NaN, 300.0, 950.0,
                Double.NaN, Double.NaN, 0.03e6, 0.15);
  final static public Element Gadolinium =
    new RareEarthMetal("Gadolinium", "Gd", 64, 157, 1.2, 1.61, 2.54, 1585.0, 3539.0,
                7.895, 0.23, 0.00736e6, 0.106);
  final static public Element Gallium =
    new Metal("Gallium", "Ga", 31, 70, 1.81, 1.25, 1.81, 302.9, 2478.0,
                5.907, 0.37, 0.0678e6, 0.406);
  final static public Element Germanium =
    new Metal("Germanium", "Ge", 32, 72, 2.01, 1.22, 1.52, 1210.4, 3107.0,
                5.323, 0.32, 0.0145, 0.599);
  final static public Element Gold =
    new TransitionMetal("Gold", "Au", 79, 197, 2.54, 1.34, 1.79, 1337.5, 3130.0,
                19.32, 0.128, 0.452e6, 3.17);
  final static public Element Hafnium =
    new TransitionMetal("Hafnium", "Hf", 72, 178, 1.3, 1.44, 2.16, 2500.0, 4876.0,
                13.2, 0.14, 0.0312e6, 0.230);
  final static public Element Helium =
    new NobleGas("Helium", "He", 2, 4, 0.0, 0.93, 0.49, 0.95, 4.215,
                0.0001787, 5.193, 0.0, 0.00152);
  final static public Element Holmium =
    new RareEarthMetal("Holmium", "Ho", 67, 165, 1.23, 1.58, 2.47, 1743.0, 2968.0,
                8.80, 0.16, 0.0124e6, 0.162);
  final static public Element Hydrogen =
    new NonMetal("Hydrogen", "H", 1, 1, 2.20, 0.371, 0.79, 14.025, 20.268,
                0.0000899, 14.304, 0.0, 0.001815);
  final static public Element Indium =
    new Metal("Indium", "In", 49, 115, 1.78, 1.50, 2.00, 429.76, 2346.0,
                7.31, 0.23, 0.116e6, 0.816);
  final static public Element Iodine =
    new Halogen("Iodine", "I", 53, 127, 2.66, 1.333, 1.32, 386.7, 458.4,
                4.93, 0.214, 8.0e-10, 0.00449);
  final static public Element Iridium =
    new TransitionMetal("Iridium", "Ir", 77, 192, 2.20, 1.27, 1.87, 2716.0, 4701.0,
                22.42, 0.130, 0.197e6, 1.47);
  final static public Element Iron =
    new TransitionMetal("Iron", "Fe", 26, 56, 1.83, 1.165, 1.72, 1809.0, 3135.0,
                7.86, 0.44, 0.0993e6, 0.802);
  final static public Element Krypton =
    new NobleGas("Krypton", "Kr", 36, 84, 0.0, 1.12, 1.03, 115.78, 119.8,
                0.003708, 0.248, 0.0, 0.0000949);
  final static public Element Lanthanum =
    new TransitionMetal("Lanthanum", "La", 57, 139, 1.10, 1.69, 2.74, 1193.0, 3730.0,
                6.7, 0.19, 0.0126e6, 0.135);
  final static public Element Lawrencium =
    new RareEarthMetal("Lawrencium", "Lr", 103, 260, Double.NaN, Double.NaN, Double.NaN, Double.NaN, Double.NaN,
                Double.NaN, Double.NaN, Double.NaN, 0.1);
  final static public Element Lead =
    new Metal("Lead", "Pb", 82, 207, 2.33, 1.54, 1.81, 600.6, 2023.0,
                11.34, 0.13, 0.0481e6, 0.353);
  final static public Element Lithium =
    new AlkaliMetal("Lithium", "Li", 3, 7, 0.98, 1.23, 2.05, 453.7, 1615.0,
                0.53, 3.6, 0.108e6, 0.847);
  final static public Element Lutetium =
    new RareEarthMetal("Lutetium", "Lu", 71, 175, 1.27, 1.56, 2.25, 1936.0, 3668.0,
                9.85, 0.15, 0.0185e6, 0.164);
  final static public Element Magnesium =
    new AlkaliEarthMetal("Magnesium", "Mg", 12, 24, 1.31, 1.36, 1.72, 922.0, 1363.0,
                1.738, 1.02, 0.226e6, 1.56);
  final static public Element Manganese =
    new TransitionMetal("Manganese", "Mn", 25, 55, 1.55, 1.17, 1.79, 1517.0, 2335.0,
                7.43, 0.48, 0.00695e6, 0.0782);
  final static public Element Mendelevium =
    new RareEarthMetal("Mendelevium", "Md", 101, 256, 1.3, Double.NaN, Double.NaN, Double.NaN, Double.NaN,
                Double.NaN, Double.NaN, Double.NaN, 0.1);
  final static public Element Mercury =
    new TransitionMetal("Mercury", "Hg", 80, 201, 2.00, 1.44, 1.76, 234.28, 630.0,
                13.546, 0.139, 0.0104e6, 0.0834);  
  final static public Element Molybdenum =
    new TransitionMetal("Molybdenum", "Mo", 42, 96, 2.16, 1.29, 2.01, 2890.0, 4912.0,
                10.2, 0.25, 0.187e6, 1.38);
  final static public Element Neodymium =
    new RareEarthMetal("Neodymium", "Nd", 60, 144, 1.14, 1.64, 2.64, 1289.0, 3341.0,
                7.0, 0.19, 0.0157e6, 0.165);
  final static public Element Neon =
    new NobleGas("Neon", "Ne", 10, 20, 0.0, 0.71, 0.51, 24.553, 27.096,
                0.000901, 0.904, 0.0, 0.000493);
  final static public Element Neptunium =
    new RareEarthMetal("Neptunium", "Np", 93, 237, 1.36, Double.NaN, Double.NaN, 910.0, Double.NaN,
                20.45, 0.12, 0.00822e66, 0.063);
  final static public Element Nickel =
    new TransitionMetal("Nickel", "Ni", 28, 59, 1.91, 1.15, 1.62, 1726.0, 3187.0,
                8.90, 0.44, 0.143e6, 0.907);
  final static public Element Niobium =
    new TransitionMetal("Niobium", "Nb", 41, 93, 1.6, 1.34, 2.08, 2740.0, 5017.0,
                8.57, 0.26, 0.0693e6, 0.537);
  final static public Element Nitrogen =
    new NonMetal("Nitrogen", "N", 7, 14, 3.04, 0.70, 0.75, 63.14, 77.35,
                0.0012506, 1.04, Double.NaN, 0.0002598);
  final static public Element Nobelium =
    new RareEarthMetal("Nobelium", "No", 102, 259, 1.3, Double.NaN, Double.NaN, Double.NaN, Double.NaN,
                Double.NaN, Double.NaN, Double.NaN, 0.1);
  final static public Element Osmium =
    new TransitionMetal("Osmium", "Os", 76, 190, 2.2, 1.26, 1.92, 3300.0, 5285.0,
                22.40, 0.13, 0.109e6, 0.876); 
  final static public Element Oxygen =
    new NonMetal("Oxygen", "O", 8, 16, 3.44, 0.66, 0.65, 50.35, 90.13,
                0.001429, 0.92, 0.0, 0.0002674);
  final static public Element Palladium =
    new TransitionMetal("Palladium", "Pd", 46, 106, 2.20, 1.28, 1.79, 1825.0, 3237.0,
                12.02, 0.24, 0.0950e6, 0.718);
  final static public Element Phosphorus =
    new NonMetal("Phosphorus", "P", 15, 31, 2.19, 1.10, 1.23, 317.3, 550.0,
                1.82, 0.77, 1.0e-11, 0.00235);
  final static public Element Platinum =
    new TransitionMetal("Platinum", "Pt", 78, 195, 2.28, 1.30, 1.83, 2045.0, 4100.0,
                21.45, 0.13, 0.0966e6, 0.716);
  final static public Element Plutonium =
    new RareEarthMetal("Plutonium", "Pu", 94, 242, 1.28, Double.NaN, Double.NaN, 913.0, 3503.0,
                19.8, 0.13, 0.00666e6, 0.0674);
  final static public Element Polonium =
    new Metal("Polonium", "Po", 84, 210, 2.0, 1.53, 1.53, 527.0, 1235.0,
                9.4, 0.12, 0.0219e6, 0.20);
  final static public Element Potassium =
    new AlkaliMetal("Potassium", "K", 19, 39, 0.82, 2.025, 2.77, 336.35, 1032.0,
                0.862, 0.75, 0.139e6, 1.024);
  final static public Element Praseodymium =
    new RareEarthMetal("Praseodymium", "Pr", 59, 141, 1.13, 1.65, 2.67, 1204.0, 3785.0,
                6.77, 0.19, 0.0148e6, 0.125);
  final static public Element Promethium =
    new RareEarthMetal("Promethium", "Pm", 61, 145, 1.13, 1.63, 2.62, 1204.0, 3785.0,
                6.475, 0.18, Double.NaN, 0.179);
  final static public Element Protactinium =
    new RareEarthMetal("Protactinium", "Pa", 91, 231, 1.5, Double.NaN, Double.NaN, Double.NaN, Double.NaN,
                15.4, 0.12, 0.0529e6, 0.47);
  final static public Element Radium =
    new AlkaliMetal("Radium","Ra", 88, 226, 0.9, Double.NaN, Double.NaN, 973.0, 1809.0,
                5.0, 0.12, Double.NaN, 0.186);
  final static public Element Radon =
    new NobleGas("Radon","Rn", 86, 222, 0.0, Double.NaN, 1.34, 202.0, 211.0,
                0.00973, 0.09, 0.0, 0.0000364);
  final static public Element Rhenium =
    new TransitionMetal("Rhenium", "Re", 75, 186, 1.9, 1.28, 1.97, 3453.0, 5869.0,
                21.00, 0.13, 0.0542e6, 0.479);
  final static public Element Rhodium =
    new TransitionMetal("Rhodium", "Rh", 45, 103, 2.28, 1.25, 1.83, 2236.0, 3970.0,
                12.4, 0.242, 0.211e6, 1.50);  
  final static public Element Rubidium =
    new AlkaliMetal("Rubidium", "Rb", 37, 85, 0.82, 2.16, 2.98, 312.64, 961.0,
                1.53, 0.363, 0.0779e6, 0.582);  
  final static public Element Ruthenium =
    new TransitionMetal("Ruthenium", "Ru", 44, 101, 2.2, 1.25, 1.89, 2523.0, 4423.0,
                12.2, 0.238, 0.137e6, 1.17);
  final static public Element Samarium =
    new RareEarthMetal("Samarium", "Sm", 62, 150, 1.17, 1.62, 2.59, 1345.0, 2064.0,
                7.54, 0.20, 0.956e4, 0.133);
  final static public Element Scandium =
    new TransitionMetal("Scandium", "Sc", 21, 45, 1.36, 1.44, 2.09, 1812.0, 3104.0,
                3.0, 0.6, 0.0177e6, 0.158);
  final static public Element Selenium =
    new NonMetal("Selenium", "Se", 34, 79, 2.55, 1.17, 1.22, 494.0, 958.0,
                4.79, 0.32, 1.0e-6, 0.0204); 
  final static public Element Silicon =
    new NonMetal("Silicon", "Si", 14, 28, 1.90, 1.17, 1.46, 1685.0, 3540.0,
                2.33, 0.71, 2.52e-6, 1.48);
  final static public Element Silver =
    new TransitionMetal("Silver", "Ag", 47, 108, 1.93, 1.34, 1.75, 1234.0, 2436.0,
                10.05, 0.235, 0.630e6, 4.29);
  final static public Element Sodium =
    new AlkaliMetal("Sodium", "Na", 11, 23, 0.93, 1.57, 2.23, 371.0, 1156.0,
                0.971, 1.23, 0.139e6, 1.41);
  final static public Element Strontium =
    new AlkaliEarthMetal("Strontium", "Sr", 38, 88, 0.95, 1.92, 2.45, 1041.0, 1650.0,
                2.6, 0.30, 0.0762e6, 0.353);
  final static public Element Sulphur =
    new NonMetal("Sulphur", "S", 16, 32, 2.58, 1.04, 1.09, 388.36, 717.75,
                2.07, 0.71, 0.5e-17, 0.00269);
  final static public Element Tantalum =
    new TransitionMetal("Tantalum", "Ta", 73, 181, 1.5, 1.34, 2.09, 3287.0, 5731.0,
                16.6, 0.14, 0.0761e6, 0.575);
  final static public Element Technetium =
    new TransitionMetal("Technetium", "Tc", 43, 99, 1.9, 1.27, 1.95, 2473.0, 4538.0,
                11.5, 0.21, 0.067e6, 0.506);
  final static public Element Tellurium =
    new NonMetal("Tellurium", "Te", 52, 128, 2.1, 1.37, 1.42, 722.65, 1621.0,
                6.24, 0.20, 2.0, 0.0235);
  final static public Element Terbium =
    new RareEarthMetal("Terbium", "Tb", 65, 159, 1.2, 1.59, 2.51, 1630.0, 3496.0,
                8.27, 0.18, 0.00889e6, 0.111);
  final static public Element Thallium =
    new Metal("Thallium", "Tl", 81, 204, 2.04, 1.55, 2.08, 577.0, 1746.0,
                11.85, 0.13, 0.0617e6, 0.461);
  final static public Element Thorium =
    new RareEarthMetal("Thorium", "Th", 90, 232, 1.3, 1.65, Double.NaN, 2028.0, 5061.0,
                11.7, 0.12, 0.0653e6, 0.540);
  final static public Element Thulium =
    new RareEarthMetal("Thulium", "Tm", 69, 169, 1.25, 1.56, 2.42, 1818.0, 2220.0,
                9.33, 0.16, 0.0150e6, 0.168);
  final static public Element Tin =
    new Metal("Tin", "Sn", 50, 119, 1.96, 1.40, 1.72, 505.06, 2876.0,
                7.30, 0.227, 0.0917e6, 0.666);
  final static public Element Titanium =
    new TransitionMetal("Titanium", "Ti", 22, 48, 1.54, 1.32, 2.00, 1943.0, 3562.0,
                4.50, 0.52, 0.0234e6, 0.219);
  final static public Element Tungsten =
    new TransitionMetal("Tungsten", "W", 74, 184, 2.36, 1.30, 2.02, 3680.0, 5828.0,
                19.3, 0.13, 0.189e6, 1.74);
  final static public Element Meitnerium =
    new TransitionMetal("Meitnerium", "Mt", 109, 266, Double.NaN, Double.NaN, Double.NaN, Double.NaN, Double.NaN,
                Double.NaN, Double.NaN, Double.NaN, Double.NaN);
  final static public Element Seaborgium =
    new TransitionMetal("Seaborgium", "Sg", 106, 263, Double.NaN, Double.NaN, Double.NaN, Double.NaN, Double.NaN,
                Double.NaN, Double.NaN, Double.NaN, Double.NaN);
  final static public Element Hassium =
    new TransitionMetal("Hassium", "Hs", 108, 265, Double.NaN, Double.NaN, Double.NaN, Double.NaN, Double.NaN,
                Double.NaN, Double.NaN, Double.NaN, Double.NaN);
  final static public Element Dubnium =
    new TransitionMetal("Dubnium", "Db", 105, 262, Double.NaN, Double.NaN, Double.NaN, Double.NaN, Double.NaN,
                Double.NaN, Double.NaN, Double.NaN, 0.58);
  final static public Element Rutherfordium =
    new TransitionMetal("Rutherfordium", "Rf", 104, 261, Double.NaN, Double.NaN, Double.NaN, Double.NaN, Double.NaN,
                Double.NaN, Double.NaN, Double.NaN, 0.23);
  final static public Element Bohrium =
    new TransitionMetal("Bohrium", "Bh", 107, 262, Double.NaN, Double.NaN, Double.NaN, Double.NaN, Double.NaN,
                Double.NaN, Double.NaN, Double.NaN, Double.NaN);
  final static public Element Ununnilium =
    new TransitionMetal("Ununnilium", "Unn", 110, 272, Double.NaN, Double.NaN, Double.NaN, Double.NaN, Double.NaN,
                Double.NaN, Double.NaN, Double.NaN, Double.NaN); 
  final static public Element Uranium =
    new RareEarthMetal("Uranium", "U", 92, 238, 1.38, 1.42, Double.NaN, 1405.0, 4407.0,
                18.9, 0.12, 0.038e6, 0.276);
  final static public Element Vanadium =
    new TransitionMetal("Vanadium", "V", 23, 51, 1.63, 1.22, 1.92, 2175.0, 3682.0,
                5.8, 0.49, 0.0489e6, 0.307);
  final static public Element Xenon =
    new NobleGas("Xenon", "Xe", 54, 131, 0.0, 1.31, 1.24, 161.36, 165.03,
                0.00588, 0.158, 0.0, 0.0569e-3);
  final static public Element Ytterbium =
    new RareEarthMetal("Ytterbium", "Yb", 70, 173, 1.1, 1.74, 2.40, 1097.0, 1467.0,
                6.98, 0.15, 0.0351e6, 0.349);
  final static public Element Yttrium =
    new TransitionMetal("Yttrium", "Y", 39, 89, 1.22, 1.62, 2.27, 1799.0, 3611.0,
                4.47, 0.3, 0.0166e6, 0.172);
  final static public Element Zirconium =
    new TransitionMetal("Zirconium", "Zr", 40, 91, 1.33, 1.45, 2.16, 2125.0, 4682.0,
                6.4, 0.27, 0.0236e6, 0.227);
  final static public Element Zinc =
    new TransitionMetal("Zinc", "Zn", 30, 65, 1.65, 1.25, 1.53, 692.73, 1180.0,
                7.14, 0.39, 0.166e6, 1.16);

}