/*--- formatted by Jindent 2.0b2, (www.c-lab.de/~jindent) ---*/

package com.hartmath.expression;

import com.hartmath.util.*;
import com.hartmath.lib.*;

import java.util.Random;
import java.io.*;
import java.math.*;
import java.io.Serializable;

/**
 * Immutable arbitrary-precision complex.
 * 
 * @see HFraction
 * 
 * @author Klaus Hartlage  <A HREF="mailto:khartlage@t-online.de">khartlage@t-online.de</A>
 */
public class HComplex implements HObject, HNumber {
	final HFraction	real;
	final HFraction	imag;

	/*
	 * protected HComplex(HFraction n) {
	 * real = n;
	 * imag = HInteger.B1;
	 * }
	 */

	/**
	 * Class Constructor.
	 * 
	 * 
	 * @param r
	 * @param i
	 * 
	 * @see
	 */
	public HComplex(HFraction r, HFraction i) {
		real = r;
		imag = i;
			//{{INIT_CONTROLS
		//}}
}

	/**
	 * Class Constructor.
	 * 
	 * 
	 * @param n
	 * @param d
	 * 
	 * @see
	 */
	public HComplex(HInteger n, HInteger d) {
		real = new HFraction(n.bigIntegerValue(), HInteger.B1);
		imag = new HFraction(d.bigIntegerValue(), HInteger.B1);
	}

	/**
	 * Class Constructor.
	 * 
	 * 
	 * @param r
	 * @param d
	 * 
	 * @see
	 */
	public HComplex(HFraction r, HInteger d) {
		real = r;
		imag = new HFraction(d.bigIntegerValue(), HInteger.B1);
	}

	/**
	 * Class Constructor.
	 * 
	 * 
	 * @param n
	 * @param d
	 * 
	 * @see
	 */
	public HComplex(HInteger n, HFraction d) {
		real = new HFraction(n.bigIntegerValue(), HInteger.B1);
		imag = d;
	}

	/*
	 * HComplex(byte[] barr) {
	 * real = new HFraction( barr );
	 * }
	 * HComplex(int i, byte[] barr) {
	 * real = new HFraction( i, barr );
	 * }
	 * HComplex(int i0, int i1, Random rand) {
	 * real = new HFraction( i0, i1, rand );
	 * }
	 * HComplex(int i, Random rand) {
	 * real = new HFraction(i, rand );
	 * }
	 */

	/**
	 * Class Constructor.
	 * 
	 * 
	 * @param strrn
	 * @param strrd
	 * @param strin
	 * @param strid
	 * 
	 * @see
	 */
	HComplex(String strrn, String strrd, String strin, String strid) {
		real = new HFraction(strrn, strrd);
		imag = new HFraction(strin, strid);
	}

	/**
	 * Class Constructor.
	 * 
	 * 
	 * @param strrn
	 * @param rin
	 * @param strrd
	 * @param rid
	 * @param strin
	 * @param iin
	 * @param strid
	 * @param iid
	 * 
	 * @see
	 */
	HComplex(String strrn, int rin, String strrd, int rid, String strin, 
				int iin, String strid, int iid) {
		real = new HFraction(strrn, rin, strrd, rid);
		imag = new HFraction(strin, iin, strid, iid);
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param b
	 * 
	 * @return
	 * 
	 * @see
	 */
	public boolean isEqual(HComplex b) {
		return (real.equals(b.real)) && (imag.equals(b.imag));
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param b
	 * 
	 * @return
	 * 
	 * @see
	 */
	public boolean isUnequal(HComplex b) {
		return !isEqual(b);
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @return
	 * 
	 * @see
	 */
	public boolean isZero() {
		return (real.equals(C.C0) && imag.equals(C.C0));
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @return
	 * 
	 * @see
	 */
	public boolean isNonZero() {
		return (isZero() != true);
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param obj
	 * 
	 * @return
	 * 
	 * @see
	 */
	public boolean matches(HObject obj) {
		return equals(obj);
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param obj
	 * 
	 * @return
	 * 
	 * @see
	 */
	public boolean equals(Object obj) {
		if (obj instanceof HComplex) {
			return (real.equals(((HComplex) obj).real)) 
					 && (imag.equals(((HComplex) obj).imag));
		}

		return false;
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param obj
	 * 
	 * @return
	 * 
	 * @see
	 */
	public boolean unequals(Object obj) {
		return !equals(obj);
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param obj
	 * 
	 * @return
	 * 
	 * @see
	 */
	public boolean less(Object obj) {
		if (obj instanceof HComplex) {
			if (real.equals(((HComplex) obj).real)) {
				return (imag.less(((HComplex) obj).imag));
			}

			return (real.less(((HComplex) obj).real)) 
					 && (imag.less(((HComplex) obj).imag));
		}
		if (obj instanceof HObject) {
			return (hierarchy() < ((HObject) obj).hierarchy());
		}

		return false;
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param obj
	 * 
	 * @return
	 * 
	 * @see
	 */
	public boolean greater(Object obj) {
		if (obj instanceof HObject) {
			return ((HObject) obj).less(this);
		}

		return false;
	}

	/**
	 * Returns true if <code>this</code> can be evaluated.
	 * <code>obj</code> is the evaluation result.
	 * @param obj         the (new) evaluated object
	 */
	public HObject evaluate() {
    //Context ct = ((ContextThread) Thread.currentThread()).getContext();
		if (C.numericFlag) {
			return new HDoubleComplex( real.doubleValue(), 
				                         imag.doubleValue() );
		}
		if (imag.isEqual(HFraction.F0)) {
			if (real.denom().isEqual(C.C1)) {
				return real.numer();
			}

			return real;
		}

		return null;
	}

	/**
	 * Returns true if <code>this</code> is (pattern-) matches the object
	 * <code>obj</code>
	 * @param obj         an object to compare with
	 * @param subst       the substitution for the object
	 */
	public HObject substitute(HObject obj, HObject subst) {
		if (equals(obj)) {
			return subst;
		}

		return this;
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @return
	 * 
	 * @see
	 */
	public HObject substitutePattern() {
		return this;
	}

	/**
	 * Returns an specific integer value for each class in the
	 * math-objects hierarchy
	 */
	final public int hierarchy() {
		return HObject.COMPLEXID; // 6;
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param os
	 * 
	 * @exception IOException
	 * 
	 * @see
	 */      /*
	public void writeTo(DataOutputStream os) throws IOException {
		os.writeByte(HObject.COMPLEXFILEID);
		real.writeTo(os);
		imag.writeTo(os);
	}          */

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @return
	 * 
	 * @see
	 */
	public HSymbol head() {
		return C.Complex;
	}

  public HObject apply(HObject head) {
    if (C.DEBUG) {                   // only in debug mode
      if (head==null) {
        throw new HThrowException(C.HeadFormatError, C.Symbol, C.Null);
      }
    }
    if (head instanceof HSymbol) {
      return C.EV( new HFunction((HSymbol)head, this) );
    }
    if ( (head instanceof HFunction) &&
         ((HFunction)head).isPureFunction() ) {
      HFunction l = (HFunction) ((HFunction)head).clone();
      l.add(this);
      return C.EV(l);
    }
    throw new HThrowException(C.HeadFormatError, C.Lambda, head);
  }
	/**
	 * Method Declaration.
	 * 
	 * 
	 * @return
	 * 
	 * @see
	 */
	public boolean isList() {
		return false;
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @return
	 * 
	 * @see
	 */
	public boolean isNumber() {
		return true;
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @return
	 * 
	 * @see
	 */
	public boolean isRational() {
		return false;
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param obj
	 * 
	 * @return
	 * 
	 * @see
	 */
	public boolean isMember(HObject obj) {
		return equals(obj);
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param sym
	 * 
	 * @return
	 * 
	 * @see
	 */
	public boolean isHeadMember(HSymbol sym) {
		return C.Complex.equals(sym);
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @return
	 * 
	 * @see
	 */
	public String toString() {
		if (real.equals(C.C0)) {
			if (imag.equals(C.C1)) {
				return "I";
			}
			if (imag.equals(C.CN1)) {
				return "-I";
			}

			return imag.toString() + "*I";
		}
		if (imag.equals(C.C1)) {
			return real.toString() + "+I";
		}
		if (imag.equals(C.CN1)) {
			return real.toString() + "-I";
		}
		if (imag.isNegative()) {
			return real.toString() + imag.toString() + "*I";
		} else {
			return real.toString() + "+" + imag.toString() + "*I";
		}
	}

  public void toStringBuffer(StringBuffer buf)
  {
    buf.append(toString());
  }
	/**
	 * Returns the absolute value.
	 * If the argument is not negative, the argument is returned.
	 * If the argument is negative, the negation of the argument is returned.
	 * 
	 * @return  the absolute value of the object.
	 * @see java.lang.Math
	 */
	public HObject abs() {
		if (imag.equals(C.C0)) {
			return real.abs();
		}
		if (real.equals(C.C0)) {
			return imag.abs();
		}

		HFraction	R = (HFraction) real.abs();
		HFraction	I = (HFraction) imag.abs();
		HFraction	sqr;

		if (I.less(R)) {
			sqr = imag.divide(real);
			sqr = sqr.multiply(sqr).add(HFraction.F1);

			return C.EV(C.Multiply.f(R, C.Pow.f(sqr, C.C1D2)));
		} else {
			sqr = real.divide(imag);
			sqr = sqr.multiply(sqr).add(HFraction.F1);

			return C.EV(C.Multiply.f(I, C.Pow.f(sqr, C.C1D2)));
		}
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param parm1
	 * 
	 * @return
	 * 
	 * @exception java.lang.ArithmeticException
	 * 
	 * @see
	 */
	public HComplex add(HComplex parm1) throws java.lang.ArithmeticException {
		return new HComplex(real.add(parm1.real), imag.add(parm1.imag));
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param parm1
	 * 
	 * @return
	 * 
	 * @exception java.lang.ArithmeticException
	 * 
	 * @see
	 */
	public HComplex divide(HComplex parm1) 
			  throws java.lang.ArithmeticException {
		HFraction	d = 
			parm1.real.multiply(parm1.real).add(parm1.imag.multiply(parm1.imag));

		return new HComplex(real.multiply(parm1.real).add(imag.multiply(parm1.imag)).divide(d), 
								  imag.multiply(parm1.real).subtract(real.multiply(parm1.imag)).divide(d));
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @return
	 * 
	 * @see
	 */
	public int hashCode() {
		return real.hashCode() + imag.hashCode();
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param parm1
	 * 
	 * @return
	 * 
	 * @see
	 */
	public HComplex multiply(HComplex parm1) {
		return new HComplex(real.multiply(parm1.real).subtract(imag.multiply(parm1.imag)), 
							real.multiply(parm1.imag).add(parm1.real.multiply(imag)));
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @return
	 * 
	 * @see
	 */
	public HComplex negate() {
		return new HComplex(real.negate(), imag.negate());
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param parm1
	 * 
	 * @return
	 * 
	 * @exception java.lang.ArithmeticException
	 * 
	 * @see
	 */
	public HComplex pow(int parm1) throws java.lang.ArithmeticException {
		int	temp = parm1;

		if ((parm1 == 0) && real.isEqual(HFraction.F0) 
				  && imag.isEqual(HFraction.F0)) {
			throw new java.lang.ArithmeticException();
		}

		HComplex res = new HComplex(HFraction.F1, HFraction.F0);

		if (parm1 < 0) {
			temp *= -1;
		}

		for (int i = 0; i < temp; i++) {
			res = res.multiply(this);
		}

		if (parm1 < 0) {
			HFraction	d = 
				res.real.multiply(res.real).add(res.imag.multiply(res.imag));

			return new HComplex(res.real.divide(d), res.imag.negate().divide(d));
		}

		return res;
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param parm1
	 * 
	 * @return
	 * 
	 * @see
	 */
	public HComplex subtract(HComplex parm1) {
		return new HComplex(real.subtract(parm1.real), 
								  imag.subtract(parm1.imag));
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @return
	 * 
	 * @see
	 */
	public int precedence() {
		if (real.equals(C.C0)) {
			if (imag.equals(C.C1)) {
				return 999999;
			}

			return 200;
		}
		if (real.less(C.C0)) {
			return 99;
		}

		return 100;
	}

	/**
	 * 0 for this == 0;
	 * +1 for re(this) > 0 or ( re(this)==0 and im(this)>0 );
	 * -1 for re(this) < 0 or ( re(this)==0 and im(this)<0 );
	 * 
	 * @see HSignedNumber.signum()
	 */
	public int compSignum() {
		int	i = real.signum();

		if (i == 0) {
			return imag.signum();
		}

		return i;
	}

  /**
	 * returns the conjugate complex of a number
	 */
  public HObject conjugate() {
    return new HComplex(real,imag.negate());
	}

	/**
	 * returns the real part of a number
	 */
	public HObject re() {
		return real;
	}

	/**
	 * returns the imaginary part of a number
	 */
	public HObject im() {
		return imag;
	}

/*
 * public HFraction im() {
 * return imag;
 * }
 * 
 * public HFraction re() {
 * return real;
 * }
 */
	//{{DECLARE_CONTROLS
	//}}
}



/*--- formatting done in "Sun Java Convention" style on 07-01-1999 ---*/