/*--- formatted by Jindent 2.1, (www.c-lab.de/~jindent) ---*/

package com.hartmath.expression;

import com.hartmath.util.*;
import com.hartmath.lib.*;

import java.lang.*;
import java.io.*;
import java.math.*;
import java.io.Serializable;

/**
 * HDoubleComplex represents a floating-point complex value.
 * @see HDouble
 * 
 * @author Klaus Hartlage  <A HREF="mailto:khartlage@t-online.de">khartlage@t-online.de</A>
 */
public class HDoubleComplex extends MathNumber implements HObject, HNumber
{
   private double                       re;
   private double                       im;
   public static final HDoubleComplex   DC0 = new HDoubleComplex(0.0, 0.0);
   public static final HDoubleComplex   DC1 = new HDoubleComplex(1.0, 0.0);
   public static final HDoubleComplex   DCN1 = new HDoubleComplex(-1.0, 0.0);
   public static final HDoubleComplex   DCI = new HDoubleComplex(0.0, 1.0);
   public static final HDoubleComplex   DCNI = new HDoubleComplex(0.0, -1.0);
   private static final HDoubleComplex  PI = new HDoubleComplex(Math.PI, 0.0);
   private static final HDoubleComplex  PI_I = new HDoubleComplex(0.0, 
   Math.PI);
   private static final HDoubleComplex  PI_2 = new HDoubleComplex(Math.PI 
   / 2.0, 0.0);
   private static final HDoubleComplex  MINUS_PI_2 = 
      new HDoubleComplex(-Math.PI / 2.0, 0.0);
   private static final HDoubleComplex  PI_2_I = new HDoubleComplex(0.0, 
   Math.PI / 2.0);
   private static final HDoubleComplex  MINUS_PI_2_I = new HDoubleComplex(0.0, 
   -Math.PI / 2.0);

   /**
    * Returns a new Double re initialized to the re represented by the
    * specified String.
    * @param s		the String to be parsed
    * @exception NumberFormatException If the String cannot be parsed.
    */

   // public static void valueOf(String s) throws NumberFormatException;

   /**
    * Return true if either the real or imaginary part is infinite.
    * @return true if <code>this</code> is infinite.
    */
   public boolean isInfinite()
   {
      return (Double.isInfinite(re) || Double.isInfinite(im));
   }

   /**
    * Constructs a Double wrapper for the specified double re.
    * @param re the initial re of the double
    */
   public HDoubleComplex(double rValue)
   {
      this.re = rValue;
      this.im = 0.0d;

      // {{INIT_CONTROLS
      // }}
   }

   /**
    * Class Constructor.
    * 
    * 
    * @param rValue
    * @param iValue
    * 
    * @see
    */
   public HDoubleComplex(double rValue, double iValue)
   {
      this.re = rValue;
      this.im = iValue;
   }

   /**
    * Class Constructor.
    * 
    * 
    * @param rValue
    * 
    * @see
    */
   public HDoubleComplex(HDouble rValue)
   {
      this.re = rValue.doubleValue();
      this.im = 0.0d;
   }

   /**
    * Class Constructor.
    * 
    * 
    * @param rValue
    * @param iValue
    * 
    * @see
    */
   public HDoubleComplex(HDouble rValue, HDouble iValue)
   {
      this.re = rValue.doubleValue();
      this.im = iValue.doubleValue();
   }

   /**
    * Returns true if this DoubleComplex is the special Not-a-Number (NaN) .
    */
   public boolean isNaN()
   {
      return Double.isNaN(re) || Double.isNaN(im);
   }

   /**
    * Returns a String representation of this DoubleComplex object.
    */
   public String toString()
   {
      if (re == 0.0d)
      {
         if (im == 1.0d)
         {
            return "I";
         }

         if (im < 0.0d)
         {
            return "I*" + "(" + String.valueOf(im) + ")";
         }

         return "I*" + String.valueOf(im);
      }

      if (im == 1.0d)
      {
         return Double.toString(re) + "+I";
      }

      if (im == -1.0d)
      {
         return String.valueOf(re) + "-I";
      }

      if (im < 0.0d)
      {
         return String.valueOf(re) + String.valueOf(im) + "*I";
      }
      else
      {
         return String.valueOf(re) + "+" + String.valueOf(im) + "*I";
      }
   }
   public void toStringBuffer(StringBuffer buf)
   {
     buf.append(toString());
   }
   /**
    * Returns a hashcode for this DoubleComplex.
    */
   public int hashCode()
   {
      return ((int) re) ^ ((int) im);
   }

   /**
    * Compares this object against the specified object.
    * <p>
    * <em>Note: To be useful in hashtables this method
    * considers two NaN double values to be equal. This
    * is not according to IEEE specification</em>
    * 
    * @param obj		the object to compare with
    * @return 		true if the objects are the same; false otherwise.
    */
   public boolean equals(Object obj)
   {
      return ((obj instanceof HDoubleComplex) 
              && (Double.doubleToLongBits(((HDoubleComplex) obj).re) == Double.doubleToLongBits(re)) 
              && (Double.doubleToLongBits(((HDoubleComplex) obj).im) 
                  == Double.doubleToLongBits(im)));
   }

   /**
    * Method Declaration.
    * 
    * 
    * @param b
    * 
    * @return
    * 
    * @see
    */
   public boolean isEqual(HDoubleComplex b)
   {
      return (re == b.re) && (im == b.im);
   }

   /**
    * Method Declaration.
    * 
    * 
    * @param b
    * 
    * @return
    * 
    * @see
    */
   public boolean isUnequal(HDoubleComplex b)
   {
      return !isEqual(b);
   }

   /**
    * Method Declaration.
    * 
    * 
    * @return
    * 
    * @see
    */
   public boolean isZero()
   {
      return (re == 0.0d) && (im == 0.0d);
   }

   /**
    * Method Declaration.
    * 
    * 
    * @return
    * 
    * @see
    */
   public boolean isNonZero()
   {
      return (re != 0.0d) || (im != 0.0d);
   }

   /**
    * Method Declaration.
    * 
    * 
    * @param obj
    * 
    * @return
    * 
    * @see
    */
   public boolean matches(HObject obj)
   {
      return equals(obj);
   }

   /**
    * Method Declaration.
    * 
    * 
    * @param obj
    * 
    * @return
    * 
    * @see
    */
   public boolean unequals(Object obj)
   {
      return !equals(obj);
   }

   /**
    * Method Declaration.
    * 
    * 
    * @param obj
    * 
    * @return
    * 
    * @see
    */
   public boolean less(Object obj)
   {
      if (obj instanceof HDoubleComplex)
      {
         if (re == ((HDoubleComplex) obj).re)
         {
            return (im < ((HDoubleComplex) obj).im);
         }

         return (re < ((HDoubleComplex) obj).re);
      }

      if (obj instanceof HObject)
      {
         return (hierarchy() < ((HObject) obj).hierarchy());
      }

      return false;
   }

   /**
    * Method Declaration.
    * 
    * 
    * @param obj
    * 
    * @return
    * 
    * @see
    */
   public boolean greater(Object obj)
   {
      if (obj instanceof HObject)
      {
         return ((HObject) obj).less(this);
      }

      return false;
   }

   /**
    * Returns true if <code>this</code> can be evaluated.
    * <code>obj</code> is the evaluation result.
    * @param obj         the (new) evaluated object
    */
   public HObject evaluate()
   {
      //Context   ct = ((ContextThread) Thread.currentThread()).getContext();

      if (!C.numericFlag)
      {
         C.numericFlag = true;

         if (im == 0.0d)
         {
            return new HDouble(re);
         }

         return this;
      }

      if (im == 0.0d)
      {
         return new HDouble(re);
      }

      return null;
   }

   /**
    * Returns true if <code>this</code> is (pattern-) matches the object
    * <code>obj</code>
    * @param obj         an object to compare with
    * @param subst       the substitution for the object
    */
   public HObject substitute(HObject obj, HObject subst)
   {
      if (equals(obj))
      {
         return subst;
      }

      return this;
   }

   /**
    * Method Declaration.
    * 
    * 
    * @return
    * 
    * @see
    */
   public HObject substitutePattern()
   {
      return this;
   }

   /**
    * Returns an specific integer value for each class in the
    * math-objects hierarchy
    */
   final public int hierarchy()
   {
      return HObject.DOUBLECOMPLEXID;           // 3;
   }

   /**
    * Method Declaration.
    * 
    * 
    * @param os
    * 
    * @exception IOException
    * 
    * @see
    */        /*
   public void writeTo(DataOutputStream os) throws IOException
   {
      os.writeByte(HObject.DOUBLECOMPLEXFILEID);
      os.writeDouble(re);
      os.writeDouble(im);
   }            */

   /**
    * Method Declaration.
    * 
    * 
    * @return
    * 
    * @see
    */
   public HSymbol head()
   {
      return C.DoubleComplex;
   }

   /**
    * Method declaration
    * 
    * 
    * @param head
    * 
    * @return
    * 
    * @see
    */
   public HObject apply(HObject head)
   {
      if (C.DEBUG)
      {         // only in debug mode
         if (head == null)
         {
            throw new HThrowException(C.HeadFormatError, C.Symbol, C.Null);
         }
      }

      if (head instanceof HSymbol)
      {
         return C.EV(new HFunction((HSymbol) head, this));
      }

      if ((head instanceof HFunction) && ((HFunction) head).isPureFunction())
      {
         HFunction      l = (HFunction) ((HFunction) head).clone();

         l.add(this);

         return C.EV(l);
      }

      throw new HThrowException(C.HeadFormatError, C.Lambda, head);
   }

   /**
    * Return <code>true</code> if this object ia a List
    * 
    * @return <code>false</code>
    * 
    * @see HFunction
    */
   public boolean isList()
   {
      return false;
   }

   /**
    * Method Declaration.
    * 
    * 
    * @return
    * 
    * @see
    */
   public boolean isNumber()
   {
      return true;
   }

   /**
    * Method Declaration.
    * 
    * 
    * @return
    * 
    * @see
    */
   public boolean isRational()
   {
      return false;
   }

   /**
    * Method Declaration.
    * 
    * 
    * @param obj
    * 
    * @return
    * 
    * @see
    */
   public boolean isMember(HObject obj)
   {
      return equals(obj);
   }

   /**
    * Method Declaration.
    * 
    * 
    * @param sym
    * 
    * @return
    * 
    * @see
    */
   public boolean isHeadMember(HSymbol sym)
   {
      return C.DoubleComplex.equals(sym);
   }

   /**
    * Returns the absolute value.
    * If the argument is not negative, the argument is returned.
    * If the argument is negative, the negation of the argument is returned.
    * 
    * @return  the absolute value of the object.
    * @see java.lang.Math
    */
   public HObject abs()
   {
      if (im == 0.0)
      {
         return new HDouble(Math.abs(re));
      }

      if (re == 0.0)
      {
         return new HDouble(Math.abs(im));
      }

      double    R = Math.abs(re);
      double    I = Math.abs(im);

      return (R >= I) ? new HDouble(R * Math.sqrt(1 + (im / re) * (im / re))) 
             : new HDouble(I * Math.sqrt(1 + (re / im) * (re / im)));
   }

   public double absReal()
   {
      if (im == 0.0)
      {
         return Math.abs(re);
      }

      if (re == 0.0)
      {
         return Math.abs(im);
      }

      double    R = Math.abs(re);
      double    I = Math.abs(im);

      return (R >= I) ? R * Math.sqrt(1 + (im / re) * (im / re))
             : I * Math.sqrt(1 + (re / im) * (re / im));
   }
   /**
    * Method Declaration.
    * 
    * 
    * @return
    * 
    * @see
    */
   public HDouble arg()
   {
      return new HDouble(argReal());
   }

   /**
    * Returns the argument of this complex number.
    */
   public double argReal()
   {
      if (re == 0.0)
      {
         if (im > 0.0)
            return Math.PI / 2.0;
         else if (im < 0.0)
            return -Math.PI / 2.0;
         else
            return 0.0;
      }
      else if (re < 0.0)
      {
         if (im >= 0.0)
            return Math.atan(im / re) + Math.PI;
         else
            return Math.atan(im / re) - Math.PI;
      }
      else
         return Math.atan(im / re);
   }

   /**
    * Method Declaration.
    * 
    * 
    * @param parm1
    * 
    * @return
    * 
    * @exception java.lang.ArithmeticException
    * 
    * @see
    */
   public HDoubleComplex add(HDoubleComplex parm1) 
      throws java.lang.ArithmeticException
   {
      return new HDoubleComplex(re + parm1.re, im + parm1.im);
   }

   /**
    * Returns the addition of this complex number with a real part.
    * @param d a double number
    */
   public HDoubleComplex add(final double d)
   {
      return new HDoubleComplex(re + d, im);
   }

   /**
    * Method Declaration.
    * 
    * 
    * @param parm1
    * 
    * @return
    * 
    * @exception java.lang.ArithmeticException
    * 
    * @see
    */                 /*
   public HDoubleComplex divide(HDoubleComplex parm1)
      throws java.lang.ArithmeticException
   {
      double    denom = parm1.re * parm1.re + parm1.im * parm1.im;

      // this exception will be thrown by the system:
      // if ( denom == 0.0) {
      // throw new java.lang.ArithmeticException();
      // }

      return new HDoubleComplex((re * parm1.re + im * parm1.im) / denom,
                                (parm1.re * im - re * parm1.im) / denom);
   }                     */
       /**
        * Returns the division of this complex number by another.
        * @param z a complex number
        * @exception ArithmeticException If divide by zero.
        */
        public HDoubleComplex divide(final HDoubleComplex z) {
                double denominator,real,imag,a;
                if(Math.abs(z.re)<Math.abs(z.im)) {
                        a=z.re/z.im;
                        denominator=z.re*a+z.im;
                        real=re*a+im;
                        imag=im*a-re;
                } else {
                        a=z.im/z.re;
                        denominator=z.re+z.im*a;
                        real=re+im*a;
                        imag=im-re*a;
                }
                return new HDoubleComplex(real/denominator,imag/denominator);
        }
   /**
    * Returns the division of this complex number by a scalar.
    * @param x a real number
    * @exception ArithmeticException If divide by zero.
    */
   public HDoubleComplex divide(final double x)
   {
      return new HDoubleComplex(re / x, im / x);
   }

   /**
    * Method Declaration.
    * 
    * 
    * @param parm1
    * 
    * @return
    * 
    * @see
    */
   public HDoubleComplex multiply(HDoubleComplex parm1)
   {
      return new HDoubleComplex(re * parm1.re - im * parm1.im, 
                                im * parm1.re + re * parm1.im);
   }

   /**
    * Method Declaration.
    * 
    * 
    * @param parm1
    * 
    * @return
    * 
    * @see
    */
   public HDoubleComplex subtract(HDoubleComplex parm1)
   {
      return new HDoubleComplex(re - parm1.re, im - parm1.im);
   }

   /**
    * Method Declaration.
    * 
    * 
    * @param d
    * 
    * @return
    * 
    * @see
    */
   public HDoubleComplex subtract(final double d)
   {
      return new HDoubleComplex(re - d, im);
   }

   /**
    * Method Declaration.
    * 
    * 
    * @return
    * 
    * @see
    */
   public HDoubleComplex exp()
   {
      double    magn = Math.exp(re);

      return new HDoubleComplex(magn * Math.cos(im), magn * Math.sin(im));
   }

   /**
    * Returns the natural logarithm (base e) of a complex number.
    */
   public HDoubleComplex log()
   {
      return new HDoubleComplex(Math.log(mod()), argReal());
   }

   /**
    * Return a new HDoubleComplex with the specified magnitude and angle.
    */
   public static HDoubleComplex polarToComplex(double magnitude, double angle)
   {
      if (magnitude < 0.0)
      {
         angle += Math.PI;
         magnitude = -magnitude;
      }
      else
      {
         if (magnitude == 0.0)
         {
            return DC0;
         }
      }

      return new HDoubleComplex(magnitude * Math.cos(angle), 
                                magnitude * Math.sin(angle));
   }
   
   /**
    * Returns this complex number raised to the power of another.
    * @param z a complex number
    */
   public HDoubleComplex pow(final HDoubleComplex z)
   {
      final double      thisMod = mod();
      final double      thisArg = argReal();
      final double      r = Math.pow(thisMod, z.re) 
                            / Math.exp(thisArg * z.im);
      final double      a = thisArg * z.re + Math.log(thisMod) * z.im;

      return polarToComplex(r, a);
   }

   /**
    * Returns this complex number raised to the power of a scalar.
    * @param x a real number
    */
   public HDoubleComplex pow(final double x)
   {
      return polarToComplex(Math.pow(mod(), x), argReal() * x);
   }

   /**
    * Returns the trigonometric sine of a complex angle.
    * @param z an angle that is measured in radians
    */
   public HDoubleComplex sin()
   {
      return new HDoubleComplex(Math.sin(re) * ExtraMath.cosh(im), 
                                Math.cos(re) * ExtraMath.sinh(im));
   }

   /**
    * Returns the trigonometric cosine of a complex angle.
    * @param z an angle that is measured in radians
    */
   public HDoubleComplex cos()
   {
      return new HDoubleComplex(Math.cos(re) * ExtraMath.cosh(im), 
                                -Math.sin(re) * ExtraMath.sinh(im));
   }

   /**
    * Returns the trigonometric tangent of a complex angle.
    */
   public HDoubleComplex tan()
   {
      final double      sinRe = Math.sin(re);
      final double      cosRe = Math.cos(re);
      final double      sinhIm = ExtraMath.sinh(im);
      final double      coshIm = ExtraMath.cosh(im);
      final double      denom = cosRe * cosRe * coshIm * coshIm 
                                + sinRe * sinRe * sinhIm * sinhIm;

      return new HDoubleComplex(sinRe * cosRe / denom, 
                                sinhIm * coshIm / denom);
   }

   /**
    * Returns the hyperbolic sine of a complex number.
    */
   public HDoubleComplex sinh()
   {
      return new HDoubleComplex(ExtraMath.sinh(re) * Math.cos(im), 
                                ExtraMath.cosh(re) * Math.sin(im));
   }

   /**
    * Returns the hyperbolic cosine of a complex number.
    * @param z a complex number
    */
   public HDoubleComplex cosh()
   {
      return new HDoubleComplex(ExtraMath.cosh(re) * Math.cos(im), 
                                ExtraMath.sinh(re) * Math.sin(im));
   }

   /**
    * Returns the hyperbolic tangent of a complex number.
    * @param z a complex number
    */
   public HDoubleComplex tanh()
   {
      final double      sinhRe = ExtraMath.sinh(re);
      final double      coshRe = ExtraMath.cosh(re);
      final double      sinIm = Math.sin(im);
      final double      cosIm = Math.cos(im);
      final double      denom = coshRe * coshRe * cosIm * cosIm 
                                + sinhRe * sinhRe * sinIm * sinIm;

      return new HDoubleComplex(sinhRe * coshRe / denom, 
                                sinIm * cosIm / denom);
   }

   /**
    * Return a new HDoubleComplex with conjugated value of
    * of <code>this</code>.
    */
   public HDoubleComplex conj()
   {
      if (im != 0.0)
      {         // avoid -1.0 * 0.0
         return new HDoubleComplex(re, -im);
      }
      else
      {
         return new HDoubleComplex(re, im);
      }
   }

   /**
    * Returns the square of this complex number.
    */
   public HDoubleComplex sqr() {
     return new HDoubleComplex(re*re-im*im,2.0d*re*im);
   }
   /**
    * Returns the square root of this complex number.
    */
   public HDoubleComplex sqrt() {
     return polarToComplex(Math.pow(mod(),0.5),argReal()*0.5);
   }
   
   /**
    * Returns the arc sine of a complex number.
    */                         
   public HDoubleComplex asin()
   {
      if (equals(DC1))
         return PI_2;
      else if (equals(DCN1))
         return MINUS_PI_2;
      else
      {

         // atan(z/sqrt(1-z*z))
         final double   oneMzzRe = 1.0 - re * re + im * im;
         final double   oneMzzIm = -2.0 * re * im;

         return divide((new HDoubleComplex(oneMzzRe, 
                                           oneMzzIm)).sqrt()).atan();
      }
   }        

   /**
    * Returns the arc cosine of a complex number.
    */
   public HDoubleComplex acos()
   {
      if (equals(DC1))
         return DC0;
      else if (equals(DCN1))
         return PI;
      else
      {

         // atan(-z/sqrt(1-z*z))+PI/2
         final double   oneMzzRe = 1.0 - re * re + im * im;
         final double   oneMzzIm = -2.0 * re * im;

         return negate().divide((new HDoubleComplex(oneMzzRe, oneMzzIm)).sqrt()).atan().addReal(Math.PI 
                                / 2.0);
      }
   }

   /**
    * Method Declaration.
    * 
    * 
    * @return
    * 
    * @see
    */

   /*
    * public HDoubleComplex atan()
    * {
    * double    temp = re * re;
    * 
    * return new HDoubleComplex(0.5d
    * Math.atan2(1.0 - temp - im * im, 2.0 * re),
    * 0.25d
    * Math.log(((1.0 + im) * (1.0
    * + im)
    * + temp) / ((1.0 - im)
    * (1.0 - im) + temp)));
    * }
    */

   /**
    * Returns the arc tangent of this complex number.
    */
   public HDoubleComplex atan()
   {

      // -i atanh(iz)
      return multiply(DCI).atanh().multiply(DCNI);
   }

   /**
    * Method Declaration.
    * 
    * 
    * @return
    * 
    * @see
    */
   public int precedence()
   {
      if (re == 0.0d)
      {
         if (im == 1.0d)
         {
            return 999999;
         }

         return 200;
      }

      if (re < 0.0d)
      {
         return 99;
      }

      return 100;
   }

   /**
    * 0 for this == 0;
    * +1 for re(this) > 0 or ( re(this)==0 and im(this)>0 );
    * -1 for re(this) < 0 or ( re(this)==0 and im(this)<0 );
    * 
    * @see HSignedNumber.signum()
    */
   public int compSignum()
   {
      int       i = new HDouble(re).signum();

      if (i == 0)
      {
         return new HDouble(im).signum();
      }

      return i;
   }

   /**
    * returns the conjugate complex of a number
    */
   public HObject conjugate()
   {
      return conj();
   }

   /**
    * returns the real part of a number
    */
   public HObject re()
   {
      return new HDouble(re);
   }

   /**
    * returns the imaginary part of a number
    */
   public HObject im()
   {
      return new HDouble(im);
   }

   /**
    * returns the real part of a number
    */
   public double real()
   {
      return re;
   }

   /**
    * returns the imaginary part of a number
    */
   public double imag()
   {
      return im;
   }

   /**
    * Returns the arc hyperbolic sine of a complex number.
    * @param z a complex number
    */
   public HDoubleComplex asinh()
   {
      if (equals(DCI))
         return PI_2_I;
      else if (equals(DCNI))
         return MINUS_PI_2_I;
      else
      {

         // log(z+sqrt(z*z+1))
         final double   zzPoneRe = re * re - im * im + 1.0;
         final double   zzPoneIm = 2.0 * re * im;

         return add((new HDoubleComplex(zzPoneRe, zzPoneIm)).sqrt()).log();
      }
   }

   /**
    * Returns the arc hyperbolic cosine of a complex number.
    * @param z a complex number
    */
   public HDoubleComplex acosh()
   {
      if (equals(DC1))
         return DC0;
      else if (equals(DCN1))
         return PI_I;
      else
      {

         // log(z+sqrt(z*z-1))
         final double   zzMoneRe = re * re - im * im - 1.0;
         final double   zzMoneIm = 2.0 * re * im;

         return add((new HDoubleComplex(zzMoneRe, zzMoneIm)).sqrt()).log();
      }
   }

   /**
    * Returns the arc hyperbolic tangent of a complex number.
    * @param z a complex number
    */
   public HDoubleComplex atanh()
   {

      // 1/2 log((1+z)/(1-z))
      final double      denom = im * im + re * re - 2.0 * re + 1.0;

      return new HDoubleComplex((1.0 - re * re - im * im) / denom, 
                                2.0 * im / denom).log().divide(2.0);
   }

   /**
    * Returns the negative of this complex number.
    */
   public HDoubleComplex negate()
   {
      return new HDoubleComplex(-re, -im);
   }

   /**
    * Returns the modulus of this complex number.
    */
   public double mod()
   {
      if (re == 0.0 && im == 0.0)
         return 0.0;

      final double      reAbs = Math.abs(re);
      final double      imAbs = Math.abs(im);

      if (reAbs < imAbs)
         return imAbs * Math.sqrt(1.0 + (re / im) * (re / im));
      else
         return reAbs * Math.sqrt(1.0 + (im / re) * (im / re));
   }

   /**
    * Returns the multiplication of this complex number by a scalar.
    * @param x a real number
    */
   public HDoubleComplex multiply(final double x)
   {
      return new HDoubleComplex(x * re, x * im);
   }

   /**
    * Returns the addition of this complex number with a real part.
    * @param real a real part
    */
   public HDoubleComplex addReal(final double real)
   {
      return new HDoubleComplex(re + real, im);
   }

   /**
    * Returns the subtraction of this complex number by a real part.
    * @param real a real part
    */
   public HDoubleComplex subtractReal(final double real)
   {
      return new HDoubleComplex(re - real, im);
   }

   /**
    * Returns the addition of this number and another.
    */
   public MathNumber add(final MathNumber x)
   {
      if (x instanceof HDoubleComplex)
         return add((HDoubleComplex) x);
      else if (x instanceof HDouble)
         return addReal(((HDouble) x).value());

         // else if(x instanceof MathInteger)
         // return addReal(((MathInteger)x).value());
      else
         throw new IllegalArgumentException("Number class not recognised by this method.");
   }

   /**
    * Returns the subtraction of this number and another.
    */
   public MathNumber subtract(final MathNumber x)
   {
      if (x instanceof HDoubleComplex)
         return subtract((HDoubleComplex) x);
      else if (x instanceof HDouble)
         return subtractReal(((HDouble) x).value());

         // else if(x instanceof MathInteger)
         // return subtractReal(((MathInteger)x).value());
      else
         throw new IllegalArgumentException("Number class not recognised by this method.");
   }

   /**
    * Returns the multiplication of this number and another.
    */
   public MathNumber multiply(final MathNumber x)
   {
      if (x instanceof HDoubleComplex)
         return multiply((HDoubleComplex) x);
      else if (x instanceof HDouble)
         return multiply(((HDouble) x).value());

         // else if(x instanceof MathInteger)
         // return multiply(((MathInteger)x).value());
      else
         throw new IllegalArgumentException("Number class not recognised by this method.");
   }

   /**
    * Returns the division of this number and another.
    */
   public MathNumber divide(final MathNumber x)
   {
      if (x instanceof HDoubleComplex)
         return divide((HDoubleComplex) x);
      else if (x instanceof HDouble)
         return divide(((HDouble) x).value());

         // else if(x instanceof MathInteger)
         // return divide(((MathInteger)x).value());
      else
         throw new IllegalArgumentException("Number class not recognised by this method.");
   }

}


