/*--- formatted by Jindent 2.0b2, (www.c-lab.de/~jindent) ---*/
package com.hartmath.expression;

import com.hartmath.lib.*;

import java.util.Random;
import java.io.*;
import java.math.*;
import java.io.Serializable;

/**
 * Immutable arbitrary-precision fraction.
 * @see HInteger
 * @see HComplex
 * @see HSignedNumber
 * 
 * @author Klaus Hartlage  <A HREF="mailto:khartlage@t-online.de">khartlage@t-online.de</A>
 */
public class HFraction implements HObject, HRational {
	BigInteger					numer;
	BigInteger					denom;

	/**
	 * Constant: HFraction value 1/1
	 */
	final static public HFraction F1 = new HFraction(HInteger.I1, HInteger.I1);

	/**
	 * Constant: HFraction value 0/1
	 */
	final static public HFraction F0 = new HFraction(HInteger.I0, HInteger.I1);

	/**
	 * Constant: HFraction value -1/1
	 */
	final static public HFraction FN1 = new HFraction(HInteger.IN1, 
			  HInteger.I1);

	/**
	 * Class Constructor.
	 * 
	 * 
	 * @param n
	 * @param d
	 *
	 * @see
	 */
	public HFraction(BigInteger n, BigInteger d) {
    if (d.compareTo(HInteger.B0) == 0) {
			throw new java.lang.ArithmeticException();
		}
    BigInteger  g = d.gcd(n);

    if (d.compareTo(HInteger.B0) == -1) {
			d = d.negate();
			n = n.negate();
		}

		if (g.compareTo(HInteger.B1) != 0) {
			n = n.divide(g);
			d = d.divide(g);
		}

		numer = n;
		denom = d;
			//{{INIT_CONTROLS
		//}}
}

	/**
	 * Class Constructor.
	 * 
	 * 
	 * @param n
	 * @param d
	 *
	 * @see
	 */
	public HFraction(HInteger n, HInteger d) {
		BigInteger  nm = n.bigIntegerValue();

		if (d.isZero()) {
			throw new java.lang.ArithmeticException();
		}

		BigInteger  dn = d.bigIntegerValue();

		if (dn.compareTo(HInteger.B0) == -1) {
			dn = dn.negate();
			nm = nm.negate();
		}

		BigInteger  g = dn.gcd(nm);

		if (g.compareTo(HInteger.B1) != 0) {
			nm = nm.divide(g);
			dn = dn.divide(g);
		}

		denom = dn;
		numer = nm;
	}

	/**
	 * Class Constructor.
	 * 
	 * 
	 * @param nbarr
	 * @param dbarr
	 *
	 * @see
	 */
	public HFraction(byte[] nbarr, byte[] dbarr) {
    BigInteger  n = new BigInteger(nbarr);
		BigInteger  d = new BigInteger(dbarr);
    BigInteger  g = d.gcd(n);

    if (d.compareTo(HInteger.B0) == -1) {
			d = d.negate();
			n = n.negate();
		}

		if (g.compareTo(HInteger.B1) != 0) {
			n = n.divide(g);
			d = d.divide(g);
		}
		numer = n;
		denom = d;
	}

	/*
	 * HFraction(int i, byte[] barr) {
	 * numer = new BigInteger( i, barr );
	 * }
	 * HFraction(int i0, int i1, Random rand) {
	 * numer = new BigInteger( i0, i1, rand );
	 * }
	 * HFraction(int i, Random rand) {
	 * numer = new BigInteger(i, rand );
	 * }
	 */

	/**
	 * Class Constructor.
	 * 
	 * 
	 * @param strn
	 * @param strd
	 *
	 * @see
	 */
	public HFraction(String strn, String strd) {
		BigInteger  nm = new BigInteger(strn);
		BigInteger  dn = new BigInteger(strd);

		if (dn.compareTo(HInteger.B0) == -1) {
			dn = dn.negate();
			nm = nm.negate();
		}

    BigInteger  g = dn.gcd(nm);

		if (g.compareTo(HInteger.B1) != 0) {
			nm = nm.divide(g);
			dn = dn.divide(g);
		}
    
		denom = dn;
		numer = nm;
	}

	/**
	 * Class Constructor.
	 * 
	 * 
	 * @param strn
	 * @param in
	 * @param strd
	 * @param id
	 *
	 * @see
	 */
	HFraction(String strn, int in, String strd, int id) {
		BigInteger  nm = new BigInteger(strn, in);
		BigInteger  dn = new BigInteger(strd, id);

		if (dn.compareTo(HInteger.B0) == -1) {
			dn = dn.negate();
			nm = nm.negate();
		}

    BigInteger  g = dn.gcd(nm);

		if (g.compareTo(HInteger.B1) != 0) {
			nm = nm.divide(g);
			dn = dn.divide(g);
		}
    
		denom = dn;
		numer = nm;
	}

	/**
	 * Returns a HFraction with the specified numerator / denominator.
	 * This factory is provided in preference to a (long, long) constructor
	 * because it allows for reuse
	 * of frequently used HFractions (like 1/2 and 1/3 ...), obviating the need for
	 * exported constants.
	 */
	public static HFraction valueOf(long numer, long denom) {
		return new HFraction(BigInteger.valueOf(numer), 
									BigInteger.valueOf(denom));
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param b
	 *
	 * @return
	 *
	 * @see
	 */
	public boolean isEqual(HFraction b) {
		return (numer.equals(b.numer)) && (denom.equals(b.denom));
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param b
	 *
	 * @return
	 *
	 * @see
	 */
	public boolean isUnequal(HFraction b) {
		return !isEqual(b);
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param b
	 *
	 * @return
	 *
	 * @see
	 */
	public boolean isLess(HFraction b) {
		BigInteger  b0 = numer.multiply(b.denom);
		BigInteger  b1 = denom.multiply(b.numer);

		return (b0.compareTo(b1) == -1);
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param b
	 *
	 * @return
	 *
	 * @see
	 */
	public boolean isGreater(HFraction b) {
		return b.less(this);
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @return
	 *
	 * @see
	 */
	public boolean isZero() {
		return (numer.compareTo(HInteger.B0) == 0);
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @return
	 *
	 * @see
	 */
	public boolean isNonZero() {
		return (numer.compareTo(HInteger.B0) != 0);
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param obj
	 *
	 * @return
	 *
	 * @see
	 */
	public boolean matches(HObject obj) {
		return equals(obj);
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param obj
	 *
	 * @return
	 *
	 * @see
	 */
	public boolean equals(Object obj) {
		if (obj instanceof HFraction) {
			return (numer.equals(((HFraction) obj).numer)) 
					 && (denom.equals(((HFraction) obj).denom));
		}
		if (obj instanceof HInteger) {
			if (denom.equals(HInteger.B1) 
					  && ((HInteger) obj).biggi.equals(numer)) {
				return true;
			}
		}

		return false;
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param obj
	 *
	 * @return
	 *
	 * @see
	 */
	public boolean unequals(Object obj) {
		return !equals(obj);
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param obj
	 *
	 * @return
	 *
	 * @see
	 */
	public boolean less(Object obj) {
		if (obj instanceof HInteger) {
			BigInteger  b = denom.multiply(((HInteger) obj).biggi);

			return (numer.compareTo(b) == -1);
		}
		if (obj instanceof HFraction) {
			BigInteger  b0 = numer.multiply(((HFraction) obj).denom);
			BigInteger  b1 = denom.multiply(((HFraction) obj).numer);

			return (b0.compareTo(b1) == -1);
		}
		if (obj instanceof HObject) {
			return (hierarchy() < ((HObject) obj).hierarchy());
		}

		return false;
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param obj
	 *
	 * @return
	 *
	 * @see
	 */
	public boolean greater(Object obj) {
		if (obj instanceof HObject) {
			return ((HObject) obj).less(this);
		}

		return false;
	}

	/**
	 * Returns an HObject if <code>this</code> can be evaluated.
	 * <code>obj</code> is the evaluation result.
	 * @param obj         the (new) evaluated object
	 */
	public HObject evaluate() {
    //Context ct = ((ContextThread) Thread.currentThread()).getContext();
		if (C.numericFlag) {
			return new HDouble(numer.doubleValue() / denom.doubleValue());
		}
    // don't reduce numerator and denominator here; reduce in the constructors:
    /*
		if (denom.compareTo(HInteger.B1) == 0) {
			return new HInteger(numer);
		}
    BigInteger gcd = numer.gcd(denom);
    if ( gcd.compareTo(HInteger.B1)!=0  ) {
      return new HFraction( numer.divide( gcd ), denom.divide( gcd ) );
    }
    if ( denom.compareTo(HInteger.B0)==-1 ) {
      return new HFraction( numer.negate(), denom.negate() );
    }
    */
		return null;
	}

	/**
	 * Returns true if <code>this</code> is (pattern-) matches the object
	 * <code>obj</code>
	 * @param obj         an object to compare with
	 * @param subst       the substitution for the object
	 */
	public HObject substitute(HObject obj, HObject subst) {
		if (equals(obj)) {
			return subst;
		}

		return this;
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @return
	 *
	 * @see
	 */
	public HObject substitutePattern() {
		return this;
	}

	/**
	 * Returns an specific integer value for each class in the
	 * math-objects hierarchy
	 */
	final public int hierarchy() {
		return HObject.FRACTIONID; // 5;
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param os
	 *
	 * @exception IOException
	 *
	 * @see
	 */    /*
	public void writeTo(DataOutputStream os) throws IOException {
		if (numer.equals(HInteger.B1) && denom.compareTo(HInteger.B2) >= 0 
				  && denom.compareTo(HInteger.B10) <= 0) {
			int	b = (denom.intValue() + 62) * -1;

			os.writeByte((byte) b);
		}
    else
    {
			byte[]	barr = numer.toByteArray();

			os.writeByte(HObject.FRACTIONFILEID);
			os.writeInt(barr.length);
			os.write(barr);

			barr = denom.toByteArray();

			os.writeInt(barr.length);
			os.write(barr);
		}
	}        */

	/**
	 * Method Declaration.
	 *
	 *
	 * @return
	 *
	 * @see
	 */
	public HSymbol head() {
		return C.Fraction;
	}

  public HObject apply(HObject head) {
    if (C.DEBUG) {                   // only in debug mode
      if (head==null) {
        throw new HThrowException(C.HeadFormatError, C.Symbol, C.Null);
      }
    }
    if (head instanceof HSymbol) {
      return C.EV( new HFunction((HSymbol)head, this) );
    }
    if ( (head instanceof HFunction) &&
         ((HFunction)head).isPureFunction() ) {
      HFunction l = (HFunction) ((HFunction)head).clone();
      l.add(this);
      return C.EV(l);
    }
    throw new HThrowException(C.HeadFormatError, C.Lambda, head);
  }
	/**
	 * Method Declaration.
	 * 
	 * 
	 * @return
	 *
	 * @see
	 */
	public boolean isList() {
		return false;
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @return
	 *
	 * @see
	 */
	public boolean isNumber() {
		return true;
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @return
	 *
	 * @see
	 */
	public boolean isRational() {
		return true;
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param obj
	 *
	 * @return
	 *
	 * @see
	 */
	public boolean isMember(HObject obj) {
		return equals(obj);
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param sym
	 *
	 * @return
	 *
	 * @see
	 */
	public boolean isHeadMember(HSymbol sym) {
		return C.Fraction.equals(sym);
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @return
	 *
	 * @see
	 */
	public boolean isPositive() {
		return (numer.compareTo(HInteger.B0) == 1);
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @return
	 *
	 * @see
	 */
	public boolean isNegative() {
		return (numer.compareTo(HInteger.B0) == -1);
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @return
	 *
	 * @see
	 */
	public boolean isNonNegative() {
		return (numer.compareTo(HInteger.B0) >= 0);
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @return
	 *
	 * @see
	 */
	public String toString() {
    StringBuffer buf = new StringBuffer(10);
    toStringBuffer(buf);
    return buf.toString();
	}

  public BigInteger toBigIntegerNumerator() {
    return numer;
	}

  public BigInteger toBigIntegerDenominator() {
    return denom;
	}

  public void toStringBuffer(StringBuffer buf)
  {
    if (denom.compareTo(HInteger.B1) == 0) {
      buf.append(numer.toString());
		}
    else {
      buf.append(numer.toString());
      buf.append('/');
      buf.append(denom.toString());
    }
  }
  /**
	 * Returns the absolute value.
	 * If the argument is not negative, the argument is returned.
	 * If the argument is negative, the negation of the argument is returned.
	 * 
	 * @return  the absolute value of the object.
   * @see java.lang.Math
	 */
	public HObject abs() {
    return new HFraction(numer.abs(), denom);
	}
	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param parm1
	 *
	 * @return
	 *
	 * @exception java.lang.ArithmeticException
	 *
	 * @see
	 */
	public HFraction add(HFraction parm1) 
			  throws java.lang.ArithmeticException {
		BigInteger  n = 
			(numer.multiply(parm1.denom)).add(denom.multiply(parm1.numer));
		BigInteger  d = denom.multiply(parm1.denom);

		if (d.compareTo(HInteger.B0) < 0) {
			d = d.negate();
			n = n.negate();
		}

		BigInteger  g = n.gcd(d);

		return new HFraction(n.divide(g), d.divide(g));
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param parm1
	 *
	 * @return
	 *
	 * @exception java.lang.ArithmeticException
	 *
	 * @see
	 */
	public HFraction divide(HFraction parm1) 
			  throws java.lang.ArithmeticException {
		BigInteger  n = numer.multiply(parm1.denom);
		BigInteger  d = denom.multiply(parm1.numer);

		if (d.compareTo(HInteger.B0) < 0) {
			d = d.negate();
			n = n.negate();
		}

		BigInteger  g = n.gcd(d);

		return new HFraction(n.divide(g), d.divide(g));
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @return
	 *
	 * @see
	 */
	public double doubleValue() {
		return (numer.doubleValue() / denom.doubleValue());
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @return
	 *
	 * @see
	 */
	public float floatValue() {
		return (numer.floatValue() / denom.floatValue());
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @return
	 *
	 * @see
	 */
	public int hashCode() {
		return numer.hashCode() + denom.hashCode();
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @return
	 *
	 * @see
	 */
	public int intValue() {
		if (denom.compareTo(HInteger.B1) != 0) {
			throw new java.lang.ArithmeticException();
		}

		return numer.intValue();
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @return
	 *
	 * @see
	 */
	public long longValue() {
		if (denom.compareTo(HInteger.B1) != 0) {
			throw new java.lang.ArithmeticException();
		}

		return numer.longValue();
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param parm1
	 *
	 * @return
	 *
	 * @see
	 */
	public HFraction max(HFraction parm1) {
		if (isLess(parm1)) {
			return parm1;
		}

		return this;
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param parm1
	 *
	 * @return
	 *
	 * @see
	 */
	public HFraction min(HFraction parm1) {
		if (isLess(parm1)) {
			return this;
		}

		return parm1;
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param parm1
	 *
	 * @return
	 *
	 * @see
	 */
	public HFraction multiply(HFraction parm1) {
		BigInteger  n = numer.multiply(parm1.numer);
		BigInteger  d = denom.multiply(parm1.denom);
		BigInteger  g = n.gcd(d);

		return new HFraction(n.divide(g), d.divide(g));
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param parm1
	 *
	 * @return
	 *
	 * @see
	 */
	public HFraction multiply(HInteger parm1) {
		BigInteger  n = numer.multiply(parm1.biggi);
		BigInteger  g = n.gcd(denom);

		return new HFraction(n.divide(g), denom.divide(g));
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @return
	 *
	 * @see
	 */
	public HFraction negate() {
		return new HFraction(numer.negate(), denom);
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param parm1
	 *
	 * @return
	 *
	 * @exception java.lang.ArithmeticException
	 *
	 * @see
	 */
	public HFraction pow(int parm1) throws java.lang.ArithmeticException {
		if (parm1 < 0) {
			parm1 *= -1;

			return new HFraction(denom.pow(parm1), numer.pow(parm1));
		}

		return new HFraction(numer.pow(parm1), denom.pow(parm1));
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param parm1
	 *
	 * @return
	 *
	 * @see
	 */
	public HFraction shiftLeft(int parm1) {
		return new HFraction(numer.shiftLeft(parm1), denom);
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param parm1
	 *
	 * @return
	 *
	 * @see
	 */
	public HFraction shiftRight(int parm1) {
		return new HFraction(numer.shiftRight(parm1), denom);
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param parm1
	 *
	 * @return
	 *
	 * @see
	 */
	public HFraction subtract(HFraction parm1) {
		BigInteger  n = 
			(numer.multiply(parm1.denom)).subtract(denom.multiply(parm1.numer));
		BigInteger  d = denom.multiply(parm1.denom);

		if (d.compareTo(HInteger.B0) < 0) {
			d = d.negate();
			n = n.negate();
		}

		BigInteger  g = n.gcd(d);

		return new HFraction(n.divide(g), d.divide(g));
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @return
	 *
	 * @see
	 */
	public int precedence() {
		if (numer.compareTo(HInteger.B0) == -1) {
			return 100;
		}

		return 999999;
	}

	/**
	 * 0 for this == 0
	 * +1 for re(this) > 0 or ( re(this)==0 and im(this)>0 )
	 * -1 for re(this) < 0 or ( re(this)==0 and im(this)<0 )
	 */
	public int compSignum() {
		return signum();
	}

	/**
	 * Returns the signum function of this fraction (i.e., -1, 0 or 1 as
	 * the value of this fraction is negative, zero or positive).
	 */
	public int signum() {
		return numer.signum();
	}

	/**
	 * Returns the smallest (closest to negative infinity)
	 * <code>HObject</code> value that is not less than <code>this</code> and is
	 * equal to a mathematical integer.
	 * 
	 * @return  the smallest (closest to negative infinity)
	 * <code>HObject</code> value that is not less than <code>this</code>
	 * and is equal to a mathematical integer.
	 */
	public HObject ceil() {
		BigInteger  result[] = numer.divideAndRemainder(denom);

		if (result[1].compareTo(HInteger.B0) <= 0) {
			return new HInteger(result[0]);
		}

		return new HInteger(result[0].add(HInteger.B1));
	}

	/**
	 * Returns the largest (closest to positive infinity)
	 * <code>HObject</code> value that is not greater than <code>this</code> and
	 * is equal to a mathematical integer.
	 * 
	 * @return  the largest (closest to positive infinity)
	 * <code>HObject</code> value that is not greater than <code>this</code>
	 * and is equal to a mathematical integer.
	 */
	public HObject floor() {
		BigInteger  result[] = numer.divideAndRemainder(denom);

		if (result[1].compareTo(HInteger.B0) >= 0) {
			return new HInteger(result[0]);
		}

		return new HInteger(result[0].subtract(HInteger.B1));
	}

	/**
	 * returns the numerator of a fraction
	 */
	public HInteger numer() {
		return new HInteger(numer);
	}

	/**
	 * returns the denominator of a fraction
	 */
	public HInteger denom() {
		return new HInteger(denom);
	}

  /**
	 * returns the conjugate complex of a number
	 */
	public HObject conjugate() {
		return this;
	}
  
	/**
	 * returns the real part of a fraction
	 */
	public HObject re() {
		return this;
	}

	/**
	 * returns the imaginary part of a fraction
	 */
	public HObject im() {
		return C.C0;
	}

	//{{DECLARE_CONTROLS
	//}}
}



/*--- formatting done in "Sun Java Convention" style on 06-18-1999 ---*/
