/*--- formatted by Jindent 2.0b2, (www.c-lab.de/~jindent) ---*/

package com.hartmath.expression;

import com.hartmath.util.*;
import com.hartmath.lib.*;

import java.util.Random;
import java.io.*;
import java.math.*;

/**
 * Immutable arbitrary-precision integers.
 * This class adopts much of his functionality from the BigInteger class
 * 
 * @see BigDecimals
 * @see HFraction
 * @see HRational
 * @see HSignedNumber
 * 
 * @author Klaus Hartlage  <A HREF="mailto:khartlage@t-online.de">khartlage@t-online.de</A>
 */
public class HInteger implements HObject, HRational {
	protected BigInteger			biggi;

	/**
	 * byte[] value 0
	 */
	final static public byte			I0[] = {
		0
	};

	/**
	 * byte[] value 1
	 */
	final static public byte			I1[] = {
		1
	};

	/**
	 * byte[] value 2
	 */
	final static public byte			I2[] = {
		2
	};

	/**
	 * byte[] value 3
	 */
	final static public byte			I3[] = {
		3
	};

	/**
	 * byte[] value 4
	 */
	final static public byte			I4[] = {
		4
	};

	/**
	 * byte[] value 5
	 */
	final static public byte			I5[] = {
		5
	};

	/**
	 * byte[] value 6
	 */
	final static public byte			I6[] = {
		6
	};

	/**
	 * byte[] value 7
	 */
	final static public byte			I7[] = {
		7
	};

	/**
	 * byte[] value 8
	 */
	final static public byte			I8[] = {
		8
	};

	/**
	 * byte[] value 9
	 */
	final static public byte			I9[] = {
		9
	};

	/**
	 * byte[] value 10
	 */
	final static public byte			I10[] = {
		10
	};

  /**
	 * byte[] value 100
	 */
	final static public byte			I100[] = {
		100
	};

	/**
	 * byte[] value -1
	 */
	final static public byte			IN1[] = {
		-1
	};

	/**
	 * byte[] value -2
	 */
	final static public byte			IN2[] = {
		-2
	};

	/**
	 * byte[] value -3
	 */
	final static public byte			IN3[] = {
		-3
	};

	/**
	 * byte[] value -4
	 */
	final static public byte			IN4[] = {
		-4
	};

	/**
	 * byte[] value -5
	 */
	final static public byte			IN5[] = {
		-5
	};

	/**
	 * byte[] value -6
	 */
	final static public byte			IN6[] = {
		-6
	};

	/**
	 * byte[] value -7
	 */
	final static public byte			IN7[] = {
		-7
	};

	/**
	 * byte[] value -8
	 */
	final static public byte			IN8[] = {
		-8
	};

	/**
	 * byte[] value -9
	 */
	final static public byte			IN9[] = {
		-9
	};

	/**
	 * byte[] value -10
	 */
	final static public byte			IN10[] = {
		-10
	};

	/**
	 * BigInteger value -3
	 */
	final static public BigInteger	BN3 = new BigInteger(IN3);

	/**
	 * BigInteger value -2
	 */
	final static public BigInteger	BN2 = new BigInteger(IN2);

	/**
	 * BigInteger value -1
	 */
	final static public BigInteger	BN1 = new BigInteger(IN1);

	/**
	 * BigInteger value 0
	 */
	final static public BigInteger	B0 = new BigInteger(I0);

	/**
	 * BigInteger value 1
	 */
	final static public BigInteger	B1 = new BigInteger(I1);

	/**
	 * BigInteger value 2
	 */
	final static public BigInteger	B2 = new BigInteger(I2);

	/**
	 * BigInteger value 3
	 */
	final static public BigInteger	B3 = new BigInteger(I3);

	/**
	 * BigInteger value 10
	 */
	final static public BigInteger	B10 = new BigInteger(I10);

	/**
	 * Class Constructor.
	 * 
	 * 
	 * @param b
	 * 
	 * @see
	 */
	public HInteger(BigInteger b) {
		this.biggi = b;
  }

	/**
	 * Translates a byte array containing the two's-complement representation
	 * of a (signed) integer into a HInteger.  The input array is assumed to
	 * be big-endian (i.e., the most significant byte is in the [0] position).
	 * (The most significant bit of the most significant byte is the sign bit.)
	 * The array must contain at least one byte or a NumberFormatException
	 * will be thrown.
	 */
	public HInteger(byte[] barr) {
		this.biggi = new BigInteger(barr);
	}

	/**
	 * Translates the sign-magnitude representation of an integer into a
	 * HInteger.  The sign is represented as an integer signum value (-1 for
	 * negative, 0 for zero, 1 for positive).  The magnitude is represented
	 * as a big-endian byte array (i.e., the most significant byte is in the
	 * [0] position).  An invalid signum value or a 0 signum value coupled
	 * with a nonzero magnitude will result in a NumberFormatException.
	 * A zero length magnitude array is permissible, and will result in
	 * in a value of 0 (irrespective of the given signum value).
	 */
	public HInteger(int i, byte[] barr) {
		this.biggi = new BigInteger(i, barr);
	}

	/**
	 * Class Constructor.
	 * 
	 * 
	 * @param i0
	 * @param i1
	 * @param rand
	 * 
	 * @see
	 */
	public HInteger(int i0, int i1, Random rand) {
		this.biggi = new BigInteger(i0, i1, rand);
	}

	/**
	 * Class Constructor.
	 * 
	 * 
	 * @param i
	 * @param rand
	 * 
	 * @see
	 */
	public HInteger(int i, Random rand) {
		this.biggi = new BigInteger(i, rand);
	}

	/**
	 * Translates a string containing an optional minus sign followed by a
	 * sequence of one or more decimal digits into a HInteger.  The
	 * character-to-digit mapping is provided by Character.digit.
	 * Any extraneous characters (including whitespace) will result in a
	 * NumberFormatException.
	 */
	public HInteger(String str) {
		this.biggi = new BigInteger(str);
	}

	/**
	 * Translates a string containing an optional minus sign followed by a
	 * sequence of one or more digits in the specified radix into a HInteger.
	 * The character-to-digit mapping is provided by Character.digit.
	 * Any extraneous characters (including whitespace), or a radix outside
	 * the range from Character.MIN_RADIX(2) to Character.MAX_RADIX(36),
	 * inclusive, will result in a NumberFormatException.
	 */
	public HInteger(String str, int i) {
		this.biggi = new BigInteger(str, i);
	}

	/**
	 * Returns a HInteger with the specified value.  This factory is provided
	 * in preference to a (long) constructor because it allows for reuse
	 * of frequently used HIntegers (like 0 and 1), obviating the need for
	 * exported constants.
	 */
	public static HInteger valueOf(long val) {
		return new HInteger(BigInteger.valueOf(val));
	}

	/**
	 * Checks if <code>this</code> is in the range [from, to]
	 */
	public boolean checkRange(long from, long to) {
		return ((biggi.compareTo(BigInteger.valueOf(from)) >= 0) 
				  && (biggi.compareTo(BigInteger.valueOf(to)) <= 0));
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param b
	 * 
	 * @return
	 * 
	 * @see
	 */
	public boolean isEqual(HInteger b) {
		return biggi.equals(b.biggi);
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param b
	 * 
	 * @return
	 * 
	 * @see
	 */
	public boolean isUnequal(HInteger b) {
		return !isEqual(b);
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param b
	 * 
	 * @return
	 * 
	 * @see
	 */
	public boolean isLess(HInteger b) {
		return (biggi.compareTo(b.biggi) == -1);
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param b
	 * 
	 * @return
	 * 
	 * @see
	 */
	public boolean isLessEqual(HInteger b) {
		return (biggi.compareTo(b.biggi) <= 0);
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param b
	 * 
	 * @return
	 * 
	 * @see
	 */
	public boolean isGreater(HInteger b) {
		return b.isLess(this);
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param b
	 * 
	 * @return
	 * 
	 * @see
	 */
	public boolean isGreaterEqual(HInteger b) {
		return b.isLessEqual(this);
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @return
	 * 
	 * @see
	 */
	public boolean isZero() {
		return (biggi.compareTo(B0) == 0);
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @return
	 * 
	 * @see
	 */
	public boolean isNonZero() {
		return (biggi.compareTo(B0) != 0);
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @return
	 * 
	 * @see
	 */
	public boolean isOdd() {
		return biggi.mod(B2).equals(B1);
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @return
	 * 
	 * @see
	 */
	public boolean isEven() {
		return biggi.mod(B2).equals(B0);
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param obj
	 * 
	 * @return
	 * 
	 * @see
	 */
	public boolean matches(HObject obj) {
		return equals(obj);
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param obj
	 * 
	 * @return
	 * 
	 * @see
	 */
	public boolean equals(Object obj) {
		if (obj instanceof HInteger) {
			return biggi.equals(((HInteger) obj).biggi);
		}

		return false;
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param obj
	 * 
	 * @return
	 * 
	 * @see
	 */
	public boolean unequals(Object obj) {
		return !equals(obj);
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param obj
	 * 
	 * @return
	 * 
	 * @see
	 */
	public boolean less(Object obj) {
		if (obj instanceof HFraction) {
			BigInteger  b = ((HFraction) obj).denom.multiply(biggi);

			return (b.compareTo(((HFraction) obj).numer) == -1);
		}
		if (obj instanceof HInteger) {
			return (biggi.compareTo(((HInteger) obj).biggi) == -1);
		}
		if (obj instanceof HObject) {
			return (hierarchy() < ((HObject) obj).hierarchy());
		}

		return false;
	}
  
	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param obj
	 * 
	 * @return
	 * 
	 * @see
	 */
	public boolean greater(Object obj) {
		if (obj instanceof HObject) {
			return ((HObject) obj).less(this);
		}

		return false;
	}

	/**
	 * Evaluate this. If numeric-evaluation is active, this (integer)
	 * will be translated to a HDouble.
	 * @return  <code>null</code> if the object could not be evaluated
	 */
	public HObject evaluate() {
    //Context ct = ((ContextThread) Thread.currentThread()).getContext();
		if (C.numericFlag) {
			return new HDouble(doubleValue());
		}

		return null;
	}

	/**
	 * Returns true if <code>this</code> (pattern-) matches the object
	 * <code>obj</code>
	 * @param obj         an object to compare with
	 * @param subst       the substitution for the object
	 */
	public HObject substitute(HObject obj, HObject subst) {
		if (equals(obj)) {
			return subst;
		}

		return this;
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @return
	 * 
	 * @see
	 */
	public HObject substitutePattern() {
		return this;
	}

	/**
	 * Returns an specific integer value for each class in the
	 * math-objects hierarchy
	 */
	final public int hierarchy() {
		return HObject.INTEGERID; // 4;
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param os
	 * 
	 * @exception IOException
	 * 
	 * @see
	 */      /*
	public void writeTo(DataOutputStream os) throws IOException {
		byte[]	barr = toByteArray();

		if (barr.length <= 8) {
			if (barr.length <= 4) {
				if (barr.length <= 2) {
					if (-5 <= intValue() && intValue() <= 5) {
						int	b = (intValue() + 58) * -1;

						// C.traceOut.append( new Integer( b ).toString() );

						os.writeByte((byte) b);
					} else {
						os.writeByte((byte) 22);
						os.writeShort((short) intValue());
					}
				} else {
					os.writeByte((byte) 21);
					os.writeInt(intValue());
				}
			} else {
				os.writeByte((byte) 20);
				os.writeLong(longValue());
			}
		} else {
			os.writeByte(HObject.INTEGERFILEID);
			os.writeInt(barr.length);
			os.write(barr);
		}
	}          */

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @return
	 * 
	 * @see
	 */
	public HSymbol head() {
		return C.Integer;
	}

  public HObject apply(HObject head) {
    if (C.DEBUG) {                   // only in debug mode
      if (head==null) {
        throw new HThrowException(C.HeadFormatError, C.Symbol, C.Null);
      }
    }
    if (head instanceof HSymbol) {
      return C.EV( new HFunction((HSymbol)head, this) );
    }
    if ( (head instanceof HFunction) &&
         ((HFunction)head).isPureFunction() ) {
      HFunction l = (HFunction) ((HFunction)head).clone();
      l.add(this);
      return C.EV(l);
    }
    throw new HThrowException(C.HeadFormatError, C.Lambda, head);
  }
	/**
	 * Method Declaration.
	 * 
	 * 
	 * @return
	 * 
	 * @see
	 */
	public boolean isList() {
		return false;
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @return
	 * 
	 * @see
	 */
	public boolean isNumber() {
		return true;
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @return
	 * 
	 * @see
	 */
	public boolean isRational() {
		return true;
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param obj
	 * 
	 * @return
	 * 
	 * @see
	 */
	public boolean isMember(HObject obj) {
		return equals(obj);
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param sym
	 * 
	 * @return
	 * 
	 * @see
	 */
	public boolean isHeadMember(HSymbol sym) {
		return C.Integer.equals(sym);
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @return
	 * 
	 * @see
	 */
	public boolean isPositive() {
		return (biggi.compareTo(B0) == 1);
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @return
	 * 
	 * @see
	 */
	public boolean isNegative() {
		return (biggi.compareTo(B0) == -1);
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @return
	 * 
	 * @see
	 */
	public boolean isNonNegative() {
		return (biggi.compareTo(B0) >= 0);
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @return
	 * 
	 * @see
	 */
	public BigInteger bigIntegerValue() {
		return biggi;
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @return
	 * 
	 * @see
	 */    /*
	public HInteger factorial() {
		BigInteger  result = new BigInteger(I1);

		if (biggi.compareTo(B0) == -1) {
			result = new BigInteger(IN1);

			for (BigInteger i = new BigInteger(IN2); i.compareTo(biggi) >= 0;
					  i = i.add(BN1)) {
				result = result.multiply(i);
			}
		} else {
			for (BigInteger i = new BigInteger(I2); i.compareTo(biggi) <= 0;
					  i = i.add(B1)) {
				result = result.multiply(i);
			}
		}

		HInteger i = new HInteger(result);

		// i.biggi = result;

		return i;
	}        */

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param certainty
	 * 
	 * @return
	 * 
	 * @see
	 */
	public boolean isProbablePrime(int certainty) {
		return biggi.isProbablePrime(certainty);
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @return
	 * 
	 * @see
	 */
	public String toString() {
		return biggi.toString();
	}

  public void toStringBuffer(StringBuffer buf)
  {
    buf.append( biggi.toString() );
  }
	/**
	 * Returns the absolute value.
	 * If the argument is not negative, the argument is returned.
	 * If the argument is negative, the negation of the argument is returned.
	 * 
	 * @return  the absolute value of the object.
   * @see java.lang.Math
	 */
	public HObject abs() {
    return new HInteger(biggi.abs());
	}
	/**
	 * Returns a HInteger whose value is (this + parm1).
	 */
	public HInteger add(HInteger parm1) throws java.lang.ArithmeticException {
		return new HInteger(biggi.add(parm1.biggi));
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param parm1
	 * 
	 * @return
	 * 
	 * @see
	 */
	public HInteger and(HInteger parm1) {
		return new HInteger(biggi.and(parm1.biggi));
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param parm1
	 * 
	 * @return
	 * 
	 * @see
	 */
	public HInteger andNot(HInteger parm1) {
		return new HInteger(biggi.andNot(parm1.biggi));
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param parm1
	 * 
	 * @return
	 * 
	 * @exception java.lang.ArithmeticException
	 * 
	 * @see
	 */
	public HInteger clearBit(int parm1) throws java.lang.ArithmeticException {
		return new HInteger(biggi.clearBit(parm1));
	}

	/**
	 * Returns a HInteger whose value is (this / parm1).
	 */
	public HInteger divide(HInteger parm1) 
			  throws java.lang.ArithmeticException {
		return new HInteger(biggi.divide(parm1.biggi));
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param parm1
	 * 
	 * @return
	 * 
	 * @exception java.lang.ArithmeticException
	 * 
	 * @see
	 */
	public HInteger[] divideAndRemainder(HInteger parm1) 
			  throws java.lang.ArithmeticException {
		BigInteger[]	b;

		b = biggi.divideAndRemainder(parm1.biggi);

		HInteger[]  i = new HInteger[2];

		i[0] = new HInteger(b[0]);
		i[1] = new HInteger(b[1]);

		// i[0].biggi = b[0];
		// i[1].biggi = b[1];

		return i;
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @return
	 * 
	 * @see
	 */
	public double doubleValue() {
		return biggi.doubleValue();
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param parm1
	 * 
	 * @return
	 * 
	 * @exception java.lang.ArithmeticException
	 * 
	 * @see
	 */
	public HInteger flipBit(int parm1) throws java.lang.ArithmeticException {
		return new HInteger(biggi.flipBit(parm1));
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @return
	 * 
	 * @see
	 */
	public float floatValue() {
		return biggi.floatValue();
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param parm1
	 * 
	 * @return
	 * 
	 * @see
	 */
	public HInteger gcd(HInteger parm1) {
		return new HInteger(biggi.gcd(parm1.biggi));
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @return
	 * 
	 * @see
	 */
	public int getLowestSetBit() {
		return biggi.getLowestSetBit();
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @return
	 * 
	 * @see
	 */
	public int hashCode() {
		return biggi.hashCode();
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @return
	 * 
	 * @see
	 */
	public int intValue() {
		return biggi.intValue();
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @return
	 * 
	 * @see
	 */
	public long longValue() {
		return biggi.longValue();
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param parm1
	 * 
	 * @return
	 * 
	 * @see
	 */
	public HInteger max(HInteger parm1) {
		return new HInteger(biggi.max(parm1.biggi));
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param parm1
	 * 
	 * @return
	 * 
	 * @see
	 */
	public HInteger min(HInteger parm1) {
		return new HInteger(biggi.min(parm1.biggi));
	}

	/**
	 * Returns a HInteger whose value is (this % parm1).
	 */
	public HInteger mod(HInteger parm1) {
		return new HInteger(biggi.mod(parm1.biggi));
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param parm1
	 * 
	 * @return
	 * 
	 * @exception java.lang.ArithmeticException
	 * 
	 * @see
	 */
	public HInteger modInverse(HInteger parm1) 
			  throws java.lang.ArithmeticException {
		return new HInteger(biggi.modInverse(parm1.biggi));
	}

  /**
    * Returns a BigInteger whose value is (this ** exponent) mod m.  (If 
    * exponent == 1, the returned value is (this mod m).  If exponent < 0, 
    * the returned value is the modular multiplicative inverse of 
    * (this ** -exponent).)  Throws an ArithmeticException if m <= 0. 
    */
	public HInteger modPow(HInteger exponent, HInteger m) {
		return new HInteger(biggi.modPow(exponent.biggi, m.biggi));
	}

	/**
	 * Returns a HInteger whose value is (this * parm1).
	 */
	public HInteger multiply(HInteger parm1) {
		return new HInteger(biggi.multiply(parm1.biggi));
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @return
	 * 
	 * @see
	 */
	public HInteger negate() {
		return new HInteger(biggi.negate());
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @return
	 * 
	 * @see
	 */
	public HInteger not() {
		return new HInteger(biggi.not());
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param parm1
	 * 
	 * @return
	 * 
	 * @see
	 */
	public HInteger or(HInteger parm1) {
		return new HInteger(biggi.or(parm1.biggi));
	}

	/**
	 * Returns a HInteger whose value is (this ^ parm1).
	 */
	public HInteger pow(int parm1) throws java.lang.ArithmeticException {
		return new HInteger(biggi.pow(parm1));
	}

	/**
	 * Returns a HInteger whose value close to this^(1 / parm1).
	 * this has to be a non-negative integer
	 */
	public HInteger root(int parm1) throws java.lang.ArithmeticException {
		if (isNegative()) {
			throw new java.lang.ArithmeticException();
		}

		final int		q1 = parm1 - 1;
		final HInteger q1Val = valueOf(q1);
		final HInteger parm1Val = valueOf(parm1);
		HInteger			xa = this;
		HInteger			xn = this.add(q1Val).divide(parm1Val);

		while (xn.isLess(xa)) {
			xa = xn;
			xn = 
				add(q1Val.multiply(xa.pow(parm1))).divide(parm1Val.multiply(xa.pow(q1)));
		}

		return xa;
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param parm1
	 * 
	 * @return
	 * 
	 * @exception java.lang.ArithmeticException
	 * 
	 * @see
	 */
	public HInteger remainder(HInteger parm1) 
			  throws java.lang.ArithmeticException {
		return new HInteger(biggi.remainder(parm1.biggi));
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param parm1
	 * 
	 * @return
	 * 
	 * @exception java.lang.ArithmeticException
	 * 
	 * @see
	 */
	public HInteger setBit(int parm1) throws java.lang.ArithmeticException {
		return new HInteger(biggi.setBit(parm1));
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param parm1
	 * 
	 * @return
	 * 
	 * @see
	 */
	public HInteger shiftLeft(int parm1) {
		return new HInteger(biggi.shiftLeft(parm1));
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param parm1
	 * 
	 * @return
	 * 
	 * @see
	 */
	public HInteger shiftRight(int parm1) {
		return new HInteger(biggi.shiftRight(parm1));
	}

	/**
	 * Returns a HInteger whose value is (this - parm1).
	 */
	public HInteger subtract(HInteger parm1) {
		return new HInteger(biggi.subtract(parm1.biggi));
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param parm1
	 * 
	 * @return
	 * 
	 * @exception java.lang.ArithmeticException
	 * 
	 * @see
	 */
	public boolean testBit(int parm1) throws java.lang.ArithmeticException {
		return biggi.testBit(parm1);
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @return
	 * 
	 * @see
	 */
	public byte[] toByteArray() {
		return biggi.toByteArray();
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param parm1
	 * 
	 * @return
	 * 
	 * @see
	 */
	public HInteger xor(HInteger parm1) {
		return new HInteger(biggi.xor(parm1.biggi));
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @return
	 * 
	 * @see
	 */
	public int precedence() {
		if (biggi.compareTo(B0) == -1) {
			return 100;
		}

		return 999999;
	}

  /**
	 * 0 for this == 0
	 * +1 for re(this) > 0 or ( re(this)==0 and im(this)>0 )
	 * -1 for re(this) < 0 or ( re(this)==0 and im(this)<0 )
	 */
	public int compSignum() {
		return signum();
	}

	/**
	 * Returns the signum function of this number (i.e., -1, 0 or 1 as
	 * the value of this number is negative, zero or positive).
	 */
	public int signum() {
		return biggi.signum();
	}

	/**
	 * Returns the smallest (closest to negative infinity)
	 * <code>HObject</code> value that is not less than <code>this</code> and is
	 * equal to a mathematical integer.
	 * 
	 * @return  the smallest (closest to negative infinity)
	 * <code>HObject</code> value that is not less than <code>this</code>
	 * and is equal to a mathematical integer.
	 */
	public HObject ceil() {
		return this;
	}

	/**
	 * Returns the largest (closest to positive infinity)
	 * <code>HObject</code> value that is not greater than <code>this</code> and
	 * is equal to a mathematical integer.
	 * 
	 * @return  the largest (closest to positive infinity)
	 * <code>HObject</code> value that is not greater than <code>this</code>
	 * and is equal to a mathematical integer.
	 */
	public HObject floor() {
		return this;
	}

	/**
	 * returns the numerator of a number
	 */
	public HInteger numer() {
		return this;
	}

	/**
	 * returns the denominator of a number
	 */
	public HInteger denom() {
		return C.C1;
	}

  /**
	 * returns the conjugate complex of a number
	 */
	public HObject conjugate() {
		return this;
	}
  
	/**
	 * returns the real part of a number
	 */
	public HObject re() {
		return this;
	}

	/**
	 * returns the imaginary part of a number
	 */
	public HObject im() {
		return C.C0;
	}

	//{{DECLARE_CONTROLS
	//}}
}



/*--- formatting done in "Sun Java Convention" style on 06-18-1999 ---*/