/*--- formatted by Jindent 2.0b2, (www.c-lab.de/~jindent) ---*/
package com.hartmath.expression;

import com.hartmath.lib.*;
import com.hartmath.mapping.*;

import java.util.*;
import java.lang.*;
import java.io.*;
import java.text.MessageFormat;
/**
 * Matrix wrapper for a HFunction-matrix representation
 *
 * @author Klaus Hartlage  <A HREF="mailto:khartlage@t-online.de">khartlage@t-online.de</A>
 */
public class HMatrix
{
	HFunction matrix;

  /**
	 * Constructs a new HMatrix 
	 */
	public HMatrix(HFunction f) {
    matrix = f;
  }

  public int size()
  {
    return matrix.size();
  }
  /**
    * Make a shallow copy of this matrix.
    * mClone() assumes that isMatrix() was called before,
    * so that no internal type checking is needed !
    * 
    * The structure of the matrix-function will be copied;
    * The atomic objects of the row-lists will not be "cloned".
    */
   final public HMatrix copy(int p)
   {
      HFunction m = new HFunction(matrix.head(), matrix.size());

      m.prop = p;

      for (int i = 0; i < matrix.size(); i++)
      {
         m.set(i, (HObject)((HFunction) matrix.get(i)).clone());
      }

      return new HMatrix(m);
   }

  /**
   *  get the internal representation of this polynomial
   */
  final public HFunction getFunction()
  {
    return matrix;
  }
   public HFunction get(int row)
   {
      return (HFunction) matrix.get(row);
   }

   public HFunction set(int row, HFunction f)
   {
      HFunction old = (HFunction) matrix.a[row];
		  matrix.a[row] = f;
		  return old;
   }
   /**
    * Returns the HObject at the specified row and column of a given matrix.
    * mAt() assumes that isMatrix() was called before,
    * so that no internal type checking is needed !
    * 
    * @param      the row and column of the matrix
    * @return     the HObject at the specified index.
    */
   public HObject get(int row, int col)
   {
      return  ((HFunction) matrix.get(row)).get(col);
   }

   /**
    * Sets the HObject at the specified <code>row / col</code> of this
    * HFunction to be the specified HObject. The previous HObject at that
    * position is discarded.
    * mSetAt() assumes that the rows fo the matrix are defined as HFunction-Lists
    * so that no internal type checking is needed !
    * 
    * @param      obj     what the HObject is to be set to.
    * @param      row / column   the index of the argument in this matrix
    */
   public void set(HObject obj, int row, int col)
   {
      ((HFunction) matrix.get(row)).set(col, obj);
   }
   
   /**
    * Method declaration
    * 
    * 
    * @return
    * 
    * @see
    */
   public double[][] toDoubleMatrixArray()
   {
      if (matrix.len == 0)
      {
         throw new HThrowException(C.Error, C.List, new HString("size = 0"));
      }

      int       cols = 0;

      if (matrix.a[0] instanceof HFunction)
      {
         cols = ((HFunction) matrix.a[0]).len;
      }
      else
      {
         throw new HThrowException(C.Error, C.List, 
                                   new HString("not a matrix"));
      }

      HObject   temp;
      HFunction lst;
      double    arr[][] = new double[matrix.len][cols];

      for (int i = 0; i < matrix.len; i++)
      {
         if (matrix.a[i] instanceof HFunction)
         {
            lst = (HFunction) matrix.a[i];
         }
         else
         {
            throw new HThrowException(C.Error, C.List, 
                                      new HString("not a matrix"));
         }

         if (cols != lst.len)
         {
            throw new HThrowException(C.Error, C.List, 
                                      new HString("different row lengths in matrix"));
         }

         for (int j = 0; j < cols; j++)
         {
            temp = C.NEV( lst.a[j]);

            if (temp instanceof HDouble)
            {
               arr[i][j] = ((HDouble) temp).value();
            }
            else
            {
               throw new HThrowException(C.Error, C.List, temp);
                    //                     new HString("only numerical values allowed."));
            }
         }
      }

      return arr;
   }

   /**
    * Method declaration
    *
    *
    * @return
    *
    * @see
    */
   public HDoubleComplex[][] toDoubleComplexMatrixArray()
   {
      if (matrix.len == 0)
      {
         throw new HThrowException(C.Error, C.List, new HString("size = 0"));
      }

      int       cols = 0;

      if (matrix.a[0] instanceof HFunction)
      {
         cols = ((HFunction) matrix.a[0]).len;
      }
      else
      {
         throw new HThrowException(C.Error, C.List,
                                   new HString("not a matrix"));
      }

      HObject           temp;
      HFunction         lst;
      HDoubleComplex    arr[][] = new HDoubleComplex[matrix.len][cols];

      for (int i = 0; i < matrix.len; i++)
      {
         if (matrix.a[i] instanceof HFunction)
         {
            lst = (HFunction) matrix.a[i];
         }
         else
         {
            throw new HThrowException(C.Error, C.List, 
                                      new HString("not a matrix"));
         }

         if (cols != lst.len)
         {
            throw new HThrowException(C.Error, C.List, 
                                      new HString("different row lengths in matrix"));
         }

         for (int j = 0; j < cols; j++)
         {
            temp = C.NEV( lst.a[j]);

            if (temp instanceof HDouble)
            {
               arr[i][j] = new HDoubleComplex((HDouble) temp);
            }
            else
            {
               if (temp instanceof HDoubleComplex)
               {
                  arr[i][j] = (HDoubleComplex) temp;
               }
               else
               {
                  throw new HThrowException(C.Error, C.List, 
                                            new HString("only numerical values allowed."));
               }
            }
         }
      }

      return arr;
   }
}



