/*--- formatted by Jindent 2.0b2, (www.c-lab.de/~jindent) ---*/
package com.hartmath.expression;

import com.hartmath.lib.*;
import com.hartmath.mapping.*;

import java.util.*;
import java.lang.*;
import java.io.*;

/**
 * Pattern-matching Symbol
 * A pattern is a placeholder on the left-hand-side (lhs)
 * of a transformation rule. If all pattern matches to an object in the lhs,
 * the right-hand-side will be evaluated with the matched values of the patterns.
 * 
 * @see HSymbol
 * @see PatternMatching
 * 
 * @author Klaus Hartlage  <A HREF="mailto:khartlage@t-online.de">khartlage@t-online.de</A>
 */
public class HPattern implements HObject {
	HSymbol  symb;
	HObject  headCheck;

	/**
	 * Constructs a new HPattern from the specified HSymbol.
	 * 
	 * @param sym the associated symbol for this pattern
	 */
	public HPattern(HSymbol sym) {
		symb = sym;
			//{{INIT_CONTROLS
		//}}
}

	/**
	 * Constructs a new HPattern from the specified HSymbol.
	 * 
	 * @param sym    the associated symbol for this pattern
	 * @param check  the HSymbol which the pattern should have as head() in
	 * the left-hand-side of the pattern-matching rule
	 */
	public HPattern(HSymbol sym, HSymbol check) {
		symb = sym;
		headCheck = check;
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @see
	 */
	public void initValue() {
		symb.patternValue.push(null);
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @see
	 */
	public void resetValue() {
		symb.patternValue.pop();
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param val
	 *
	 * @see
	 */
	public void setValue(HObject val) {
		symb.patternValue.pop();
		symb.patternValue.push(val);
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @return
	 *
	 * @see
	 */
	public HObject getValue() {
		return (HObject) symb.patternValue.peek();
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @return
	 *
	 * @see
	 */
	public boolean isEmpty() {
		return symb.patternValue.isEmpty();
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @return
	 *
	 * @see
	 */
	public String toString() {
		if (headCheck != null) {
			return symb.toString() + "_" + headCheck.toString();
		}

		return symb.toString() + "_";
	}
  public void toStringBuffer(StringBuffer buf)
  {
     buf.append( toString() );
  }
	/**
	 * Method Declaration.
	 * 
	 * 
	 * @return
	 *
	 * @see
	 */
	public int hashCode() {

		// has to be a constant, see HFunction.patternHashCode()

		return 0x13131313;
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param obj
	 *
	 * @return
	 *
	 * @see
	 */
	public boolean matches(HObject obj) {
		if (C.DEBUG) {   /*
			if (C.traceFlag && C.traceExtendedFlag) {
				C.traceOut.append("TRACE(try pattern)::");
				C.traceOut.append(toString());
				C.traceOut.append(" <-> ");
				C.traceOut.append(obj.toString());
				C.traceOut.append("}\n");
			}                */
		}
		if ((obj instanceof HFunction) && (((HFunction) obj).size() == 1)) {
			if ((((HFunction) obj).head().getAttributes() & HSymbol.ONEIDENTITY) 
					  == HSymbol.ONEIDENTITY) {
				obj = ((HFunction) obj).get(0);
			}
		}

		HObject  topObj = getValue();

		if (topObj == null) {
			if (headCheck != null) {
				if (headCheck instanceof HSymbol) {
          if (((HSymbol) headCheck).functionEval instanceof ELoadClass)
          {
            ELoadClass.loadClass(((HSymbol) headCheck));
          }
					if (((HSymbol) headCheck).functionEval instanceof EB1Arg) {
						if (!((EB1Arg) ((HSymbol) headCheck).functionEval).evalArg1(obj)) {
							return false;
						}
					} else {
						if (((HSymbol) headCheck).functionEval 
								  instanceof EB1ObjectArg) {
							if (!((EB1ObjectArg) ((HSymbol) headCheck).functionEval).evalArg1(obj)) {
								return false;
							}
						} else {
							if (((HSymbol) headCheck).unequals(obj.head())) {
								return false;
							}
						}
					}
				}
				if (headCheck instanceof HFunction) {
					if (((HFunction) headCheck).size() != 1) {
						return false;
					}
          if (((HFunction) headCheck).head().functionEval instanceof ELoadClass)
          {
            ELoadClass.loadClass(((HFunction) headCheck).head());
          }
					if (((HFunction) headCheck).head().functionEval
							  instanceof EB2Arg) {
						if (!((EB2Arg) ((HFunction) headCheck).head().functionEval).evalArg2(obj, 
								  ((HFunction) headCheck).get(0))) {
							return false;
						}
					}
				}
			}

			setValue(obj);

			if (C.DEBUG) { /*
				if (C.traceFlag && C.traceExtendedFlag) {
					C.traceOut.append("TRACE(try pattern):: <TRUE>");
				}              */
			}

			return true;
		}
		if (topObj.equals(obj)) {
			if (C.DEBUG) {   /*
				if (C.traceFlag && C.traceExtendedFlag) {
					C.traceOut.append("TRACE(try pattern):: <TRUE>");
				}                */
			}

			return true;
		}

		return false;
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param obj
	 *
	 * @return
	 *
	 * @see
	 */
	public boolean equals(Object obj) {
		if (obj instanceof HPattern) {
			return symb.equals(((HPattern) obj).symb);
		}

		return false;
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param obj
	 *
	 * @return
	 *
	 * @see
	 */
	public boolean unequals(Object obj) {
		return !equals(obj);
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param obj
	 *
	 * @return
	 *
	 * @see
	 */
	public boolean less(Object obj) {
		if (obj instanceof HPattern) {
			return symb.less(((HPattern) obj).symb);
		}
		if (obj instanceof HObject) {
			return (hierarchy() < ((HObject) obj).hierarchy());
		}

		return false;
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param obj
	 *
	 * @return
	 *
	 * @see
	 */
	public boolean greater(Object obj) {
		if (obj instanceof HObject) {
			return ((HObject) obj).less(this);
		}

		return false;
	}

	/**
	 * HPattern are always left unevaluated
	 * 
	 * @return null
	 */
	public HObject evaluate() {
		return null;
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param obj
	 * @param subst
	 *
	 * @return
	 *
	 * @see
	 */
	public HObject substitute(HObject obj, HObject subst) {
		if (equals(obj)) {
			return subst;
		}

		return this;
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @return
	 *
	 * @see
	 */
	public HObject substitutePattern() {
		return this;
	}

	/**
	 * Returns an specific integer value for each class in the
	 * HObjects class-hierarchy
	 */
	final public int hierarchy() {
		return HObject.PATTERNID; // 65;
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param os
	 *
	 * @exception IOException
	 *
	 * @see
	 */   /*
	public void writeTo(DataOutputStream os) throws IOException {
		if (headCheck != null) {
			os.writeByte(HObject.PATTERNFILEID);
			symb.writeTo(os);
			headCheck.writeTo(os);
		} else {
			if ((symb.str.length() == 1) && (symb.str.charAt(0) >= 'a')
					  && (symb.str.charAt(0) <= 'z')) {
				int	b = ((symb.str.charAt(0) - 'a' + 26)+1) * -1;

				// C.traceOut.append( new Integer( b ).toString() );

				os.writeByte((byte) b);
			} else {
				os.writeByte((byte) 66);
				symb.writeTo(os);
			}
		}
	}       */
  final public HSymbol getSymbol()
  {
    return symb;
  }
  final public HObject getCheckFunction()
  {
    return headCheck;
  }
	/**
	 * Method Declaration.
	 * 
	 * 
	 * @return
	 *
	 * @see
	 */
	public HSymbol head() {
		return C.Pattern;
	}

  public HObject apply(HObject head) {
    if (C.DEBUG) {                   // only in debug mode
      if (head==null) {
        throw new HThrowException(C.HeadFormatError, C.Symbol, C.Null);
      }
    }
    if (head instanceof HSymbol) {
      return C.EV( new HFunction((HSymbol)head, this) );
    }
    if ( (head instanceof HFunction) &&
         ((HFunction)head).isPureFunction() ) {
      HFunction l = (HFunction) ((HFunction)head).clone();
      l.add(this);
      return C.EV(l);
    }
    throw new HThrowException(C.HeadFormatError, C.Lambda, head);
  }
	/**
	 * Method Declaration.
	 * 
	 * 
	 * @return
	 *
	 * @see
	 */
	public boolean isList() {
		return false;
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @return
	 *
	 * @see
	 */
	public boolean isNumber() {
		return false;
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @return
	 *
	 * @see
	 */
	public boolean isRational() {
		return false;
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param obj
	 *
	 * @return
	 *
	 * @see
	 */
	public boolean isMember(HObject obj) {
		return equals(obj);
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param sym
	 *
	 * @return
	 *
	 * @see
	 */
	public boolean isHeadMember(HSymbol sym) {
		return head().equals(sym);
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @return
	 *
	 * @see
	 */
	public int precedence() {
		return 999999;
	}
}



/*--- formatting done in "Sun Java Convention" style on 06-18-1999 ---*/
