/*--- formatted by Jindent 2.0b2, (www.c-lab.de/~jindent) ---*/
package com.hartmath.expression;

import com.hartmath.lib.*;
import com.hartmath.mapping.*;

import java.util.*;
import java.lang.*;
import java.io.*;
import java.text.MessageFormat;
/**
 * Polynomial wrapper for a HFunction-polynomial representation
 *
 * @author Klaus Hartlage  <A HREF="mailto:khartlage@t-online.de">khartlage@t-online.de</A>
 */
public class HPolynomial  implements Cloneable
{
	HFunction poly;
  HObject variable;

  /**
	 * Constructs a new HPolynomial from the specified object and variable
	 */
	private HPolynomial(HObject pVar, int degree) {
    poly = new HFunction(C.List, degree+1);
    variable = pVar;
  }
  /**
	 * Constructs a new HPolynomial from the specified object and variable 
	 */
	public HPolynomial(HObject obj, HObject pVar) {
    poly = new HFunction(C.List);
    variable = pVar;
    if (! polyList(obj)) {
      Object[] arguments = { obj };
      String mess = MessageFormat.format(
        "Input: {0} is not a polynomial.", arguments);
      throw new HThrowException(C.Error, C.Error, new HString(mess));
    }
  }

  final public Object clone()
  {
    HPolynomial pol;
    try {
      pol = (HPolynomial) super.clone();
      pol.poly = (HFunction) poly.clone();
      pol.variable = variable;
    } catch (CloneNotSupportedException e) {
			throw new Error();
		}
    return pol;
  }
  /**
   *  get the internal representation of this polynomial
   */
  final public HFunction getFunction()
  {
    return poly;
  }
   /**
    *  get the coefficient at a given index
    */
   public HObject getCoeff(int index)
   {
    //  return ((HFunction) poly.a[index]).a[1];
     return poly.get(index, 1);
   }
   /**
    *  get the exponent at a given index
    */
   public HObject getExp(int index)
   {
     // return ((HFunction) poly.a[index]).a[0];
      return poly.get(index, 0);
   }
   /**
    *  set the coefficient at a given index
    */
   public void setCoeff(HObject obj, int index)
   {
      if (poly.a[index] == null)
      {
         poly.set(index, C.List.f(HInteger.valueOf(index), obj));
      }
      else
      {
         ((HFunction) poly.a[index]).set(1, obj);
      }
   }
   /**
    *  the size of this polynomial (number of terms)
    */
   public int size()
   {
      return poly.size();
   }

   /**
    *  the degree of this polynomial
    */
   public int degree()
   {
      return ((HInteger) getExp(size()-1)).intValue();
   }

   /**
    *  is this polynomial a 0-polynomial ?
    */
   public boolean isNull()
   {
      if ((size() == 1) && (getCoeff(0).equals(C.C0)))
      {
         return true;
      }

      return false;
   }
   /**
    *  extend the internal representation; create entries for all 0-coefficients
    */
   public HPolynomial blowUp(int degree)
   {
      HPolynomial newPoly = new HPolynomial(variable, degree);
      int         indx = 0;

      for (int i = 0; i < newPoly.size(); i++)
      {
         if (indx >= size())
         {
            newPoly.poly.set(i, C.List.f(HInteger.valueOf(i), C.C0));
         }
         else
         {
            if (getExp(indx).equals(HInteger.valueOf(i)))
            {
               newPoly.poly.set(i, poly.a[indx++]);
            }
            else
            {
               newPoly.poly.set(i, C.List.f(HInteger.valueOf(i), C.C0));
            }
         }
      }

      return newPoly;
   }
   /**
    *  eliminate leading 0-coefficients;
    *
    *  @param normalize - normalizes the leading coefficient
    */
   public void reduce(boolean normalize)
   {
      HObject   maincoeff = null;
      for (int i = size() - 1; i >= 0; i--)
      {
         if (getCoeff(i).equals(C.C0))
         {
            if (i != 0)
            {
               poly.remove(size()-1);
            }
            continue;
         }

         if (normalize)
         {
            maincoeff = C.EV(C.Pow.f(getCoeff(i), C.CN1));
         }

         break;
      }

      if ((maincoeff != null) && (!maincoeff.equals(C.C1)))
      {
         for (int i = size() - 1; i >= 0; i--)
         {
            if (getCoeff(i).equals(C.C0))
            {
               continue;
            }

            setCoeff(C.EV(C.Multiply.f(getCoeff(i), maincoeff)), i);
         }
      }
   }

   /**
    *  convert this polynomial representation back to a HFunction representation
    */
   public HFunction toFunction()
   {
      if (size() == 0)
      {
         return C.Add.f(C.C0);
      }

      HFunction result = new HFunction(C.Add);

      if (size() > 0)
      {
         result.add(C.Multiply.f( getCoeff(0), C.Pow.f(variable, getExp(0))));

         for (int i = 1; i < size(); i++)
         {
            //if (a[i] == null)
            //{
            //   continue;
            //}

            if (getCoeff(i).equals(C.C0))
            {
               continue;
            }

            result.add(C.Multiply.f(getCoeff(i), C.Pow.f(variable, getExp(i) )));
         }
      }

      return result;
   }
   /**
    * multiply two polynomials
    */
   public HPolynomial multiply(HPolynomial qp)
   {
      // extend the given polynomials:
      HPolynomial p = blowUp( degree() );
			HPolynomial q = qp.blowUp( qp.degree() );
      //construct the result polynomial:
      HPolynomial prod = new HPolynomial(C.C0, variable);  // 0-polynomial
      prod = prod.blowUp(p.degree() + q.degree());

      for (int i = 0; i < p.size(); i++)
      {
         for (int j = 0; j < q.size(); j++)
         {
            prod.setCoeff(C.EV( C.Add.f(prod.getCoeff(i + j),
                                C.Multiply.f(p.getCoeff(i), q.getCoeff(j)))),
                          i + j);
         }
      }

      return prod;
   }
   
   private boolean polyList(HObject expr)
   {
      if (expr instanceof HFunction)
      {
         return convertFunction((HFunction) expr, poly, variable);
      }

      if (expr.equals(variable))
      {
         poly.add(new HFunction(C.List, C.C1, C.C1));

         return true;
      }

      poly.add(new HFunction(C.List, C.C0, expr));

      return true;
   }

   public HPolynomial divide(HPolynomial oq) {
			int	pdeg = degree();
			int	qdeg = oq.degree();
			int	degree = pdeg - qdeg;

      if (degree < 0) {
				return new HPolynomial(C.C0, variable);  // 0-polynomial
			}
      HPolynomial p = (HPolynomial) clone();
      HPolynomial q = (HPolynomial) oq.clone();
			p = p.blowUp(pdeg);
			q = q.blowUp(qdeg);

			HPolynomial	quotient = new HPolynomial(variable, degree);
			HObject		obj = C.EV(C.Pow.f(q.getCoeff(qdeg), C.CN1));
			HObject		z;

			for (int i = degree; i >= 0; i--) {
				z = C.EV(C.Multiply.f(p.getCoeff(pdeg), obj));

				quotient.setCoeff(z, i);

				for (int j = i; j <= pdeg; j++) {
					p.setCoeff(C.EV(C.Add.f(p.getCoeff(j), C.Multiply.f(C.CN1, z, q.getCoeff(j
							  - i)))), j);
				}

				if (pdeg > 0) {
					pdeg--;
				}
			}

			quotient.reduce(false);

			return quotient;
		}

    public HPolynomial mod(HPolynomial oq) {
			int	pdeg = degree();
			int	qdeg = oq.degree();
      int degree = pdeg - qdeg;

      HPolynomial p = (HPolynomial) clone();
      HPolynomial q = (HPolynomial) oq.clone();
			if (pdeg < qdeg) {
        p.reduce(false);
			  return p;
			}

			p = p.blowUp(pdeg);
			q = q.blowUp(qdeg);

			HObject  obj = C.EV(C.Pow.f(q.getCoeff(qdeg), C.CN1));
			HObject  z;

			for (int i = degree; i >= 0; i--) {
				z = C.EV(C.Multiply.f(p.getCoeff(i + qdeg), obj));

				for (int j = qdeg; j >= 0; j--) {
					p.setCoeff(C.EV(C.Add.f(p.getCoeff(i + j), C.Multiply.f(C.CN1, z, q.getCoeff(j)))),
								i + j);
				}
			}

			p.reduce(false);

			return p;
		}

    public HPolynomial gcd(HPolynomial oq) {
			HPolynomial	np, nq;
			HPolynomial	gcd, pr;
			int			pdeg = degree();
			int			qdeg = oq.degree();
			int			npdeg, nqdeg;

			if (pdeg == 0 || qdeg == 0) {
				return null;
			}
			if (pdeg > qdeg) {
				np = (HPolynomial) clone();
				nq = (HPolynomial) oq.clone();
				npdeg = pdeg;
				nqdeg = qdeg;
			} else {
        nq = (HPolynomial) clone();
				np = (HPolynomial) oq.clone();
				nqdeg = pdeg;
				npdeg = qdeg;
			}

			np = np.blowUp(npdeg);
			nq = nq.blowUp(nqdeg);
			gcd = nq;
			pr = nq;

			while (!pr.isNull()) {
				gcd = pr;
				pr = np.mod(nq);
				np = nq;
				nq = pr;
			}
      return gcd;
		}

    public HPolynomial lcm(HPolynomial q) {
			HPolynomial	temp, np;
      HPolynomial p = blowUp(degree());
			q = q.blowUp(q.degree());
      np = p.multiply(q);
      np.reduce(false);
   //   C.resultOut.append(np.toString());
   //   C.resultOut.append("\n");
      temp = p.gcd(q);
      temp.reduce(true);
  //    C.resultOut.append(temp.toString());
  //    C.resultOut.append("\n");

      return np.divide(temp);
    }

    /**
     * solve polynomial with degree <= 2
     *
     */
    public HFunction solve() {
      HObject p, q, temp;
      blowUp(degree());
      HFunction lst = C.List.f();
      switch (degree()) {
        case 0:
          return lst;
        case 1:
          lst.add( C.EV( C.Multiply.f(C.CN1, getCoeff(0), C.Pow.f(getCoeff(1), C.CN1) ) ) );
          return lst;
        case 2:
          p = C.EV( C.Multiply.f(C.C1D2, getCoeff(1), C.Pow.f(getCoeff(2), C.CN1) ) );
          q = C.EV( C.Multiply.f(C.CN1, getCoeff(0), C.Pow.f(getCoeff(2), C.CN1) ) );
          temp = C.EV( C.Pow.f( C.Add.f( C.Pow.f( p, C.C2 ), q), C.C1D2 ) );
          lst.add( C.EV(C.Add.f( C.Multiply.f(C.CN1,p), temp) ) );
          lst.add( C.EV(C.Add.f( C.Multiply.f(C.CN1,p),
                                      C.Multiply.f(C.CN1,temp)) ) );
          return lst;

        case 3:
      }
      return null;
    }

    /**
    * Convert a function f into polynomial representation
    */
   private static boolean convertFunction(HFunction f, HFunction result, HObject x)
   {
      if (f.head().equals(C.Add))
      {
         for (int i = 0; i < f.size(); i++)
         {
            if (f.a[i] instanceof HFunction)
            {
               if (! monomialConvert((HFunction) f.a[i], result, x))
               {
                  return false;
               }
            }
            else
            {
               if (f.a[i].equals(x))
               {
                  result.add(new HFunction(C.List, C.C1, C.C1));
               }
               else
               {
                  result.add(new HFunction(C.List, C.C0, f.a[i]));
               }
            }
         }

         if (result.size() > 1)
         {
            result.sort();

            int         indx = 1;
            HFunction   lst = (HFunction) result.a[0];

            while (indx < result.len)
            {
               if (lst.a[0].equals(result.get(indx,0)))
               {
                  lst.set(1, 
                          C.EV(C.Add.f(lst.a[1], 
                                       result.get(indx, 1))));
                  result.remove(indx);
               }
               else
               {
                  lst = (HFunction) result.a[indx];
                  indx++;
               }
            }
         }

         return true;
      }

      return monomialConvert(f, result, x);
   }

   private static boolean monomialConvert(HFunction f, HFunction result, HObject x)
   {

      // HFunction result = new HFunction(C.List);
      HFunction temp = null;
      HFunction coeff = null;
      HInteger  exp = null;

      if (f.head().equals(C.Multiply))
      {
         for (int i = 0; i < f.size(); i++)
         {
            if (f.a[i] instanceof HFunction)
            {
               temp = (HFunction) f.a[i];

               if (temp.size() == 2 && temp.head().equals(C.Pow))
               {
                  if (temp.a[0].equals(x) && temp.a[1] instanceof HInteger 
                      && ((HInteger) temp.a[1]).isPositive())
                  {
                     coeff = (HFunction) f.clone();

                     coeff.remove(i);

                     exp = (HInteger) temp.a[1];

                     break;
                  }
               }
            }
            else
            {
               if (f.a[i].equals(x))
               {
                  coeff = (HFunction) f.clone();

                  coeff.remove(i);

                  exp = C.C1;
               }
            }
         }

         if (coeff != null)
         {
            if (coeff.isMember(x))
            {
               return false;
            }

            result.add(new HFunction(C.List, exp, coeff));

            return true;
         }
      }

      if (f.head().equals(C.Pow) && f.size() == 2 && f.a[1] instanceof HInteger
          && ((HInteger) f.a[1]).isPositive())
      {
         if (f.a[0].equals(x))
         {
            result.add(new HFunction(C.List, (f.a[1]), C.C1));

            return true;
         }

         if (!(f.a[0]).isMember(x))
         {
            result.add(new HFunction(C.List, (f.a[1]),
                                          (f.a[0])));

            return true;
         }
      }

      if (!f.isMember(x))
      {
         result.add(new HFunction(C.List, C.C0, f));

         return true;
      }

      return false;
   }
}



