/*--- formatted by Jindent 2.0b2, (www.c-lab.de/~jindent) ---*/
package com.hartmath.patternmatching;

import com.hartmath.expression.*;

import java.util.Enumeration;
import java.io.*;

import java.lang.*;

/**
 * The HRule class implements rules like
 * lhs = rhs; lhs:= rhs; lhs ^= rhs; lhs ^:= rhs; lhs -> rhs; lhs :> rhs
 * 
 * @see HRuleSet
 * 
 * @author Klaus Hartlage  <A HREF="mailto:khartlage@t-online.de">khartlage@t-online.de</A>
 */
public class HRule {
	HObject				lhs;
	HObject				rhs;
	int					ruleType;

	/**
	 * this flag indicates that a rule should be written as lhs -> rhs
	 * 
	 * @see EDefinition
	 */
	final static public int RULE = 0x00001;

	/**
	 * this flag indicates that a rule should be written as lhs :> rhs
	 * 
	 * @see EDefinition
	 */
	final static public int RULEDELAYED = 0x00002;

	/**
	 * this flag indicates that a rule should be written as lhs = rhs
	 * 
	 * @see EDefinition
	 */
	final static public int SET = 0x01000;

	/**
	 * this flag indicates that a rule should be written as lhs := rhs
	 * 
	 * @see EDefinition
	 */
	final static public int SETDELAYED = 0x02000;

	/**
	 * this flag indicates that a rule should be written as lhs ^= rhs
	 * 
	 * @see EDefinition
	 */
	final static public int UPSET = 0x04000;

	/**
	 * this flag indicates that a rule should be written as lhs ^:= rhs
	 * 
	 * @see EDefinition
	 */
	final static public int UPSETDELAYED = 0x08000;

	/**
	 * Class Constructor.
	 * 
	 * 
	 * @param rt
	 * @param lefthandside
	 * @param righthandside
	 *
	 * @see
	 */
	public HRule(int rt, HObject lefthandside, HObject righthandside) {
		lhs = lefthandside;
		rhs = righthandside;
		ruleType = rt;
			//{{INIT_CONTROLS
		//}}
}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @return
	 *
	 * @see
	 */
	public HObject lhs() {
		return lhs;
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @return
	 *
	 * @see
	 */
	public HObject rhs() {
		return rhs;
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @return
	 *
	 * @see
	 */
	public int hashCode() {
		return lhs.hashCode();
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param obj
	 *
	 * @return
	 *
	 * @see
	 */
	public boolean equals(Object obj) {
		if (obj instanceof HRule) {
			return lhs.equals(((HRule) obj).lhs);
		}

		return false;
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @return
	 *
	 * @see
	 */
	public String toString() {
		StringBuffer	buf = new StringBuffer();

		buf.append(lhs.toString());

		switch (ruleType) {

		case HRule.SET: 
			buf.append("=");

			break;

		case HRule.SETDELAYED: 
			buf.append(":=");

			break;

		case HRule.UPSET: 
			buf.append("^=");

			break;

		case HRule.UPSETDELAYED: 
			buf.append("^:=");

			break;

		case HRule.RULE: 
			buf.append("->");

			break;

		case HRule.RULEDELAYED: 
			buf.append(":>");

			break;
		}

		buf.append(rhs.toString());

		return buf.toString();
	}

}

