/*--- formatted by Jindent 2.0b2, (www.c-lab.de/~jindent) ---*/
package com.hartmath.patternmatching;

import com.hartmath.lib.*;
import com.hartmath.expression.*;

import org.kaffe.java.util.*;

import java.util.Enumeration;
import java.io.*;

import java.lang.*;
/*
class HHashEqualUpTo implements BinaryPredicate {
	public boolean execute(Object x, Object y) {
		return x.hashCode() == y.hashCode();
	}
}
 */

class HHashNumber {
  private int hCode;
  
  HHashNumber(HRule f) {
    hCode = f.hashCode();
  }

  public boolean equals(Object obj)
  {
		if (obj instanceof HHashNumber) {
			return (hCode == ((HHashNumber) obj).hCode);
		}

		return false;
	}

  public int hashCode()
  {
      return hCode;
  }
}
/**
 * The HRuleSet class represents mathematical a set of rules for an associated symbol.
 * 
 * @see HSymbol
 * 
 * @author Klaus Hartlage  <A HREF="mailto:khartlage@t-online.de">khartlage@t-online.de</A>
 */
public class HRuleSet {

	/**
	 * hashtable for associated pattern rules there the left-hand-side of the rule contains no pattern
	 * i.e.: Sin(0)=0
	 */
	private HashMap EqualDownRules;

	/**
	 * hashtable for associated pattern rules there the left-hand-side of the rule contains patterns
	 * i.e.: f(x_, y_) := g( h(y, x) )
	 */
	private HashMap PatternDownRules;

  /**
	 * hashtable for associated pattern rules there the left-hand-side of the rule contains no pattern
	 * i.e.: Sin(0)^=0
	 */
	private HashMap EqualUpRules;

	/**
	 * hashtable for associated pattern rules there the left-hand-side of the rule contains patterns
	 * i.e.: f(g(x_)) ^:= t( x)
	 */
	private HashMap PatternUpRules;

	final HSymbol		symb;

	/*
	 * static public boolean hashEquals(HObject x, HObject y) {
	 * return x.hashCode() == y.hashCode();
	 * }
	 * 
	 * static public boolean patternHashEquals(HFunction x, HFunction f) {
	 * return x.hashCode() == f.patternHashCode();
	 * }
	 */

	/**
	 * Class Constructor.
	 * 
	 * 
	 * @param sym
	 * 
	 * @see
	 */
	public HRuleSet(HSymbol sym) {
    EqualDownRules = new HashMap();
		PatternDownRules = new HashMap();

    EqualUpRules = new HashMap();
		PatternUpRules = new HashMap();
		symb = sym;
}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @see
	 */
	public void clear() {
		EqualUpRules.clear();
		PatternUpRules.clear();
    EqualDownRules.clear();
		PatternDownRules.clear();
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @return
	 * 
	 * @see
	 */
	public HSymbol getSymbol() {
		return symb;
	}

  /**
	 * associate a new "down"-rule (left-hand-side ==> right-hand-side) with this symbol
	 */
	public void putDownRule(int rt, HObject lhs, HObject rhs) {
		HRule rule = new HRule(rt, lhs, rhs);

		if (lhs instanceof HSymbol) {
			if (!((HSymbol) lhs).isEmptyBlockVariable()) {
				((HSymbol) lhs).popBlockVariable();
				((HSymbol) lhs).pushBlockVariable(rhs);
			} else {
				EqualDownRules.put(rule, rule);
			}
		} else {
			if (lhs instanceof HFunction 
					  && ((HFunction) lhs).isEqualRule() == 0) {
				EqualDownRules.put(rule, rule);
			} else {

				// PatternDownRules.add( rule );
        ArrayList harr = (ArrayList) PatternDownRules.get( new HHashNumber(rule) );
        if (harr==null) {
          harr = new ArrayList();
          harr.add(rule);
				  PatternDownRules.put(new HHashNumber(rule), harr);
        }
        else {
          for (int i=0;i<harr.size();i++) {
            if (rule.equals(harr.get(i))) {
              harr.set(i, rule);
              return;
            }
          }
          harr.add(rule);
        }
			}
		}
	}
	/**
	 * associate a new "up"-rule (left-hand-side ==> right-hand-side) with this symbol
	 */
	public void putUpRule(int rt, HObject lhs, HObject rhs) {
		HRule rule = new HRule(rt, lhs, rhs);

		if (lhs instanceof HSymbol) {
			if (!((HSymbol) lhs).isEmptyBlockVariable()) {
				((HSymbol) lhs).popBlockVariable();
				((HSymbol) lhs).pushBlockVariable(rhs);
			} else {
				EqualUpRules.put(rule, rule);
			}
		} else {
			if (lhs instanceof HFunction
					  && ((HFunction) lhs).isEqualRule() == 0) {
				EqualUpRules.put(rule, rule);
			} else {

				//PatternUpRules.add(rule);
        ArrayList harr = (ArrayList) PatternUpRules.get( new HHashNumber(rule) );
        if (harr==null) {
          harr = new ArrayList();
          harr.add(rule);
				  PatternUpRules.put(new HHashNumber(rule), harr);
        }
        else {
          for (int i=0;i<harr.size();i++) {
            if (rule.equals(harr.get(i))) {
              harr.set(i, rule);
              return;
            }
          }
          harr.add(rule);
        }
			}
		}
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param lhs
	 * 
	 * @see
	 */
	public void clearSymbolRule(HSymbol lhs) {


		if (lhs instanceof HSymbol) {
			if (!((HSymbol) lhs).isEmptyBlockVariable()) {
				((HSymbol) lhs).popBlockVariable();
				((HSymbol) lhs).pushBlockVariable(null);
			}
      else {
        HRule testRule = new HRule(HRule.SET, lhs, null);
				EqualDownRules.remove(testRule);
			}
		}
    /*
    else {
			if (lhs instanceof HFunction
					  && ((HFunction) lhs).isEqualRule() == 0) {
				EqualDownRules.remove(testRule);
			} else {
				PatternDownRules.remove(testRule);
			}
		} */
	}

	HObject testDownRules(HRule testRule, HFunction obj) {
		HRule					rule;
		HFunction			fun;

		PatternMatching	pm;
    //for (HashSetIterator j=PatternDownRules.find(testRule);!j.atEnd();j.advance()) {
    ArrayList harr = (ArrayList) PatternDownRules.get( new HHashNumber(testRule) );
    if (harr!=null) {
    for (int i=0;i<harr.size();i++) {
      rule = (HRule) harr.get(i);
      fun = (HFunction) rule.lhs();

			// if (! hashEquals(fun, obj) ) {
			// continue;
			// }
        /*
			if (C.traceExtendedFlag) {
				C.traceOut.append("TRACE(Down values)::");
				C.traceOut.append(fun.toString());
				C.traceOut.append(" <-> ");
				C.traceOut.append(obj.toString());
				C.traceOut.append("\n");
			}   */

			HArrayList arr = new HArrayList();

			fun.initPattern(arr);
			C.RHS__.initValue();
			arr.add(C.RHS__);

			try {
				pm = new PatternMatching(rule.lhs, rule.rhs);

				if (pm.execute(fun, obj)) {
					HFunction.resetPattern(arr);

					return pm.getResult();
				}
			} catch (MatchingFailed mf) {

			// check the next combination

			}
			catch (HBreakException ex) {		// if no loop defined: same effect as Check
			}
			catch (HContinueException ex) {		// if no loop defined: same effect as Check
			}
			catch (HReturnException ex) {
				HFunction.resetPattern(arr);

				return ex.getValue();
			}

			HFunction.resetPattern(arr);
		}
    }
		return null;
	}

  public boolean isEmptyDownRule() {
    return EqualDownRules.isEmpty()&&PatternDownRules.isEmpty();
  }

	public HObject getDownRule(HObject obj) {
    if ( ! EqualDownRules.isEmpty() ) {
		  HRule rule = (HRule) EqualDownRules.get(new HRule(HRule.SET, obj, null));

		  if (rule != null) {
			  return C.EV(rule.rhs());
		  }
    }
    if ( PatternDownRules.isEmpty() ) {
      return null;
		}
		if (!(obj instanceof HFunction)) {
			return null;
		}

    HRule testRule = new HRule(HRule.SET, obj, null);
		HFunction	fun = (HFunction) obj;
		HObject		result;

		int			attr = fun.getAttributes();

		if (((HSymbol.FLAT & attr) == HSymbol.FLAT)) {
			for (int i = 0; i < fun.size(); i++) {
				testRule = new HRule(HRule.SET,
											new HFunction(obj.head(), fun.get(i)),
											null);
				result = testDownRules(testRule, (HFunction) obj);

				if (result != null) {
					return result;
				}
			}
		} else {

			result = testDownRules(testRule, (HFunction) obj);

			if (result != null) {
				return result;
			}
		}

		// 2nd try for rules which have a pattern as 1st argument:

		fun = new HFunction(obj.head(), new HPattern(C.x));
		testRule = new HRule(HRule.SET, fun, null);
		result = testDownRules(testRule, (HFunction) obj);

		if (result != null) {
			return result;
		}

		return null;
	}

	public boolean matchDownRule(HObject obj) {
		if (getDownRule(obj) != null) {
			return true;
		}

		return false;
	}

  HObject testUpRules(HRule testRule, HFunction obj) {
		HRule					rule;
		HFunction			fun;
		PatternMatching	pm;


    //for (HashSetIterator j=PatternUpRules.find(testRule);!j.atEnd();j.advance()) {
    ArrayList harr = (ArrayList) PatternUpRules.get( new HHashNumber(testRule) );
    if (harr!=null) {
    for (int i=0;i<harr.size();i++) {
      rule = (HRule) harr.get(i);
      fun = (HFunction) rule.lhs();
             /*
			if (C.traceExtendedFlag) {
				C.traceOut.append("TRACE(Up values)::");
				C.traceOut.append(fun.toString());
				C.traceOut.append(" <-> ");
				C.traceOut.append(obj.toString());
				C.traceOut.append("\n");
			}        */

			HArrayList arr = new HArrayList();

			fun.initPattern(arr);
			C.RHS__.initValue();
			arr.add(C.RHS__);

			try {

				// fun.initPattern();

				pm = new PatternMatching(rule.lhs, rule.rhs());

				if (pm.execute(fun, obj)) {
					HFunction.resetPattern(arr);

					return pm.getResult();
				}
			} catch (MatchingFailed mf) {

			// check the next combination

			}
			catch (HBreakException ex) {		// if no loop defined: same effect as Check
			}
			catch (HContinueException ex) {		// if no loop defined: same effect as Check
			}
			catch (HReturnException ex) {
				HFunction.resetPattern(arr);

				return ex.getValue();
			}

			HFunction.resetPattern(arr);
		}
    }
		return null;
	}

  public boolean isEmptyUpRule() {
    return EqualUpRules.isEmpty()&&PatternUpRules.isEmpty();
  }
  
	public HObject getUpRule(HObject obj) {
    if ( ! EqualUpRules.isEmpty() ) {
		  HRule rule = (HRule) EqualUpRules.get(new HRule(HRule.SET, obj, null));

		  if (rule != null) {
			  return C.EV(rule.rhs());
		  }
    }
    if ( PatternUpRules.isEmpty() ) {
      return null;
		}
		if (!(obj instanceof HFunction)) {
			return null;
		}

    HRule testRule = new HRule(HRule.SET, obj, null);
		HFunction	fun = (HFunction) obj;
		HObject		result;

		int			attr = fun.getAttributes();

		if (((HSymbol.FLAT & attr) == HSymbol.FLAT)) {
			for (int i = 0; i < fun.size(); i++) {
				testRule = new HRule(HRule.SET,
											new HFunction(obj.head(), fun.get(i)),
											null);
				result = testUpRules(testRule, (HFunction) obj);

				if (result != null) {
					return result;
				}
			}
		} else {

			result = testUpRules(testRule, (HFunction) obj);

			if (result != null) {
				return result;
			}
		}

		// 2nd try for rules which have a pattern as 1st argument:
    /*
		fun = new HFunction(obj.head(), new HPattern(C.x));
		testRule = new HRule(HRule.SET, fun, null);
		result = testUpRules(testRule, (HFunction) obj);

		if (result != null) {
			return result;
		}
      */
		return null;
	}

  public boolean matchUpRule(HObject obj) {
		if (getUpRule(obj) != null) {
			return true;
		}

		return false;
	}
	/**
	 * Method Declaration.
	 *
	 *
	 * @return
	 *
	 * @see
	 */
	public String getDefinition() {
    StringBuffer	strBuf = new StringBuffer();
		try {
      Iterator iter = EqualUpRules.entrySet().iterator();
      ArrayList harr;
      while (iter.hasNext()) {
        strBuf.append(((AbstractMapEntry)iter.next()).getValue().toString());
				strBuf.append("\n");
      }
      iter = PatternUpRules.entrySet().iterator();
      while (iter.hasNext()) {
        harr = (ArrayList) ((AbstractMapEntry)iter.next()).getValue();
        for (int i=0;i<harr.size();i++) {
          strBuf.append(harr.get(i).toString());
				  strBuf.append("\n");
        }
      }
      iter = EqualDownRules.entrySet().iterator();
      while (iter.hasNext()) {
        strBuf.append(((AbstractMapEntry)iter.next()).getValue().toString());
				strBuf.append("\n");
      }
      iter = PatternDownRules.entrySet().iterator();
      while (iter.hasNext()) {
        harr = (ArrayList) ((AbstractMapEntry)iter.next()).getValue();
        for (int i=0;i<harr.size();i++) {
          strBuf.append(harr.get(i).toString());
				  strBuf.append("\n");
        }
      }
       /*
      for (Enumeration k = EqualUpRules.begin(); k.hasMoreElements(); ) {
				strBuf.append(k.nextElement().toString());
				strBuf.append("\n");
			}
			for (Enumeration k = PatternUpRules.begin(); k.hasMoreElements(); ) {
				strBuf.append(k.nextElement().toString());
				strBuf.append("\n");
			}
			for (Enumeration k = EqualDownRules.begin(); k.hasMoreElements(); ) {
				strBuf.append(k.nextElement().toString());
				strBuf.append("\n");
			}
			for (Enumeration k = PatternDownRules.begin(); k.hasMoreElements(); ) {
				strBuf.append(k.nextElement().toString());
				strBuf.append("\n");
			}             */
		} catch (Throwable se) {
			C.resultOut.appendLine("Exception occured: " + se.getMessage());

			if (C.DEBUG) {
				se.printStackTrace();
			}
		}

		return strBuf.toString();
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @return
	 * 
	 * @see
	 */
	public int hashCode() {
		return symb.hashCode();
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param obj
	 * 
	 * @return
	 * 
	 * @see
	 */
	public boolean equals(Object obj) {
		if (obj instanceof HRuleSet) {
			return symb.equals(((HRuleSet) obj).symb);
		}

		return false;
	}

	//{{DECLARE_CONTROLS
	//}}
}



/*--- formatting done in "Sun Java Convention" style on 07-04-1999 ---*/

