/*--- formatted by Jindent 2.0b2, (www.c-lab.de/~jindent) ---*/
package com.hartmath.expression;

import java.lang.*;

/**
 * Interface that represents a signed number (e.g. HDouble, HInteger or HFraction)
 * 
 * 
 * @see HDouble
 * @see HInteger
 * @see HFraction
 * @see HRational
 * 
 * @author Klaus Hartlage  <A HREF="mailto:khartlage@t-online.de">khartlage@t-online.de</A>
 */
public interface HSignedNumber extends HNumber {

  /**
	 * Returns the int number if possible
	 */
  abstract public int intValue();
	/**
	 * Returns true if the number is > 0
	 */
	abstract public boolean isPositive();

	/**
	 * Returns true if the number is < 0
	 */
	abstract public boolean isNegative();

	/**
	 * Returns true if the number is >= 0
	 */
	abstract public boolean isNonNegative();

	/**
	 * Returns the signum function of this number (i.e., -1, 0 or 1 as
	 * the value of this number is negative, zero or positive).
	 */
	abstract public int signum();

	/**
	 * Returns the smallest (closest to negative infinity)
	 * <code>HObject</code> value that is not less than <code>this</code> and is
	 * equal to a mathematical integer.
	 * 
	 * @return  the smallest (closest to negative infinity)
	 * <code>HObject</code> value that is not less than <code>this</code>
	 * and is equal to a mathematical integer.
	 */
	abstract public HObject ceil();

	/**
	 * Returns the largest (closest to positive infinity)
	 * <code>HObject</code> value that is not greater than <code>this</code> and
	 * is equal to a mathematical integer.
	 * 
	 * @return  the largest (closest to positive infinity)
	 * <code>HObject</code> value that is not greater than <code>this</code>
	 * and is equal to a mathematical integer.
	 */
	abstract public HObject floor();
}


