/*--- formatted by Jindent 2.0b2, (www.c-lab.de/~jindent) ---*/
package com.hartmath.expression;

import com.hartmath.lib.*;
import java.lang.*;
import java.io.*;

/**
 * The HString class represents character strings.
 * This class adopts much of his functionality from the java.lang.String class
 * 
 * @see HSymbol
 * 
 * @author Klaus Hartlage  <A HREF="mailto:khartlage@t-online.de">khartlage@t-online.de</A>
 */
public class HString implements HObject {
	protected String  str;

	/**
	 * Constructs a new empty HString.
	 */
	public HString() {
		this.str = new String();
			//{{INIT_CONTROLS
		//}}
}

	/**
	 * Constructs a new HString that is a copy of the specified String.
	 * @param value the initial value of the HString
	 */
	public HString(String value) {
		this.str = new String(value);
	}

	/**
	 * Constructs a new HString whose initial value is the specified array
	 * of characters.
	 * @param value the initial value of the HString
	 */
	public HString(char value[]) {
		this.str = new String(value);
	}

	/**
	 * Constructs a new HString whose initial value is the specified sub array of characters.
	 * The length of the new string will be count characters
	 * starting at offset within the specified character array.
	 * @param value	the initial value of the HString, an array of characters
	 * @param offset	the offset into the value of the HString
	 * @param count 	the length of the value of the HString
	 * @exception StringIndexOutOfBoundsException If the offset and count arguments are invalid.
	 */
	public HString(char value[], int offset, int count) {
		this.str = new String(value, offset, count);
	}

	/**
	 * Constructs a new HString whose initial value is the specified sub array of bytes.
	 * The high-byte of each character can be specified, it should usually be 0.
	 * The length of the new HString will be count characters
	 * starting at offset within the specified character array.
	 * @param ascii	the bytes that will be converted to characters
	 * @param hibyte	the high byte of each Unicode character
	 * @param offset	the offset into the ascii array
	 * @param count 	the length of the HString
	 * @exception StringIndexOutOfBoundsException If the offset and count arguments are invalid.
	 */

	// public HString(byte ascii[], int hibyte, int offset, int count) {
	// this.str = new String(ascii, hibyte, offset, count);
	// }

	/**
	 * Constructs a new HString whose value is the specified array of bytes.
	 * The byte array transformed into Unicode chars using hibyte
	 * as the upper byte of each character.
	 * @param ascii	the byte that will be converted to characters
	 * @param hibyte	the top 8 bits of each 16 bit Unicode character
	 */

	// public HString(byte ascii[], int hibyte) {
	// this.str = new String(ascii, hibyte);
	// }

	/**
	 * Construct a new string whose value is the current contents of the
	 * given string buffer
	 * @param buffer     the stringbuffer to be converted
	 */
	public HString(StringBuffer buffer) {
		this.str = new String(buffer);
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @return
	 *
	 * @see
	 */
	public String toString() {
		return str;
	}
  public void toStringBuffer(StringBuffer buf)
  {
     buf.append(str);
  }
	/**
	 * Method Declaration.
	 * 
	 * 
	 * @return
	 *
	 * @see
	 */
	public int hashCode() {
		return (str.hashCode() & 0x55555555);
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param obj
	 *
	 * @return
	 *
	 * @see
	 */
	public boolean matches(HObject obj) {
		return equals(obj);
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param obj
	 *
	 * @return
	 *
	 * @see
	 */
	public boolean equals(Object obj) {
		if (obj instanceof HString) {
			return str.equals(((HString) obj).str);
		}

		return false;
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param obj
	 *
	 * @return
	 *
	 * @see
	 */
	public boolean unequals(Object obj) {
		return !equals(obj);
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param obj
	 *
	 * @return
	 *
	 * @see
	 */
	public int compareTo(HString obj) {
		return str.compareTo(obj.str);
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param obj
	 *
	 * @return
	 *
	 * @see
	 */
	public boolean less(Object obj) {
		if (obj instanceof HString) {
			HString  temp = (HString) obj;

			return (str.compareTo(temp.str) < 0);
		}
		if (obj instanceof HObject) {
			return (hierarchy() < ((HObject) obj).hierarchy());
		}

		return false;
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param obj
	 *
	 * @return
	 *
	 * @see
	 */
	public boolean greater(Object obj) {
		if (obj instanceof HObject) {
			return ((HObject) obj).less(this);
		}

		return false;
	}

	/**
	 * Returns <code>null</code>; Strings will not be evaluated
	 */
	public HObject evaluate() {
		return null;
	}

	/**
	 * Substitute the HObject <code>obj</code> with the HObject <code>subst</code>,
	 * if <code>obj</code> is equal to <code>this</code>
	 * 
	 * @param obj         an object to compare with
	 * @param subst       the substitution for the object
	 */
	public HObject substitute(HObject obj, HObject subst) {
		if (equals(obj)) {
			return subst;
		}

		return this;
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @return
	 *
	 * @see
	 */
	public HObject substitutePattern() {
		return this;
	}

	/**
	 * Returns an specific integer value for each class in the
	 * math-objects hierarchy
	 */
	public int hierarchy() {
		return HObject.STRINGID;
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param os
	 *
	 * @exception IOException
	 *
	 * @see
	 */        /*
	public void writeTo(DataOutputStream os) throws IOException {
		os.writeByte(HObject.STRINGFILEID);
		os.writeUTF(str);
	}            */

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @return
	 *
	 * @see
	 */
	public HSymbol head() {
		return C.String;
	}

  public HObject apply(HObject head) {
    if (C.DEBUG) {                   // only in debug mode
      if (head==null) {
        throw new HThrowException(C.HeadFormatError, C.Symbol, C.Null);
      }
    }
    if (head instanceof HSymbol) {
      return C.EV( new HFunction((HSymbol)head, this) );
    }
    if ( (head instanceof HFunction) &&
         ((HFunction)head).isPureFunction() ) {
      HFunction l = (HFunction) ((HFunction)head).clone();
      l.add(this);
      return C.EV(l);
    }
    throw new HThrowException(C.HeadFormatError, C.Lambda, head);
  }
	/**
	 * Method Declaration.
	 * 
	 * 
	 * @return
	 *
	 * @see
	 */
	public boolean isList() {
		return false;
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @return
	 *
	 * @see
	 */
	public boolean isNumber() {
		return false;
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @return
	 *
	 * @see
	 */
	public boolean isRational() {
		return false;
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param obj
	 *
	 * @return
	 *
	 * @see
	 */
	public boolean isMember(HObject obj) {
		return equals(obj);
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param sym
	 *
	 * @return
	 *
	 * @see
	 */
	public boolean isHeadMember(HSymbol sym) {
		return head().equals(sym);
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @return
	 *
	 * @see
	 */
	public int precedence() {
		return 999999;
	}

	//{{DECLARE_CONTROLS
	//}}
}



/*--- formatting done in "Sun Java Convention" style on 06-18-1999 ---*/
