/*--- formatted by Jindent 2.1, (www.c-lab.de/~jindent) ---*/

package com.hartmath.expression;

import com.hartmath.lib.*;
import com.hartmath.mapping.*;
import com.hartmath.patternmatching.*;

import org.kaffe.java.util.*;

import java.util.Enumeration;
import java.util.Stack;
import java.io.*;
import java.io.Serializable;
//import com.objectspace.jgl.BinaryPredicate;
//import com.objectspace.jgl.algorithms.Sorting;


import java.lang.*;

// import java.awt.*;

/**
 * The HSymbol class represents mathematical symbols like variables or names of functions
 * 
 * @see HFunction
 * 
 * @author Klaus Hartlage  <A HREF="mailto:khartlage@t-online.de">khartlage@t-online.de</A>
 */
public class HSymbol extends HString
{

   /**
    * counts the implemented standard symbols
    */
   public static int            maxItems = 0;

   /**
    * contains the implemented standard symbols in string format
    */
   public static String         choiceItems[] = new String[1000];
                                        
   /**
    * contains the implemented standard symbols
    */

   private static HashMap       HT = new HashMap();
   private HRuleSet             rSet;

   /**
    * In[] and Out[] expression counter
    */
   public static int            IOCounter;

   /**
    * contains the Out[] expressions
    */
   public static HashMap        IOOut = new HashMap();

   /**
    * contains the In[] expressions
    */
   public static HashMap        IOIn = new HashMap();


   // attributes which can be defined for every HSymbol

   /**
    * HSymbol attribute which means that no attribute is set
    */
   final static public int      NOATTRIBUTE = 0x0000;           // set no Attribute

   /**
    * HSymbol attribute for a function transformation: f(x) ==> x
    */
   final static public int      ONEIDENTITY = 0x0001;           // f(x)  --> x

   /**
    * HSymbol attribute to indicate that a symbol or function is constant
    */
   final static public int      CONSTANT = 0x0002;              // f'(x) --> 0

   /**
    * HSymbol attribute for a commutative function transformation
    * The evaluation of the function will sort the arguments
    * 
    * @see HFunction.sort()
    */
   final static public int      ORDERLESS = 
      0x0004;                                                   // commutative (start sort() of arguments)

   /**
    * HSymbol attribute for a associative function transformation
    * The evaluation of the function will flatten the arguments
    * 
    * @see HFunction.flatten()
    */
   final static public int      FLAT = 0x0008;                  // associative

   /**
    * HSymbol attribute for a function, which should not be evaluated
    * 
    * @see HFunction.evalArguments()
    */
   final static public int      HOLDALL = 0x0010;       // no argument evaluation

   /**
    * HSymbol attribute for a function, there the first argument should not be evaluated
    * 
    * @see HFunction.evalArguments()
    */
   final static public int      HOLDFIRST = 
      0x0020;                                                   // don't evaluate first Argument

   /**
    * HSymbol attribute for a function, there the rest of the arguments should not be evaluated
    * 
    * @see HFunction.evalArguments()
    */
   final static public int      HOLDREST = 
      0x0040;                                                   // evaluate only first argument

   /**
    * HSymbol attribute for a function with lists as arguments
    * 
    * @see HFunction.threadList()
    */
   final static public int      LISTABLE = 
      0x0080;                                                   // ex. sin({a,b,c}) --> {sin(a),sin(b),sin(c)}

   /**
    * HSymbol attribute for a function, there the first argument should not be evaluated
    * 
    * @see HFunction.evalArguments()
    */
   final static public int      BUILTINFIRST = 0x0100;

   /**
    * HSymbol attribute for a function, there the first argument should not be evaluated
    * 
    * @see HFunction.evalArguments()
    */
   final static public int      FASTCALL = 0x0200;

   /**
    * HSymbol attribute as a combination of HSymbol.FLAT and HSymbol.ORDERLESS
    * 
    * @see HFunction.sort()
    * @see HFunction.flatten()
    */
   final static public int      FLATORDERLESS = 0x000c;

   /**
    * HSymbol attribute for a function, which should be written in operator notation
    * 
    * @see FunctionOpEvaluator
    */
   final static public int      OPERATOR = 
      0x1000;                                                   // symbol has an operator (+,*,^...)

   /**
    * Evaluation-Object for functions:
    */
   protected FunctionEvaluator  functionEval;

   /*
    * Attributes of this symbol
    */
   int                          attr;
   int                          hCode;

   /**
    * hashtable for associated pattern rules there the left-hand-side of the rule contains no pattern
    * i.e.: Sin(0)=0
    */

   // public HashMap ERule;

   /**
    * hashtable for associated pattern rules there the left-hand-side of the rule contains patterns
    * i.e.: f(x_, y_) := g( h(y, x) )
    */

   // protected HashMap PRule;

   /**
    * stack for an associated pattern
    */
   protected Stack              patternValue;

   /**
    * stack for the value of this symbol if used in an local context
    */
   protected Stack              VStack;

   /**
    * check if there are associated rules for this symbol
    */

   // public boolean isEmptyRule() {
   // return ( ERule.isEmpty() && PRule.isEmpty() );
   // }

   /*
    * associate a new rule (left-hand-side ==> right-hand-side) with this symbol
    */

   /*
    * public void putRule(HObject lhs, HObject rhs ) {
    * if ( lhs instanceof HSymbol ) {
    * if ( ! ((HSymbol)lhs).isEmptyBlockVariable( ) ) {
    * ((HSymbol)lhs).popBlockVariable( );
    * ((HSymbol)lhs).pushBlockVariable( rhs );
    * }
    * else {
    * ERule.put( lhs, rhs);
    * }
    * }
    * else {
    * if ( lhs instanceof HFunction &&
    * ((HFunction)lhs).isEqualRule() ) {
    * ERule.put( lhs, rhs);
    * }
    * else {
    * PRule.put( lhs, rhs );
    * }
    * }
    * }
    * 
    * public void clearRule(HObject lhs ) {
    * if ( lhs instanceof HSymbol ) {
    * if ( ! ((HSymbol)lhs).isEmptyBlockVariable( ) ) {
    * ((HSymbol)lhs).popBlockVariable( );
    * ((HSymbol)lhs).pushBlockVariable( null );
    * }
    * else {
    * ERule.remove( lhs );
    * }
    * }
    * else {
    * if ( lhs instanceof HFunction &&
    * ((HFunction)lhs).isEqualRule() ) {
    * ERule.remove( lhs );
    * }
    * else {
    * PRule.remove( lhs );
    * }
    * }
    * }
    */

   // public boolean containsKeyRule( ) {
   // return ERule.get( this ) != null;
   // }

   /*
    * static public boolean hashEquals(HObject x, HObject y) {
    * return x.hashCode() == y.hashCode();
    * }
    * 
    * static public boolean patternHashEquals(HFunction x, HFunction f) {
    * return x.hashCode() == f.patternHashCode();
    * }
    * 
    * public HObject getRule(HObject obj ) {
    * HObject temp = (HObject) ERule.get( obj );
    * if (temp != null) {
    * return temp;
    * }
    * if ( ! ( obj instanceof HFunction ) ) {
    * return null;
    * }
    * HFunction fun;
    * Array pats;
    * PatternMatching pm;
    * // Enumeration i = PRule.find( obj );
    * for (Enumeration e = PRule.keys() ; e.hasMoreElements() ;) {
    * fun = (HFunction) e.nextElement();
    * if (! hashEquals(fun, obj) ) {
    * continue;
    * }
    * if (C.traceFlag && C.traceExtendedFlag) {
    * C.traceOut.append( "TRACE(try match)::");
    * C.traceOut.append( fun.toString() );
    * C.traceOut.append( " <-> ");
    * C.traceOut.append( obj.toString() );
    * C.traceOut.append( "\n" );
    * }
    * Array arr = new Array();
    * fun.initPattern( arr );
    * try {
    * // fun.initPattern();
    * pm = new PatternMatching( ((HObject)PRule.get(fun)) );
    * if ( pm.execute(fun, obj) ) {
    * HFunction.resetPattern(arr);
    * return pm.getResult();
    * }
    * }
    * catch (MatchingFailed mf) {
    * // check the next combination
    * }
    * catch ( HBreakException ex ) {  // if no loop defined: same effect as Check
    * }
    * catch ( HContinueException ex ) {  // if no loop defined: same effect as Check
    * }
    * catch ( HReturnException ex ) {
    * HFunction.resetPattern(arr);
    * return ex.getValue();
    * }
    * HFunction.resetPattern(arr);
    * }
    * for (Enumeration e = PRule.keys() ; e.hasMoreElements() ;) {
    * fun = (HFunction) e.nextElement();
    * if (! patternHashEquals(fun, (HFunction) obj) ) {
    * continue;
    * }
    * if (C.traceFlag && C.traceExtendedFlag) {
    * C.traceOut.append( "TRACE(try pattern-match)::");
    * C.traceOut.append( fun.toString() );
    * C.traceOut.append( " <-> ");
    * C.traceOut.append( obj.toString() );
    * C.traceOut.append( "\n" );
    * }
    * Array arr = new Array();
    * fun.initPattern( arr );
    * try {
    * //   fun.initPattern();
    * 
    * pm = new PatternMatching( ((HObject)PRule.get(fun)) );
    * if ( pm.execute(fun, obj) ) {
    * HFunction.clearPattern(arr);
    * return pm.getResult();
    * }
    * }
    * catch (MatchingFailed mf) {
    * // check the next combination
    * }
    * catch ( HBreakException ex ) {  // if no loop defined: same effect as Check
    * }
    * catch ( HContinueException ex ) {  // if no loop defined: same effect as Check
    * }
    * catch ( HReturnException ex ) {
    * HFunction.resetPattern(arr);
    * return ex.getValue();
    * }
    * HFunction.resetPattern(arr);
    * }
    * return null;
    * }
    */

   /**
    * Constructs a new empty HSymbol.
    * public HSymbol() {
    * super();
    * functionEval = dummyFunction;
    * attr = NOATTRIBUTE;
    * ERule = new HashMap();
    * PRule = new HashMap();
    * VStack = new Stack();
    * patternValue = new Stack();
    * }
    */

   /**
    * Constructs a new HSymbol that is a copy of the specified String.
    * @param value the initial value of the HSymbol
    */
   public HSymbol(String value)
   {
      super(value);

      hCode = computeHashCode();

      functionEval = C.dummyFunction;

      init();           // sets NOATTRIBUTE

      // {{INIT_CONTROLS
      // }}
   }

   /**
    * Constructs a new HSymbol that is a copy of the specified String.
    * Puts an entry in the Hashtable
    * 
    * @param value the initial value of the HSymbol
    * @feval evaluates the given function for this symbol
    */
   public HSymbol(String value, FunctionEvaluator feval)
   {
      super(value);

      hCode = computeHashCode();
      functionEval = feval;

      init();           // sets NOATTRIBUTE

      choiceItems[maxItems++] = value;
   }

   /**
    * Constructs a new HSymbol that is a copy of the specifidd String.
    * Puts an entry in the Hashtable
    * 
    * @param value the initial value of the HSymbol
    * @feval evaluates the given function for this symbol
    */
   public HSymbol(String value, FunctionEvaluator feval, int att)
   {
      super(value);

      hCode = computeHashCode();
      functionEval = feval;

      init();           // sets NOATTRIBUTE

      attr = att;       // set the attributes of the constructor
      choiceItems[maxItems++] = value;
   }

   /**
    * Constructs a new HSymbol that is a copy of the specified String.
    * Puts an entry in the Hashtable
    * 
    * @param value the initial value of the HSymbol
    * @feval evaluates the given function for this symbol
    */
   public HSymbol(String value, int att)
   {
      super(value);

      hCode = computeHashCode();
      functionEval = C.dummyFunction;

      init();           // sets NOATTRIBUTE

      attr = att;       // set the attributes of the constructor
   }

   /**
    * Constructs a new HSymbol whose initial value is the specified array
    * of characters.
    * @param value the initial value of the HSymbol
    */
   public HSymbol(char value[])
   {
      super(value);

      hCode = computeHashCode();
      functionEval = C.dummyFunction;

      init();
   }

   /**
    * Constructs a new HSymbol whose initial value is the specified sub array of characters.
    * The length of the new string will be count characters
    * starting get offset within the specified character array.
    * @param value	the initial value of the HSymbol, an array of characters
    * @param offset	the offset into the value of the HSymbol
    * @param count 	the length of the value of the HSymbol
    * @exception StringIndexOutOfBoundsException If the offset and count arguments are invalid.
    */
   public HSymbol(char value[], int offset, int count)
   {
      super(value, offset, count);

      hCode = computeHashCode();
      functionEval = C.dummyFunction;

      init();
   }

   /**
    * Construct a new string whose value is the current contents of the
    * given string buffer
    * @param buffer     the stringbuffer to be converted
    */
   public HSymbol(StringBuffer buffer)
   {
      super(buffer);

      hCode = computeHashCode();
      functionEval = C.dummyFunction;

      init();
   }

   /**
    * Construct a new string whose value is the current contents of the
    * given string buffer
    * Puts an entry in the Hashtable
    * 
    * @param buffer     the stringbuffer to be converted
    */
   public HSymbol(StringBuffer buffer, FunctionEvaluator feval)
   {
      super(buffer);

      hCode = computeHashCode();
      functionEval = feval;

      init();
   }

   /**
    * Method Declaration.
    * 
    * 
    * @see
    */
   private void init()
   {
      attr = NOATTRIBUTE;

      // ERule = new HashMap();
      // PRule = new HashMap();

      VStack = new Stack();
      patternValue = new Stack();
   }

   /**
    * set the evaluating function for this symbol
    * 
    */
   public void setEval(FunctionEvaluator feval)
   {
      functionEval = feval;
      choiceItems[maxItems++] = toString();
   }

   public FunctionEvaluator getEval( )
   {
      return functionEval;
   }

   /**
    * set the new evaluating function for this symbol
    * 
    */
   public void setNewEval(FunctionEvaluator feval)
   {
      functionEval = feval;
   }
   /**
    * Method Declaration.
    * 
    * 
    * @return
    * 
    * @see
    */
   public String toString()
   {
      return str;
   }

   /*
    * public void clear() {
    * ERule.clear();
    * PRule.clear();
    * }
    * 
    * public String getDefinition() {
    * StringBuffer strBuf = new StringBuffer();
    * HObject a0;
    * for (Enumeration k = ERule.keys(),e = ERule.elements() ; e.hasMoreElements() ;) {
    * a0 = (HObject) k.nextElement();
    * strBuf.append( a0.toString() );
    * if (a0 instanceof HSymbol) {
    * if ( ( ((HSymbol)a0).getProperties() & HSymbol.SET) == HSymbol.SET) {
    * strBuf.append( "=" );
    * }
    * else {
    * if ( ( ((HSymbol)a0).getProperties() & HSymbol.SETDELAYED) == HSymbol.SETDELAYED) {
    * strBuf.append( ":=" );
    * }
    * }
    * }
    * if (a0 instanceof HFunction) {
    * if ( ( ((HFunction)a0).getProperties() & HSymbol.SET) == HSymbol.SET) {
    * strBuf.append( "=" );
    * }
    * else {
    * if ( ( ((HFunction)a0).getProperties() & HSymbol.SETDELAYED) == HSymbol.SETDELAYED) {
    * strBuf.append( ":=" );
    * }
    * }
    * }
    * strBuf.append( e.nextElement().toString() );
    * strBuf.append( "\n" );
    * }
    * for (Enumeration k = PRule.keys(), e = PRule.elements() ; e.hasMoreElements() ;) {
    * a0 = (HObject) k.nextElement();
    * strBuf.append( a0.toString() );
    * if (a0 instanceof HSymbol) {
    * if ( ( ((HSymbol)a0).getProperties() & HSymbol.SET) == HSymbol.SET) {
    * strBuf.append( "=" );
    * }
    * else {
    * if ( ( ((HSymbol)a0).getProperties() & HSymbol.SETDELAYED) == HSymbol.SETDELAYED) {
    * strBuf.append( ":=" );
    * }
    * }
    * }
    * if (a0 instanceof HFunction) {
    * if ( ( ((HFunction)a0).getProperties() & HSymbol.SET) == HSymbol.SET) {
    * strBuf.append( "=" );
    * }
    * else {
    * if ( ( ((HFunction)a0).getProperties() & HSymbol.SETDELAYED) == HSymbol.SETDELAYED) {
    * strBuf.append( ":=" );
    * }
    * }
    * }
    * strBuf.append( e.nextElement().toString() );
    * strBuf.append( "\n" );
    * }
    * return strBuf.toString();
    * }
    */

   /**
    * Method Declaration.
    * 
    * 
    * @param fun
    * 
    * @return
    * 
    * @see
    */
   final public String toOpString(HFunction fun)
   {
      if (functionEval instanceof ELoadClass)
      {
        ELoadClass.loadClass( this );
      }
      if (functionEval instanceof FunctionOpEvaluator)
      {
         return ((FunctionOpEvaluator) functionEval).toOpString(fun);
      }
      return "op to string failed";
   }

   /**
    * Method Declaration.
    * 
    * 
    * @return
    * 
    * @see
    */
   public int precedence()
   {
      if (functionEval instanceof FunctionOpEvaluator)
      {
         return ((FunctionOpEvaluator) functionEval).precedence();
      }

      return 999999;
   }

   /**
    * Method Declaration.
    * 
    * 
    * @return
    * 
    * @see
    */
   final public int getAttributes()
   {
      return attr;
   }

   /**
    * Method Declaration.
    * 
    * 
    * @param att
    * 
    * @see
    */
   final public void setAttributes(int att)
   {
      attr |= att;
   }

   /**
    * Method Declaration.
    * 
    * 
    * @param att
    * 
    * @see
    */
   final public void clearAttributes(int att)
   {
      attr &= (att & 0xFFFF);
   }

   /*
    * final public int getProperties() {
    * return attr;
    * }
    * 
    * final public void setProperties(int att) {
    * attr |= att;
    * }
    * 
    * final public void clearProperties(int att) {
    * attr &= (att & 0xFFFF);
    * }
    */

   /**
    * Method Declaration.
    * 
    * 
    * @param att
    * 
    * @return
    * 
    * @see
    */
   final public boolean checkAttributes(int att)
   {
      return ((attr & att) == att);
   }

   /**
    * Method declaration
    *
    *
    * @param arr
    *
    * @return
    *
    * @see
    */
   public HFunction body(HArrayList arr)
   {
      return new HFunction(this, arr);
   }

   /**
    * Constructs a vector with no arguments
    * @param obj1 the first argument of the function
    * @param obj2 the second argument of the function
    * @param obj3 the third argument of the function
    * @param obj4 the 4th argument of the function
    */
   public HFunction f()
   {
      return new HFunction(this);
   }

   /**
    * Constructs a vector with 1 argument
    * @param obj1 the first argument of the function
    */
   public HFunction f(HObject obj1)
   {
      return new HFunction(this, obj1);
   }

   /**
    * Constructs a vector with 2 arguments
    * @param obj1 the first argument of the function
    * @param obj2 the second argument of the function
    */
   public HFunction f(HObject obj1, HObject obj2)
   {
      return new HFunction(this, obj1, obj2);
   }

   /**
    * Constructs a vector with 3 arguments
    * @param obj1 the first argument of the function
    * @param obj2 the second argument of the function
    * @param obj3 the third argument of the function
    */
   public HFunction f(HObject obj1, HObject obj2, HObject obj3)
   {
      return new HFunction(this, obj1, obj2, obj3);
   }

   /**
    * Constructs a vector with 4 arguments
    * @param obj1 the first argument of the function
    * @param obj2 the second argument of the function
    * @param obj3 the third argument of the function
    * @param obj4 the 4th argument of the function
    */
   public HFunction f(HObject obj1, HObject obj2, HObject obj3, HObject obj4)
   {
      return new HFunction(this, obj1, obj2, obj3, obj4);
   }

   /**
    * Method Declaration.
    * 
    * 
    * @return
    * 
    * @see
    */
   public int hashCode()
   {
      // return str.hashCode();
      return hCode;
   }

   /**
    * Method declaration
    *
    *
    * @return
    *
    * @see
    */
   public int computeHashCode()
   {

      // return str.hashCode();  String
      int       h = 0;
      char      ch;
      int       len = str.length();

      if (len > 4)
      {
         for (int i = 0; i < 5; i++)
         {
            ch = str.charAt(i);

            if (ch >= '0' && ch <= '9')
            {
               h = (h << 5) + (((int) ch) - 47);
            }
            else
            {
               if (ch >= 'A' && ch <= 'Z')
               {
                  h = (h << 5) + (((int) ch) - 54);
               }
               else
               {
                  if (ch == '$') {
                    h = (h << 5) + 63;
                  }
                  else {
                    h = (h << 5) + (((int) ch) - 60);
                  }
               }
            }
         }
      }
      else
      {
         for (int i = 0; i < len; i++)
         {
            ch = str.charAt(i);

            if (ch >= '0' && ch <= '9')
            {
               h = (h << 5) + (((int) ch) - 47);
            }
            else
            {
               if (ch >= 'A' && ch <= 'Z')
               {
                  h = (h << 5) + (((int) ch) - 54);
               }
               else
               {
                  if (ch == '$') {
                    h = (h << 5) + 63;
                  }
                  else {
                    h = (h << 5) + (((int) ch) - 60);
                  }
               }
            }
         }

         for (int i = 0; i < 5 - len; i++)
         {
            h = (h << 5);
         }
      }

      return h;
   }

   /**
    * Method Declaration.
    * 
    * 
    * @param obj
    * 
    * @return
    * 
    * @see
    */
   public boolean equals(Object obj)
   {
      if (obj instanceof HSymbol)
      {
         if (hCode != ((HSymbol) obj).hCode)
         {
            return false;
         }

         if (this == obj)
         {
            return true;
         }

         return str.equals(((HSymbol) obj).str);
      }

      return false;
   }

   /**
    * Method Declaration.
    * 
    * 
    * @param obj
    * 
    * @return
    * 
    * @see
    */
   public boolean unequals(Object obj)
   {
      return !equals(obj);
   }

   /**
    * Method Declaration.
    * 
    * 
    * @param obj
    * 
    * @return
    * 
    * @see
    */
   public boolean less(Object obj)
   {
      if (obj instanceof HSymbol)
      {
         if (hCode < ((HSymbol) obj).hCode)
         {
            return true;
         }

         if (hCode > ((HSymbol) obj).hCode)
         {
            return false;
         }

         return (str.compareTo(((HSymbol) obj).str) < 0);
      }

      if (obj instanceof HObject)
      {
         return (hierarchy() < ((HObject) obj).hierarchy());
      }

      return false;
   }

   /**
    * Compares two HSymbols
    * The comparison is based on the Unicode value of each character in
    * the strings.
    * 
    * @param   anotherString   the <code>String</code> to be compared.
    * @return  the value <code>0</code> if the argument symbol is equal to
    * this symbol; a value less than <code>0</code> if this symbol
    * is less than the symbol argument; and a
    * value greater than <code>0</code> if this symbol is
    * greater than the string argument.
    */
   public int compareTo(HSymbol sym)
   {
      int       i = hCode - sym.hCode;

      if (i != 0)
      {
         return i;
      }

      return str.compareTo(sym.str);
   }

   /**
    * Method Declaration.
    * 
    * 
    * @param obj
    * 
    * @return
    * 
    * @see
    */
   public boolean greater(Object obj)
   {
      if (obj instanceof HObject)
      {
         return ((HObject) obj).less(this);
      }

      return false;
   }

   /**
    * Returns true if <code>this</code> can be evaluated.
    * <code>obj</code> is the evaluation result.
    * @param obj         the (new) evaluated object
    */
   public HObject evaluate()
   {
      if (!VStack.isEmpty())
      {

         // HObject temp = (HObject) VStack.peek();

         HObject        temp = (HObject) VStack.peek();

         if ((temp == null) || equals(temp))
         {      // prevent infinity recursion from pattern-matching:
            return null;
         }

         return temp;
      }

      HObject   result = getDownRule(this);

      if (result != null)
      {
         return result;
      }

      if (functionEval instanceof ConstantNumericEvaluator)
      {
         //Context ct = ((ContextThread) Thread.currentThread()).getContext();
      	 if (C.numericFlag)
         {
         	 return ((ConstantNumericEvaluator) functionEval).getNumericValue();
         }
         else
         {
         	  if (functionEval instanceof ConstantEvaluator)
         	  {
         	    return ((ConstantEvaluator) functionEval).getSymbolicValue();
            }
         }
      }

      return null;
   }

   /**
    * Method Declaration.
    * 
    * 
    * @return
    * 
    * @see
    */
   public HObject substitutePattern()
   {
      if (!patternValue.isEmpty())
      {
         return (HObject) patternValue.peek();
      }

      return this;
   }

   /**
    * Method Declaration.
    * 
    * 
    * @param obj
    * 
    * @see
    */
   public void pushBlockVariable(HObject obj)
   {
      VStack.push(obj);
   }

   /**
    * Method Declaration.
    * 
    * 
    * @return
    * 
    * @see
    */
   public HObject popBlockVariable()
   {
      if (VStack.isEmpty())
      {
         return null;
      }

      return (HObject) VStack.pop();
   }

   /**
    * Method Declaration.
    * 
    * 
    * @return
    * 
    * @see
    */
   public boolean isEmptyBlockVariable()
   {
      return VStack.isEmpty();
   }

   /**
    * Returns an specific integer value for each class in the
    * math-objects hierarchy
    */
   final public int hierarchy()
   {
      return HObject.SYMBOLID;
   }

   /**
    * Method Declaration.
    * 
    * 
    * @param os
    * 
    * @exception IOException
    * 
    * @see
    */     /*
   public void writeTo(DataOutputStream os) throws IOException
   {
      if ((str.length() == 1) && (str.charAt(0) >= 'a')
          && (str.charAt(0) <= 'z'))
      {
         int    b = ((str.charAt(0) - 'a') + 1) * -1;

         // C.traceOut.append( new Integer( b ).toString() );

         os.writeByte((byte) b);
      }
      else
      {
         Short  num = (Short) HSymbol.IOOut.get(str);

         if (num != null)
         {
            os.writeByte((byte) 33);
            os.writeShort(num.shortValue());
         }
         else
         {
            os.writeByte(HObject.SYMBOLFILEID);
            os.writeUTF(str);
            os.writeInt(attr);

            num = new Short((short) IOCounter);

            HSymbol.IOOut.put(str, num);

            IOCounter++;
         }
      }
   }         */

   /**
    * Method Declaration.
    *
    *
    * @return
    *
    * @see
    */
   public HSymbol head()
   {
      return C.Symbol;
   }

   /**
    * Returns true if <code>this</code> can be evaluated.
    * <code>obj</code> is the evaluation result.
    * @param obj         the (new) evaluated object
    */
   public HObject evaluateFunction(HFunction f)
   {

      // return functionEval.evaluate(f);
      // }
      HObject   result = null;

      if ((attr & HSymbol.FASTCALL) == HSymbol.FASTCALL)
      {
         return functionEval.evaluate(f);
      }

      // result = C.ELoop( this );
      if ((result = C.ELoop(this)) != null)
      {

         // apply always returns a result or throws an exception:
         return f.apply(result);
      }

      int       sz = f.size();
      /*
      if (sz >= 1)
      {
         if (sz == 1)
         {
            result = f.evaluateAttributes1();
         }
         else
         {
            if (sz == 2)
            {
               result = f.evaluateAttributes2();
            }
            else
            {
               result = f.evaluateAttributes();
            }
         }
      }     */
      if ( (result=f.evaluateArgAttr()) != null)
      {
         return result;
      }

      if ((attr & HSymbol.BUILTINFIRST) == HSymbol.BUILTINFIRST)
      {

         // result = functionEval.evaluate(f);
         if ((result = functionEval.evaluate(f)) != null)
         {
            return result;
         }
      }

      if (sz > 0)
      {
         HObject        elem;

         if ((HSymbol.FLAT & attr) != HSymbol.FLAT)
         {
            elem = f.get(0);

            if (elem instanceof HSymbol)
            {
               result = ((HSymbol) elem).getUpRule(f);
            }
            else
            {
               result = elem.head().getUpRule(f);
            }

            if (result != null)
            {
               return result;
            }
         }
         else
         {      // associative
            HArrayList       arr = new HArrayList();

            elem = f.get(0);

            HObject      last;

            if (elem instanceof HSymbol)
            {
               last = elem;

               if (!((HSymbol) last).isEmptyUpRule())
               {
                  arr.add(last);
               }
            }
            else
            {
               last = elem.head();

               if (!((HSymbol) last).isEmptyUpRule())
               {
                  arr.add(last);
               }
            }

            for (int i = 1; i < sz; i++)
            {
               elem = f.get(i);

               if (elem instanceof HSymbol)
               {
                  if (elem == last)
                  {
                     continue;
                  }

                  last = elem;

                  if (((HSymbol) last).isEmptyUpRule())
                  {
                     continue;
                  }

                  arr.add(last);
               }
               else
               {
                  if (elem.head() == last)
                  {
                     continue;
                  }

                  last = elem.head();

                  if (((HSymbol) last).isEmptyUpRule())
                  {
                     continue;
                  }

                  arr.add(last);
               }
            }

            arr.sort();

            if (arr.size() > 0)
            {
               last = arr.get(0);

               // result = ((HSymbol) last).getRuleSet().getRule(f);
               if ((result = ((HSymbol) last).getUpRule(f)) != null)
               {
                  return result;
               }

               for (int i = 1; i < arr.size(); i++)
               {
                  if (!arr.get(i).equals(last))
                  {
                     last = arr.get(i);

                     // result = ((HSymbol) last).getRuleSet().getRule(f);
                     if ((result = ((HSymbol) last).getUpRule(f)) != null)
                     {
                        return result;
                     }
                  }
               }
            }
         }
      }

      // result = getRuleSet().getRule(f);
      if ((result = getDownRule(f)) != null)
      {

         // if (C.traceAttributeFlag) {
         // C.traceOut.append("TRACE::RULE\n");
         // }
         return result;
      }

      if ((attr & HSymbol.BUILTINFIRST) != HSymbol.BUILTINFIRST)
      {
         return functionEval.evaluate(f);
      }

      return null;
   }

   /**
    * add a new standard identifier to the hashtable HT
    * if the object already exists don't replace it in the hashtable
    */
   final public HSymbol addSymbol()
   {
       HSymbol temp = (HSymbol) HSymbol.HT.put(this, this);
       if (temp == null)
       {
         return this;
       }
       HSymbol.HT.put(temp, temp);
       return temp;
 //    return addConstantSymbol();
   /*
     ContextThread ctThread = (ContextThread) Thread.currentThread();
     Context ct = ctThread.getContext();
     synchronized(ct)
     {
       if (! HSymbol.HT.containsKey(this))
       {
         HSymbol temp = (HSymbol) ct.HT.put(this, this);
         if (temp == null)
         {
           return this;
         }
         ct.HT.put(temp, temp);
         return temp;
       }
       return addConstantSymbol();
     }                               */
   }
                       /*
   final public HSymbol addConstantSymbol()
   {
       HSymbol temp = (HSymbol) HSymbol.HT.put(this, this);
       if (temp == null)
       {
         return this;
       }
       HSymbol.HT.put(temp, temp);
       return temp;
   }
                         */
   /**
    * put a new standard identifier to the hashtable HT
    * if the object already exists replace it in the hashtable
    */
   public boolean putSymbol()
   {
      if (HSymbol.HT.containsKey(this))
      {
         HSymbol.HT.put(this, this);
         C.resultOut.append("Warning: Symbol " + toString() 
                            + " replaces old symbol in hashtable\n");
         return false;
      }
      HSymbol.HT.put(this, this);
      return true;
   }

   /**
    * get the associated "up" results
    */
   public boolean isEmptyUpRule()
   {
      return (rSet == null) ? true : rSet.isEmptyUpRule();
   }

   /**
    * get the associated "down" results
    */
   public HObject getDownRule(HObject obj)
   {
      return (rSet == null) ? null : rSet.getDownRule(obj);
   }

   /**
    * get the associated "up" results
    */
   public HObject getUpRule(HObject obj)
   {
      return (rSet == null) ? null : rSet.getUpRule(obj);
   }

   /**
    * Method declaration
    *
    *
    * @see
    */
   public void clear()
   {
      if (rSet != null)
      {
         rSet.clear();
      }
   }

   /**
    * Method declaration
    *
    *
    * @param sym
    *
    * @see
    */
   public void clearSymbolRule(HSymbol sym)
   {
      if (rSet != null)
      {
         rSet.clearSymbolRule(sym);
      }
   }

   /**
    * Method declaration
    *
    *
    * @return
    *
    * @see
    */
   public String getDefinition()
   {
      StringBuffer      strBuf = new StringBuffer();
      int               attr = getAttributes();

      strBuf.append("Attributes( ");

      if ((attr & HSymbol.CONSTANT) == HSymbol.CONSTANT)
      {
         strBuf.append("Constant ");
      }

      if ((attr & HSymbol.FASTCALL) == HSymbol.FASTCALL)
      {
         strBuf.append("FastCall ");
      }

      if ((attr & HSymbol.FLAT) == HSymbol.FLAT)
      {
         strBuf.append("Flat ");
      }

      if ((attr & HSymbol.HOLDALL) == HSymbol.HOLDALL)
      {
         strBuf.append("HoldAll ");
      }

      if ((attr & HSymbol.HOLDFIRST) == HSymbol.HOLDFIRST)
      {
         strBuf.append("HoldFirst ");
      }

      if ((attr & HSymbol.HOLDREST) == HSymbol.HOLDREST)
      {
         strBuf.append("HoldRest ");
      }

      if ((attr & HSymbol.LISTABLE) == HSymbol.LISTABLE)
      {
         strBuf.append("Listable ");
      }

      if ((attr & HSymbol.ONEIDENTITY) == HSymbol.ONEIDENTITY)
      {
         strBuf.append("OneIdentity ");
      }

      if ((attr & HSymbol.ORDERLESS) == HSymbol.ORDERLESS)
      {
         strBuf.append("Orderless ");
      }

      strBuf.append(")\n");

      if (rSet != null)
      {
         return strBuf.toString() + rSet.getDefinition().toString();
      }

      return strBuf.toString();
   }

   /**
    * add a "down"-rule to the associated RuleSet
    */
   public void putDownRule(int rt, HObject lhs, HObject rhs)
   {
      if (rSet == null)
      {
         rSet = new HRuleSet(this);
      }

      rSet.putDownRule(rt, lhs, rhs);
   }

   /**
    * add a "up"-rule to the associated RuleSet
    */
   public void putUpRule(int rt, HObject lhs, HObject rhs)
   {
      if (rSet == null)
      {
         rSet = new HRuleSet(this);
      }

      rSet.putUpRule(rt, lhs, rhs);
   }

   /**
    * Method Declaration.
    * 
    * 
    * @see
    */
   public static void sortFunctions()
   {

      // Sorting.sort( choiceItems, new HLess() );

      String    s1, s2;
      boolean   evaled = false;
      int       j, k, l, r;

      if (maxItems > 1)
      {
         l = 1;
         r = maxItems - 1;
         k = r;

         do
         {
            for (j = r; j >= l; j--)
            {
               s1 = choiceItems[j - 1];
               s2 = choiceItems[j];

               if (s2.compareTo(s1) < 0)
               {
                  evaled = true;
                  choiceItems[j - 1] = s2;
                  choiceItems[j] = s1;
                  k = j;
               }
            }                   // for

            l = k + 1;

            for (j = l; j <= r; j++)
            {
               s1 = choiceItems[j - 1];
               s2 = choiceItems[j];

               if (s2.compareTo(s1) < 0)
               {
                  evaled = true;
                  choiceItems[j - 1] = s2;
                  choiceItems[j] = s1;
                  k = j;
               }
            }                   // for

            r = k - 1;
         }
         while (l <= r);        // do .. while
      }                         // if
   }                            // end sortFunctions

            /*
   public static void main(String arg[]) {

                try {
                  HSymbol sym = new HSymbol("Zz09$");

                  byte b[] = {};
                  System.in.read(b);
                } catch(Exception e) {
                System.out.println("err: "+e.toString());}

   }    */
}

/**
 * Class Declaration.
 * 
 * 
 * @see
 * 
 * @author
 * @version   %I%, %G%
 */
final class HMatches //implements BinaryPredicate
{

   /**
    * Tests whether or not the hashCodes of the arguments are equal
    * 
    * @param x   First argument
    * @param y   Second argument
    * @return    true if the hashCodes are equal, false otherwise
    */
   public boolean execute(Object x, Object y)
   {
      return x.hashCode() == y.hashCode();
   }

}