/*--- formatted by Jindent 2.0b2, (www.c-lab.de/~jindent) ---*/
package com.hartmath.expression;

import com.hartmath.lib.*;
import com.hartmath.mapping.*;

import java.util.*;
import java.lang.*;
import java.io.*;
import java.text.MessageFormat;
/**
 * Vector wrapper for a HFunction-vector representation
 *
 * @author Klaus Hartlage  <A HREF="mailto:khartlage@t-online.de">khartlage@t-online.de</A>
 */
public class HVector
{
	HFunction vector;

  /**
	 * Constructs a new HVector
	 */
	public HVector(HFunction f) {
    vector = f;
  }

  final public Object clone()
  {
    HVector v;
    try {
      v = (HVector) super.clone();
      v.vector = (HFunction) vector.clone();
    } catch (CloneNotSupportedException e) {
			throw new Error();
		}
    return v;
  }

  public int size()
  {
    return vector.size();
  }
  

  /**
   *  get the internal representation of this polynomial
   */
  final public HFunction getFunction()
  {
    return vector;
  }
   public HObject get(int indx)
   {
      return vector.get(indx);
   }

   public HObject set(int indx, HObject f)
   {
      HObject old = vector.a[indx];
		  vector.a[indx] = f;
		  return old;
   }

   public double[] toDoubleArray()
   {
      HObject   temp;
      double    arr[] = new double[vector.len];

      for (int i = 0; i < vector.len; i++)
      {
         temp = C.NEV( vector.a[i]);

         if (temp instanceof HDouble)
         {
            arr[i] = ((HDouble) temp).value();
         }
         else
         {
            throw new HThrowException(C.Error, C.List, temp);
                              //        new HString("only numerical values allowed."));
         }
      }

      return arr;
   }

   /**
    * Method declaration
    * 
    * 
    * @return
    * 
    * @see
    */
   public HDoubleComplex[] toDoubleComplexArray()
   {
      HObject           temp;
      HDoubleComplex    arr[] = new HDoubleComplex[vector.len];

      for (int i = 0; i < vector.len; i++)
      {
         temp = C.NEV(vector.a[i]);

         if (temp instanceof HDouble)
         {
            arr[i] = new HDoubleComplex((HDouble) temp);
         }
         else
         {
            if (temp instanceof HDoubleComplex)
            {
               arr[i] = (HDoubleComplex) temp;
            }
            else
            {
               throw new HThrowException(C.Error, C.List, 
                                         new HString("only numerical values allowed."));
            }
         }
      }

      return arr;
   }

}



