/*--- formatted by Jindent 2.0b2, (www.c-lab.de/~jindent) ---*/
package com.hartmath.lib;

import com.hartmath.expression.*;
import com.hartmath.util.*;


import java.lang.*;
import java.io.*;
import java.util.Enumeration;

/**
 * The IOMapper class implements the persistance of objects in the HObject class hierarchy
 * 
 * @author Klaus Hartlage  <A HREF="mailto:khartlage@t-online.de">khartlage@t-online.de</A>
 */
public class IOMapper {

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param in
	 *
	 * @return
	 *
	 * @exception IOException
	 *
	 * @see
	 */
	static public HObject map(DataInputStream in) throws IOException {
		HObject		obj = null;
		String		str = null;
		HSymbol		sym = null;
		HSymbol		hc = null;
		HFunction	f = null;
//		HInteger		hint = null;
		Integer		num = null;
		byte[]		barr;
		byte[]		dbarr;
		double		realValue, imagValue;
		int			i;
		byte			hier = in.readByte();

    /*
       static public final byte DOUBLEFILEID        = 2;
       static public final byte DOUBLECOMPLEXFILEID = 3;
       static public final byte INTEGERFILEID       = 4;
       static public final byte FRACTIONFILEID      = 5;
       static public final byte COMPLEXFILEID       = 6;
       static public final byte STRINGFILEID        = 16;
       static public final byte SYMBOLFILEID        = 32;
       static public final byte FUNCTIONFILEID      = 64;
       static public final byte PATTERNFILEID       = 65;
     */
		if (hier < 0) {
			hier *= -1;

			if (hier < 73) {		// predefined values
				obj = C.VarTable[hier-1];
      //  C.resultOut.append(obj.toString()+",");
			} else {
				throw new IOException();
			}
		} else {
    //  C.hierStat[hier]++;
			switch (hier) {

			case 0: 
				obj = null;

				break;

			case HObject.DOUBLEFILEID: 
				return new HDouble(in.readDouble());

			case HObject.DOUBLECOMPLEXFILEID:
				realValue = in.readDouble();
				imagValue = in.readDouble();

				return new HDoubleComplex(realValue, imagValue);

			case HObject.INTEGERFILEID: 
				i = in.readInt();
				barr = new byte[i];

				in.readFully(barr, 0, i);

				// biggi = new BigInteger( barr );

				return new HInteger(barr);

			case HObject.FRACTIONFILEID: 
				i = in.readInt();
				barr = new byte[i];

				in.readFully(barr, 0, i);

				i = in.readInt();
				dbarr = new byte[i];

				in.readFully(dbarr, 0, i);

				return new HFraction(barr, dbarr);

			case HObject.COMPLEXFILEID: 
				HFraction	re = (HFraction) map(in);
				HFraction	im = (HFraction) map(in);

				return new HComplex(re, im);

			case HObject.STRINGFILEID:
				return new HString(in.readUTF());

			case 20:					// long Integer
				return HInteger.valueOf(in.readLong());

			case 21:					// int Integer
				return HInteger.valueOf(in.readInt());

			case 22:					// short Integer
				return HInteger.valueOf(in.readShort());
      case 23:					// byte Integer
				return HInteger.valueOf(in.readByte());

      case 24:					// long Fraction
				return HFraction.valueOf(in.readLong(), in.readLong());
			case 25:					// int Fraction
				return HFraction.valueOf(in.readInt(), in.readInt());

			case 26:					// short Fraction
				return HFraction.valueOf(in.readShort(), in.readShort());
      case 27:					// byte Fraction
				return HFraction.valueOf(in.readByte(),in.readByte());
			case HObject.SYMBOLFILEID: 					// HSymbol [the first time]
				str = in.readUTF();

				int	attr = in.readInt();

				sym = new HSymbol(str, attr);

				HSymbol  res = sym.addSymbol();

				if (res != null) {
					obj = res;
				} else {
					obj = sym;
				}

				num = new Integer(HSymbol.IOCounter);

				HSymbol.IOIn.put(num, obj);

				HSymbol.IOCounter++;

				return obj;

			case 33:					// HSymbol from table
				i = in.readShort();
				num = new Integer(i);
				sym = (HSymbol) HSymbol.IOIn.get(num);
      //  C.resultOut.append(sym.toString()+",");
				if (sym != null) {
					return sym;
				} else {
					throw new IOException();
				}
      case 34:
        hier = in.readByte();
				sym = C.IOConstantTable[hier];
        if (sym != null) {
					return sym;
				} else {
					throw new IOException();
				}
      case 35:
        hier = in.readByte();
				sym = C.IOSymbolTable[hier];
        if (sym != null) {
					return sym;
				} else {
					throw new IOException();
				}
			case 58:					// HFunction 5 arguments
				f = new HFunction((HSymbol) IOMapper.map(in), 5);

				for (int j = 0; j < 5; j++) {
					f.set(j, IOMapper.map(in));
				}

				return f;

			case 59:					// HFunction 4 arguments
				f = new HFunction((HSymbol) IOMapper.map(in), 4);

				for (int j = 0; j < 4; j++) {
					f.set(j, IOMapper.map(in));
				}

				return f;

			case 60:					// HFunction 3 arguments
				f = new HFunction((HSymbol) IOMapper.map(in), 3);

				for (int j = 0; j < 3; j++) {
					f.set(j, IOMapper.map(in));
				}

				return f;

			case 61:					// HFunction 2 arguments
				f = new HFunction((HSymbol) IOMapper.map(in), 2);

				f.set(0, IOMapper.map(in));
				f.set(1, IOMapper.map(in));

				return f;

			case 62:					// HFunction 1 argument
				f = new HFunction((HSymbol) IOMapper.map(in), 1);

				f.set(0, IOMapper.map(in));

				return f;

			case 63:					// HFunction no arguments
				f = new HFunction((HSymbol) IOMapper.map(in));

				return f;

			case HObject.FUNCTIONFILEID: 					// HFunction
				i = in.readInt();
				f = new HFunction((HSymbol) IOMapper.map(in), i);

				for (int j = 0; j < i; j++) {
					f.set(j, IOMapper.map(in));
				}

				return f;

			case HObject.PATTERNFILEID: 					// HPattern
				sym = (HSymbol) map(in);
				hc = (HSymbol) map(in);

				return new HPattern(sym, hc);

			case 66:					// HPattern with no header check
				sym = (HSymbol) map(in);

				return new HPattern(sym);

			case 80:					// HFunction no arguments
				hier = in.readByte();
				sym = C.IOSymbolTable[hier];
				f = new HFunction(sym);

				return f;

			case 81:					// HFunction 1 argument
				hier = in.readByte();
				sym = C.IOSymbolTable[hier];
				f = new HFunction(sym, 1);

				f.set(0, IOMapper.map(in));

				return f;

			case 82:					// HFunction 2 arguments
				hier = in.readByte();
				sym = C.IOSymbolTable[hier];
				f = new HFunction(sym, 2);

				f.set(0, IOMapper.map(in));
				f.set(1, IOMapper.map(in));

				return f;

			case 83:					// HFunction 3 arguments
				hier = in.readByte();
				sym = C.IOSymbolTable[hier];
				f = new HFunction(sym, 3);

				for (int j = 0; j < 3; j++) {
					f.set(j, IOMapper.map(in));
				}

				return f;

			case 84:					// HFunction 4 arguments
				hier = in.readByte();
				sym = C.IOSymbolTable[hier];
				f = new HFunction(sym, 4);

				for (int j = 0; j < 4; j++) {
					f.set(j, IOMapper.map(in));
				}

				return f;

			case 85:					// HFunction 5 arguments
				hier = in.readByte();
				sym = C.IOSymbolTable[hier];
				f = new HFunction(sym, 5);

				for (int j = 0; j < 5; j++) {
					f.set(j, IOMapper.map(in));
				}

				return f;
			}

		// if (obj != null) {
		// obj.readFrom( in );
		// }

		}

		// C.traceOut.append(new Integer(hier).toString() );
		// if (obj != null) {
		// C.traceOut.appendLine( obj.toString() );
		// }

		return obj;
	}

}



/*--- formatting done in "Sun Java Convention" style on 06-18-1999 ---*/
