package com.hartmath.patternmatching;

import com.hartmath.expression.*;
import com.hartmath.lib.*;

import java.util.Stack;

import java.util.Enumeration;
import java.io.*;
//import java.io.Serializable;

import java.lang.*;

class PatPermutation {
  HArrayList x;
  int[] p;
  int[] d;
  int n;
  boolean end;

  public PatPermutation(HArrayList arr) {
    x =  (HArrayList) arr.clone();
    n = x.size();
    p = new int[n+1];
    d = new int[n+1];
    for (int j=0;j<n;j++) {
      p[j]=0;
      d[j]=1;
    }
    end = false;
  }

  public boolean hasNext() {
    return (! end);
  }

  public void nextPerm() {
     int k=0;
     int m=1;
     int sig;
     HObject h;
     if (n<=1) {
       end = true;
       return;
     }
     sig=p[m]+d[m];
     p[m]=sig;
     while ( (sig==n-m+1) || (sig==0))
     {
       if(sig==0)
       {
         d[m]=1;
         k++;
       }
       else
       {
         d[m]=(-1);
       }
       if (m==(n-1)) {
         h=x.get(n-2);
         x.set(n-2,x.get(n-1));
         x.set(n-1,h);
         end = true;
         sig=1;
       }
       else
       {
         m++;
         sig=p[m]+d[m];
         p[m]=sig;
       }
     }
     if (! end)
     {
       h=x.get(k+sig-1);
       x.set(k+sig-1, x.get(k+sig));
       x.set(k+sig, h);
     }
   }

}

/**
 * PatternMatching is a class for matching the left-hand-side of a rule with a given expression
 * 
 * @author Klaus Hartlage  <A HREF="mailto:khartlage@t-online.de">khartlage@t-online.de</A>
 */
public class PatternMatching  {
	HObject  pm_lhs;
	HObject  rhs;
	HObject  result;


	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param stck
	 * @param lhs
	 * @param expr
	 *
	 * @return
	 *
	 * @see
	 */
	private boolean kSubsets(Stack stck, HFunction lhs, HFunction expr) {
		Stack tempStck;
		HArrayList arr = new HArrayList();

		lhs.getPattern(arr);

		boolean		isOrderless = 
			((HSymbol.ORDERLESS & lhs.head().getAttributes()) 
			 == HSymbol.ORDERLESS);

		// expr = [a,b,c,d,e] lhs = [x_,y_,z_]

		final int	n = expr.size() - 1;		// 4
		final int	k = lhs.size() - 1;		// 2
//		final int	stop = n - k + 1;			// 3
		int			x[] = new int[k + 2];

		x[k + 1] = n + 1;

		for (int i = 0; i < k + 1; i++) {
			x[i] = i;
		}

		// x = [0,{1,2},5]

		int	i;

		do {
			try {

				// special test for Orderless to avoid multiple testing:
				// hier steckt noch der Wurm drin ?!?:

				if (isOrderless) {
					for (int l = 0; l < lhs.size(); l++) {
						for (int m = x[l]; m < x[l + 1] - 1; m++) {
							if (expr.get(m).less(expr.get(m + 1))) {
								throw new MatchingFailed();
							}
						}
					}
				}

				// check the given kSubset:
				// ************************

				tempStck = (Stack) stck.clone();

				for (int l = 0; l < lhs.size(); l++) {
					HObject  subExpr = new HFunction(lhs.head());

					for (int m = x[l]; m < x[l + 1]; m++) {
						((HFunction) subExpr).add(expr.get(m));
					}

					if ((((HFunction) subExpr).size() == 1) 
							  && ((HSymbol.ONEIDENTITY & ((HFunction) subExpr).head().getAttributes()) 
									== HSymbol.ONEIDENTITY)) {

						// oneidentity

						subExpr = ((HFunction) subExpr).get(0);
					}

					// check the next kSubset !
          /*
					if (C.DEBUG) {
						HObject  pval;

						if (C.traceFlag && C.traceExtendedFlag) {
							C.traceOut.append("TRACE(try subset)::");
							C.traceOut.append(subExpr.toString());
							C.traceOut.append(" <-> ");
							C.traceOut.append(lhs.get(l).toString());
							C.traceOut.append(" {");

							for (int trc = 0; trc < arr.size(); trc++) {
								C.traceOut.append(arr.get(trc).toString());
								C.traceOut.append("->");

								pval = ((HPattern) arr.get(trc)).getValue();

								if (pval == null) {
									C.traceOut.append("<null>");
								} else {
									C.traceOut.append(pval.toString());
								}
							}

							C.traceOut.append("}\n");
						}
					} */
					if (lhs.get(l) instanceof HFunction) {
						if (! patternMatches(((HFunction) lhs.get(l)), subExpr,
																				  tempStck) ) {
							throw new MatchingFailed();
						}
					} else {
						if (!((HObject) lhs.get(l)).matches(subExpr)) {
							throw new MatchingFailed();
						}
					}   /*
					if (C.DEBUG) {
						HObject  pval;

						if (C.traceFlag && C.traceExtendedFlag) {
							C.traceOut.append("TRACE(try subset)::");
							C.traceOut.append("<TRUE>\n");
						}
					}     */
				}

				return matchStack(tempStck);
			} catch (MatchingFailed mf) {}

			// PatternMatching.clearPattern(arr);

			HFunction.clearPattern(arr);

			// ************************
			// create next kSubset

			i = k;

			while (x[i] == n - k + i) {
				i--;
			}

			// check if the last kSubset is reached

			if (i == 0) {
				throw new MatchingFailed();
			}

			x[i] = x[i] + 1;

			for (int j = i + 1; j <= k; j++) {
				x[j] = x[j - 1] + 1;
			}
		} while (true);
	}

	/**
	 * Class Constructor.
	 * 
	 * 
	 * @see
	 */
	public PatternMatching() {
		pm_lhs = null;
		rhs = null;
		result = null;
  }

	/**
	 * Class Constructor.
	 * 
	 * 
	 * @param p_lhs
	 * @param p_rhs
	 *
	 * @see
	 */
	public PatternMatching(HObject p_lhs, HObject p_rhs) {
		pm_lhs = p_lhs;
		rhs = p_rhs;
		result = null;
	}

  public boolean tryStack(Stack stck) {
    try {
      return matchStack(stck);
		} catch (MatchingFailed mf) {
			return false;
    }
  }
  
	/**
	 * Method Declaration.
	 * 
	 * 
	 * @param stck
	 *
	 * @return
	 *
	 * @see
	 */
	public boolean matchStack(Stack stck) {

		// f(x_+y_,y_+z_):=Check(Print(x,";",y,";",z,"\n"))

		if (stck.isEmpty()) {

			// evaluate rhs

			if (rhs != null) {
				result = rhs.substitutePattern();
        try {
          result = C.EV(result);
        }
        catch (HReturnException ex) {
          result = ex.getValue();
			  }
				HObject  temp = C.RHS__.getValue();

				if (temp != null) {
					result = new HFunction(pm_lhs.head(), temp, result);
          result = C.EV(result);
				}    /*
        if (C.DEBUG) {
          if (C.traceFlag && C.traceExtendedFlag) {
						C.traceOut.append("TRACE(eval match result)::");
            if (result instanceof HFunction) {
              C.traceOut.append(((HFunction)result).toFullForm());
            }
            else {
						  C.traceOut.append(result.toString());
            }
						C.traceOut.append("\n");
          }
        }      */
		 //		result = C.EV(result);
			} else {
				result = null;
			}

			return true;
		}

		HPair			pr = (HPair) stck.pop();
		HFunction	lhs = (HFunction) ((HFunction) pr.car).clone();
		HFunction	expr = (HFunction) ((HFunction) pr.cdr).clone();

    if (lhs.size() > expr.size()) {
      return false;
    }
    if ( lhs.head().unequals(expr.head()) ) {
      return false;
    }
    else {
      if (lhs.size()==0 && expr.size()==0) {
        return matchStack(stck);
      }
    }
		if ((lhs.size() < expr.size())
				  && ((HSymbol.FLATORDERLESS & lhs.head().getAttributes())
						== HSymbol.FLATORDERLESS)) {
      //Array jexpr = expr.toJGLArray();
			//do {
      PatPermutation perm = new PatPermutation(expr);
      while (perm.hasNext()) {
        perm.nextPerm();
				try {   /*
					if (C.traceFlag && C.traceExtendedFlag) {
						C.traceOut.append("TRACE(try permutation)::");
						C.traceOut.append(expr.toString());
						C.traceOut.append(" <-> ");
						C.traceOut.append(lhs.toString());
						C.traceOut.append("\n");
					}       */

					//return kSubsets(stck, lhs, new HFunction(expr.head(),jexpr));
          return kSubsets(stck, lhs, expr);
				} catch (MatchingFailed mf) {

				// check the next permutation!

				}
			//} while (Permuting.nextPermutation(jexpr, new HLessThan()));
      }
			throw new MatchingFailed();
		}
		if ((HSymbol.ORDERLESS & lhs.head().getAttributes())
				  == HSymbol.ORDERLESS) {
			Stack tempStck;
			HArrayList arr = new HArrayList();

			lhs.getPattern(arr);

			// HArrayList arr = PatternMatching.getPattern(lhs);
			// permutation over all elements
      PatPermutation perm = new PatPermutation(expr);
      while (perm.hasNext()) {
        perm.nextPerm();
      //Array jexpr = expr.toJGLArray();
			//do {
				try {
					tempStck = (Stack) stck.clone();

					for (int i = 0; i < lhs.size(); i++) {
						if (lhs.get(i) instanceof HFunction) {
			 //				if (! patternMatches(((HFunction) lhs.get(i)), (HObject) jexpr.get(i),
              if (! patternMatches(((HFunction) lhs.get(i)), (HObject) expr.get(i),
																					  tempStck) ) {
								throw new MatchingFailed();
							}
						} else {
			//				if (!((HObject) lhs.get(i)).matches((HObject) jexpr.get(i))) {
              if (!((HObject) lhs.get(i)).matches((HObject) expr.get(i))) {
								throw new MatchingFailed();
							}
						}
					}

					return matchStack(tempStck);
				} catch (MatchingFailed mf) {

				// check the next permutation!

				}

				HFunction.clearPattern(arr);

			// PatternMatching.clearPattern(arr);
      }
			//} while (Permuting.nextPermutation(jexpr, new HLessThan()));

			throw new MatchingFailed();
		}
		if ((HSymbol.FLAT & lhs.head().getAttributes()) == HSymbol.FLAT) {
			return kSubsets(stck, lhs, expr);
		}

		throw new Error("matchStack: no attribute defined");
	}

  // ???

  /**
	 * Method Declaration.
	 *
	 *
	 * @param newlhs
	 * @param newexpr
	 *
	 * @return
	 *
	 * @see
	 */
	protected boolean reduceEqualOrderless(HFunction lhs, HFunction newlhs, HFunction newexpr) {
		boolean  flag;
		HObject  tmpI;
    for (int i = 0; i < lhs.size(); i++) {

      tmpI = lhs.get(i);

			if (tmpI instanceof HFunction
					  && (((HFunction) tmpI).getProperties() != HFunction.MATCHEQUAL)) {
        newlhs.add(tmpI);
				continue;
			}
			if (tmpI instanceof HPattern) {
				newlhs.add(tmpI);
				continue;
			}

			flag = true;

			for (int j = 0; j < newexpr.size(); j++) {
				if (tmpI.equals(newexpr.get(j))) {
					newexpr.remove(j);

					flag = false;

					break;
				}
			}

			if (flag) {
				return false;
			}

    }
		return true;
	}
	/**
	 * Method Declaration.
	 *
	 *
	 * @param newlhs
	 * @param newexpr
	 *
	 * @return
	 *
	 * @see
	 */
	protected boolean reduceOrderless(HFunction newlhs, HFunction newexpr, Stack stck) {
		boolean  flag;
		HObject  tmpI;
    HObject  tmpJ;
    HArrayList arr = new HArrayList();
		for (int i = 0; i < newlhs.size(); i++) {
			tmpI = newlhs.get(i);
			if (tmpI instanceof HFunction &&
					(((HFunction) tmpI).getProperties() != HFunction.MATCHSEQUENCE) ) {
				continue;
			}
			if (tmpI instanceof HPattern) {
				continue;
			}
			flag = true;
      ((HFunction)tmpI).getPattern(arr);
      for (int j = 0; j < newexpr.size(); j++) {
				if ( tmpI.matches(newexpr.get(j)) ) {
          newlhs.remove(i);
          tmpJ = (HObject) newexpr.remove(j);
          if ( reduceOrderless(newlhs, newexpr, stck) ) {
            return true;
          }
          else {
            newlhs.add(i, tmpI);       // ???
            newexpr.add(j, tmpJ);
          }
        }
        HFunction.clearPattern(arr);
      }
      return false;
		}
    if (newlhs.size() < newexpr.size()) {
			newlhs.add(new HFunction(newlhs.head(), C.RHS__));
  	}
    Stack tempStck = (Stack) stck.clone();
  	tempStck.push(new HPair(newlhs, newexpr));
    try {
		  return matchStack(tempStck);
		} catch (MatchingFailed mf) {
			return false;
    }
	}

  /**
	 * Matches all simple patterns of the function
	 * and return "difficult" pattern-matching Pairs on the Stack
	 */
	public boolean patternMatches(HFunction lhs, HObject obj, Stack stck) {
    int prop = lhs.getProperties();
    HSymbol lhsSym = lhs.head();
    /*
		if (C.DEBUG) {
			if (C.traceFlag && C.traceExtendedFlag) {
				C.traceOut.append("TRACE(try patternMatches)::");
				C.traceOut.append(lhs.toString());
				C.traceOut.append(" {");
				C.traceOut.append(new Integer(prop).toString());
				C.traceOut.append("} <-> ");
				C.traceOut.append(obj.toString());
				C.traceOut.append("}\n");
			}
		} */
		if ((lhs.size() == 1)
				  && ((HSymbol.ONEIDENTITY & lhsSym.getAttributes())
						== HSymbol.ONEIDENTITY)) {
			if ( lhs.get(0).matches(obj)) {
				return tryStack( stck );
			}
		}

    if ((prop & HFunction.MATCHEQUAL) == HFunction.MATCHEQUAL) {
				if ( lhs.equals(obj) ) {
          return tryStack( stck );
        }
        return false;
    }
		if ((prop & HFunction.MATCHSEQUENCE) == HFunction.MATCHSEQUENCE) {
				if ( lhs.matches(obj) ) {
          return tryStack( stck );
        }
        return false;
    }


		if (obj instanceof HFunction) {
			HFunction	temp = (HFunction) obj;

			if (lhsSym.unequals(temp.head())) {
				return false;
			}
			if ((lhs.size() < temp.size()) && (lhs.size() > 0)) {
				if ((HSymbol.FLAT & temp.head().getAttributes())
						  == HSymbol.FLAT) {
          Stack tempStck = (Stack) stck.clone();
          tempStck.push(new HPair(lhs, temp));
          return tryStack( tempStck );
				} else {
					return false;
				}
			}
			if (lhs.size() != temp.size()) {
				return false;
			}
			if ((HSymbol.ORDERLESS & temp.head().getAttributes()) 
					  == HSymbol.ORDERLESS) {
        Stack tempStck = (Stack) stck.clone();
        tempStck.push(new HPair(lhs, temp));
        return tryStack( tempStck );
			}

			for (int i = 0; i < lhs.size(); i++) {
      /*
				if (lhs.get(i) instanceof HFunction) {
					if (!patternMatches(((HFunction) lhs.get(i)), (HObject) temp.get(i),
																		 stck)) {
						return false;
					}
				} else {
					if (! lhs.get(i).matches((HObject) temp.get(i)) ) {
						return false;
					}
				}         */
        if ( lhs.get(i) instanceof HFunction) {
          if ( temp.get(i) instanceof HFunction &&
               ((HSymbol.FLATORDERLESS & lhs.get(i).head().getAttributes())
                   != HSymbol.NOATTRIBUTE) ) {
            stck.push(new HPair(lhs.get(i), temp.get(i)));
          }
          else {
            if (! patternMatching(((HFunction)lhs.get(i)), (HObject) temp.get(i)) ) {
		   				return false;
		  			}
          }
				} else {
					if (!((HObject)lhs.get(i)).matches((HObject) temp.get(i))) {
						return false;
					}
				}
			}
      return tryStack( stck );
		}

		return false;
	}

	/**
	 * Matches all simple patterns of the function
	 * and return "difficult" pattern-matching Pairs on the Stack
	 */
	public boolean patternMatching(HFunction lhs, HObject obj) {
    /*
		if (C.DEBUG) {
			if (C.traceFlag && C.traceExtendedFlag) {
				C.traceOut.append("TRACE(try patternMatches)::");
				C.traceOut.append(lhs.toString());
				C.traceOut.append(" <-> ");
				C.traceOut.append(obj.toString());
				C.traceOut.append("}\n");
			}
		} */
    HSymbol lhsSym = lhs.head();
		if ((lhs.size() == 1)
				  && ((HSymbol.ONEIDENTITY & lhsSym.getAttributes())
						== HSymbol.ONEIDENTITY)) {
			return lhs.get(0).matches(obj);
		}
		if (obj instanceof HFunction) {
			HFunction	temp = (HFunction) obj;
			if (lhsSym.unequals(temp.head())) {
				return false;
			}
			if ((lhs.size() < temp.size()) && (lhs.size() > 0)) {
				if ((HSymbol.FLATORDERLESS & temp.head().getAttributes()) 
						  == HSymbol.FLATORDERLESS) {
					HFunction	newlhs = new HFunction( lhsSym );
					HFunction	newexpr = (HFunction) temp.clone();
          if (! reduceEqualOrderless(lhs, newlhs, newexpr)) {
            return false;
          }
          Stack stck = new Stack();
					if (reduceOrderless(newlhs, newexpr, stck)) {
						return true;
					}
				} else {
					if ((HSymbol.FLAT & temp.head().getAttributes())
							  == HSymbol.FLAT) {
            Stack stck = new Stack();
						stck.push(new HPair(lhs, temp));
            return tryStack( stck );
					}
				}

				return false;
			}
			if (lhs.size() != temp.size()) {
				return false;
			}
			if ((HSymbol.ORDERLESS & temp.head().getAttributes())
					  == HSymbol.ORDERLESS) {
				HFunction	newlhs = new HFunction( lhsSym );
				HFunction	newexpr = (HFunction) temp.clone();
        if (! reduceEqualOrderless(lhs, newlhs, newexpr)) {
          return false;
        }
        Stack stck = new Stack();
				if (reduceOrderless(newlhs, newexpr, stck)) {
					return true;
				}
				return false;
			}
      int prop = lhs.getProperties();
			if ((prop & HFunction.MATCHEQUAL) == HFunction.MATCHEQUAL) {
				if ( lhs.equals(obj) ) {
          Stack stck = new Stack();
          return tryStack( stck );
        }
        return false;
			}
			if ((prop & HFunction.MATCHSEQUENCE) == HFunction.MATCHSEQUENCE) {
				if ( lhs.matches(obj) ) {
          Stack stck = new Stack();
          return tryStack( stck );
        }
        return false;
			}
      Stack stck = new Stack();
			for (int i = 0; i < lhs.size(); i++) {
				if ( lhs.get(i) instanceof HFunction) {
          if ( temp.get(i) instanceof HFunction &&
               ((HSymbol.FLATORDERLESS & lhs.get(i).head().getAttributes())
                   != HSymbol.NOATTRIBUTE) ) {
            stck.push(new HPair(lhs.get(i), temp.get(i)));
          }
          else {
            if (! patternMatching(((HFunction)lhs.get(i)), (HObject) temp.get(i)) ) {
		   				return false;
		  			}
          }
				} else {
					if (!((HObject)lhs.get(i)).matches((HObject) temp.get(i))) {
						return false;
					}
				}
			}
      return tryStack( stck );
		}

		return false;
	}
  
	/**
	 * try to match a left-hand-side with a given expression
	 */
	public boolean execute(Object lhs, Object expr) {
		if (lhs instanceof HFunction && expr instanceof HObject) {
      return patternMatching(((HFunction) lhs), ((HObject) expr) );
	 	//	Stack stck = new Stack();
	  // 	if (patternMatches(((HFunction) lhs), ((HObject) expr), stck) ) {
    //    return tryStack( stck );
	 	//	}
		}

		return false;
	}

	/**
	 * Method Declaration.
	 * 
	 * 
	 * @return
	 *
	 * @see
	 */
	public HObject getResult() {
		return result;
	}

	//{{DECLARE_CONTROLS
	//}}
}



/*--- formatting done in "Sun Java Convention" style on 06-18-1999 ---*/
